using AweCoreDemo.Data;
using AweCoreDemo.Models;
using Microsoft.AspNetCore.Mvc;
using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Awesome.MultiLookup
{
    public class CategoriesMultiLookupController : Controller
    {
        public IActionResult GetItems(int[] v)
        {
            var items = new List<Category>();

            if (v != null)
            {
                items = Db.Categories
                    .Where(o => v.Contains(o.Id)).ToList();
            }

            return Json(items.Select(o => new KeyContent(o.Id, o.Name)));
        }

        public IActionResult Search(int[] selected, int page, string search = "")
        {
            selected = selected ?? new int[] { };

            var query = Db.Categories.Where(o => o.Name.Contains(search) && !selected.Contains(o.Id)).AsQueryable();

            var res = AweUtil.BuildList(query, page, keyContentFunc: o => new KeyContent(o.Id, o.Name));

            return Json(res);
        }

        public IActionResult Selected(int[] selected)
        {
            var items = new List<Category>();

            if (selected != null)
            {
                items = Db.Categories
                    .Where(o => selected.Contains(o.Id)).ToList();
            }

            return Json(new AjaxListResult
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name))
            });
        }
    }
}