using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.Utils.Awesome;
using Microsoft.AspNetCore.Mvc;

using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Awesome.MultiLookup
{
    public class MealsCustomItemMultiLookupController : Controller
    {
        public IActionResult SearchForm()
        {
            return PartialView();
        }

        public IActionResult GetItems(int[] v)
        {
            var query = Db.Meals.AsQueryable();

            if (v != null)
            {
                query = query.Where(o => v.Contains(o.Id));
            }

            ViewData["caption"] = true;
            var res = AweUtil.BuildList(
                query,
                setContentFunc: (items) => this.RenderPartialView("items", items));

            return Json(res);

        }

        public IActionResult Search(int[] selected, int page, string search)
        {
            selected = selected ?? new int[] { };

            var query = Db.Meals
                .Where(o => !selected.Contains(o.Id))
                .AsQueryable();

            query = query.Filter(search, o => o.Name);

            var res = AweUtil.BuildList<Meal>(
                query,
                page,
                setContentFunc: (items) => this.RenderPartialView("items", items));

            return Json(res);
        }

        public IActionResult Selected(IEnumerable<int> selected)
        {
            var query = Db.Meals
                .AsQueryable();

            if (selected != null)
            {
                query = query.Where(o => selected.Contains(o.Id));
            }

            var res = AweUtil.BuildList(
                query,
                setContentFunc: (items) => this.RenderPartialView("items", items));

            return Json(res);
        }

        // used by the details button, in items view
        public IActionResult Details(int id)
        {
            return PartialView(Db.Find<Meal>(id));
        }
    }
}