using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.Utils.Awesome;
using Microsoft.AspNetCore.Mvc;

using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Awesome.MultiLookup
{
    public class MealsMultiLookupController : Controller
    {
        public IActionResult GetItems(int[] v)
        {
            var items = new List<Meal>();

            if (v != null)
            {
                items = Db.Meals
                    .Where(o => v.Contains(o.Id)).ToList();
            }

            return Json(items.Select(meal => new KeyContent(meal.Id, meal.Name)));
        }

        public IActionResult Search(int[] selected, int page, string search)
        {
            selected = selected ?? new int[] { };

            var query = Db.Meals.AsQueryable().Filter(search, o => o.Name).Where(o => !selected.Contains(o.Id));

            var res = AweUtil.BuildList(query, page, keyContentFunc: o => new KeyContent(o.Id, o.Name));

            return Json(res);
        }

        public IActionResult Selected(int[] selected)
        {
            var items = new List<Meal>();

            if (selected != null)
            {
                items = Db.Meals
                    .Where(o => selected.Contains(o.Id)).ToList();
            }

            return Json(new AjaxListResult
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name))
            });
        }
    }
}