using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Demos.Grid
{
    public class GridInlineBatchEditingController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Dinner o)
        {
            return new
            {
                o.Id,
                o.Name,
                Date = o.Date.ToShortDateString(),
                ChefName = o.Chef.FullName,
                Meals = string.Join(", ", o.Meals.Select(m => m.Name)),
                BonusMeal = o.BonusMeal.Name,
                o.Organic,
                DispOrganic = o.Organic ? "Yes" : "No",

                // below properties used for inline editing only
                MealsIds = o.Meals.Select(m => m.Id.ToString()).ToArray(), // value for meals multiselect
                ChefId = o.Chef.Id, // value for chef dropdown
                BonusMealId = o.BonusMeal.Id // value for bonus meal dropdown
            };
        }

        public IActionResult GridGetData(GridParams g, string search = "")
        {
            var query = Db.Dinners.Where(o => o.Name.Contains(search)).AsQueryable();

            var model = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel,
            };

            return Json(model.Build());
        }

        [HttpPost]
        public IActionResult BatchSave(DinnerInput[] inputs)
        {
            var res = new List<object>();

            foreach (var input in inputs)
            {
                var vldState = ModelUtil.Validate(input);

                // custom validation example
                if (input.Name != null && input.Name.Contains("aaa"))
                {
                    vldState.Add("Name", "Name can't contain aaa");
                }

                if (vldState.IsValid())
                {
                    try
                    {
                        var isCreate = !input.Id.HasValue;

                        var ent = isCreate ? new Dinner() :
                            Db.Dinners.First(o => o.Id == input.Id);

                        ent.Name = input.Name;
                        ent.Date = input.Date.Value;
                        ent.Chef = Db.Find<Chef>(input.ChefId);

                        // ToList req for EF
                        ent.Meals = Db.Meals
                            .Where(o => input.MealsIds.Contains(o.Id)).ToList();

                        ent.BonusMeal = Db.Find<Meal>(input.BonusMealId);
                        ent.Organic = input.Organic ?? false;

                        if (isCreate)
                        {
                            Db.Add(ent);
                        }

                        res.Add(new { });
                    }
                    catch (Exception ex)
                    {
                        vldState.Add("Name", ex.Message);
                    }
                }

                if (!vldState.IsValid())
                {
                    res.Add(vldState.ToInlineErrors());
                }
            }


            return Json(res);
        }

        public IActionResult Delete(int id)
        {
            var dinner = Db.Find<Dinner>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "dinner",
                Name = dinner.Name
            });
        }

        [HttpPost]
        public IActionResult Delete(DeleteConfirmInput input)
        {
            Db.Remove(Db.Find<Dinner>(input.Id));

            // the PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }
    }
}