using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Demos.Grid.MasterDetailCrud
{
    public class AddressesGridCrudController : Controller
    {
        private object mapToGridModel(RestaurantAddress o)
        {
            return new
            {
                o.Id,
                o.Line1,
                o.Line2,
                ChefName = o.Chef.FullName,

                ChefId = o.Chef.Id // for inline editing, value to the Chef inline dropdown
            };
        }

        public IActionResult GridGetItems(GridParams g, int restaurantId)
        {
            var query = Db.RestaurantAddresses
                .Where(o => o.RestaurantId == restaurantId)
                .AsQueryable();

            var gmb = new GridModelBuilder<RestaurantAddress>(query, g)
            {
                KeyProp = o => o.Id,
                Map = mapToGridModel,
            };

            return Json(gmb.Build());
        }

        public IActionResult Create(int restaurantId)
        {
            return PartialView(new RestaurantAddressInput { RestaurantId = restaurantId });
        }

        [HttpPost]
        public IActionResult Create(RestaurantAddressInput input)
        {
            if (!ModelState.IsValid)
            {
                return PartialView(input);
            }

            var address = new RestaurantAddress
            {
                Line1 = input.Line1,
                Line2 = input.Line2,
                RestaurantId = input.RestaurantId,
                Chef = Db.Find<Chef>(input.ChefId)
            };

            Db.Add(address);



            return Json(mapToGridModel(address));
        }

        public IActionResult Edit(int id)
        {
            var address = Db.RestaurantAddresses
                .First<RestaurantAddress>(o => o.Id == id);

            return PartialView(
                "Create",
                new RestaurantAddressInput
                {
                    Line1 = address.Line1,
                    Line2 = address.Line2,
                    ChefId = address.Chef.Id,
                    RestaurantId = address.RestaurantId
                });
        }

        [HttpPost]
        public IActionResult Edit(RestaurantAddressInput input)
        {
            if (!ModelState.IsValid)
            {
                return PartialView("Create", input);
            }

            var address = Db.Find<RestaurantAddress>(input.Id);
            address.Line1 = input.Line1;
            address.Line2 = input.Line2;
            address.Chef = Db.Find<Chef>(input.ChefId);


            return Json(new { input.Id });
        }

        public IActionResult Delete(int id)
        {
            var address = Db.Find<RestaurantAddress>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "restaurant address",
                Name = address.Line1 + " " + address.Line2
            });
        }

        [HttpPost]
        public IActionResult Delete(DeleteConfirmInput input)
        {
            Db.Remove(Db.Find<RestaurantAddress>(input.Id));



            return Json(new { input.Id });
        }

        #region for inline editing
        [HttpPost]
        public IActionResult CreateInline(RestaurantAddressInput input)
        {
            if (!ModelState.IsValid)
            {
                return Json(ModelState.GetErrorsInline());
            }

            var ent = new RestaurantAddress
            {
                RestaurantId = input.RestaurantId,
                Line1 = input.Line1,
                Line2 = input.Line2,
                Chef = Db.Find<Chef>(input.ChefId)
            };

            Db.Add(ent);


            return Json(new { });
        }

        [HttpPost]
        public IActionResult EditInline(RestaurantAddressInput input)
        {
            if (!ModelState.IsValid)
            {
                return Json(ModelState.GetErrorsInline());
            }

            var ent = Db.Find<RestaurantAddress>(input.Id);
            ent.Line1 = input.Line1;
            ent.Line2 = input.Line2;
            ent.Chef = Db.Find<Chef>(input.ChefId);



            return Json(new { });
        }
        #endregion
    }
}