using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;
using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Demos.Grid.MasterDetailCrud
{
    public class MasterDetailCrudDemoController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        public IActionResult RestaurantGridGetItems(GridParams g)
        {
            var query = Db.Restaurants.Where(o => o.IsCreated).AsQueryable();

            var gmb = new GridModelBuilder<Restaurant>(query, g)
            {
                KeyProp = o => o.Id,
                GetItem = () => Db.Get<Restaurant>(Convert.ToInt32(g.Key))
            };

            return Json(gmb.Build());
        }

        public IActionResult Create()
        {
            // needed so we could add addresses even before the restaurant is created/saved
            var rest = new Restaurant();

            Db.Add(rest);


            return PartialView(new RestaurantInput { Id = rest.Id });
        }

        [HttpPost]
        public IActionResult Create(RestaurantInput input)
        {
            if (!ModelState.IsValid)
            {
                return PartialView(input);
            }

            var restaurant = Db.Find<Restaurant>(input.Id);
            restaurant.Name = input.Name;
            restaurant.IsCreated = true;



            return Json(new { input.Id });
        }

        public IActionResult Edit(int id)
        {
            var rest = Db.Find<Restaurant>(id);
            return PartialView("Create", new RestaurantInput { Id = id, Name = rest.Name });
        }

        [HttpPost]
        public IActionResult Edit(RestaurantInput input)
        {
            if (!ModelState.IsValid)
            {
                return PartialView("Create", input);
            }

            var rest = Db.Find<Restaurant>(input.Id);

            rest.Name = input.Name;

            return Json(new { rest.Id });
        }

        public IActionResult Delete(int id)
        {
            var restaurant = Db.Find<Restaurant>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "restaurant",
                Name = restaurant.Name
            });
        }

        [HttpPost]
        public IActionResult Delete(DeleteConfirmInput input)
        {
            Db.Remove(Db.Find<Restaurant>(input.Id));


            return Json(new { input.Id });
        }
    }
}