using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.Utils.Awesome;
using AweCoreDemo.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;
using Omu.AwesomeMvc;

namespace AweCoreDemo.Controllers.Demos.Misc
{
    public class CrudInLookupController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        [HttpPost]
        public IActionResult Index(LookupCrudDemoInput input)
        {
            return View(input);
        }

        public IActionResult DinnerLookup()
        {
            return PartialView();
        }

        public IActionResult DinnerMultiLookup()
        {
            return PartialView();
        }

        private static object MapToGridModel(Dinner o)
        {
            return
                new
                {
                    o.Id,
                    o.Name,
                    Date = o.Date.ToShortDateString(),
                    ChefName = o.Chef.FullName,
                    Meals = string.Join(", ", o.Meals.Select(m => m.Name))
                };
        }

        public IActionResult DinnersGridSearch(GridParams g, int[] selected, string search)
        {
            selected = selected ?? new int[] { };

            var query = Db.Dinners
                .Where(o => !selected.Contains(o.Id))
                .AsQueryable();

            query = query.Filter(search, o => o.Name);

            var gmb = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel
            };

            return Json(gmb.Build());
        }

        public IActionResult DinnersGridSelected(GridParams g, int[] selected)
        {
            selected = selected ?? new int[] { };

            var query = Db.Dinners
                .Where(o => selected.Contains(o.Id))
                .AsQueryable();

            var gmb = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel
            };

            return Json(gmb.Build());
        }
    }
}