namespace AweCoreDemo.Models
{
    public static class MySiteMap
    {
        private static readonly IList<SiteMapItem> items = new List<SiteMapItem>();

        public static IEnumerable<SiteMapItem> GetAll()
        {
            return items;
        }

        static MySiteMap()
        {
            var grid = new SiteMapItem { Name = "Grid", Keywords = "gridview", Collapsed = false, Controller = "GridDemo", Action = "Index" };
            items.Add(grid);
            //items.Add(new SiteMapItem { Name = "Overview", Controller = "GridDemo", Action = "Index", Parent = grid });
            items.Add(new SiteMapItem { Name = "Custom Format, Action Buttons", Controller = "GridDemo", Action = "CustomFormat", Parent = grid });
            items.Add(new SiteMapItem { Name = "Grouping/Aggregates", Controller = "GridDemo", Action = "Grouping", Parent = grid, Keywords = "sum max count total" });

            var filtering = new SiteMapItem
            {
                Name = "Filtering",
                Parent = grid,
                Keywords = "search filtering query"
            };

            items.Add(filtering);
            items.Add(new SiteMapItem { Name = "FilterRow server data", Controller = "GridFilterRowServerSideData", Action = "Index", Parent = filtering, Keywords = "search filtering query" });
            items.Add(new SiteMapItem { Name = "FilterRow client data", Controller = "GridFilterRowClientData", Action = "Index", Parent = filtering, Keywords = "search filtering query" });
            items.Add(new SiteMapItem
            {
                Name = "FilterRow large data source",
                Controller = "GridFilterRowServerSideData",
                Action = "LargeDataSource",
                Parent = filtering,
                Keywords = "search filtering query large data source remote"
            });

            items.Add(new SiteMapItem { Name = "Save filters", Controller = "GridSaveFilters", Action = "Index", Parent = filtering, Keywords = "persistence save filter row" });
            items.Add(new SiteMapItem { Name = "Misc", Controller = "GridDemo", Action = "Filtering", Parent = filtering, Keywords = "Filter" });


            #region inline editing

            var inlineEdit = new SiteMapItem
            {
                Name = "Grid Inline Editing",
                Controller = "GridInlineEditDemo",
                Action = "Index",
                Keywords = "crud signalr",
                Parent = grid
            };

            items.Add(inlineEdit);

            items.Add(new SiteMapItem { Name = "Inline Batch Editing", Controller = "GridInlineBatchEditing", Action = "Index", Keywords = "editing save crud", Parent = inlineEdit });
            items.Add(new SiteMapItem { Name = "Client Instant Validation Batch Editing", Controller = "GridInlineClientValidation", Action = "Index", Keywords = "ovld editing", Parent = inlineEdit });
            items.Add(new SiteMapItem
            {
                Name = "TreeGrid inline editing",
                Controller = "TreeGridInlineEditing",
                Action = "Index",
                Parent = inlineEdit
            });

            var extraInlineEdit = new SiteMapItem { Name = "More inline editing", Collapsed = true, Parent = inlineEdit };
            items.Add(extraInlineEdit);

            items.Add(new SiteMapItem { Name = "Inline Cascade", Controller = "GridInlineEditParent", Action = "Index", Keywords = "parent inline edit grid", Parent = extraInlineEdit });
            items.Add(new SiteMapItem { Name = "Client Save", Controller = "GridInlineEditDemo", Action = "ClientSave", Keywords = "editing save crud", Parent = extraInlineEdit });

            items.Add(new SiteMapItem
            {
                Name = "Inline Editing Conditional",
                Controller = "GridInlineEditDemo",
                Action = "ConditionalDemo",
                Parent = extraInlineEdit
            });

            items.Add(new SiteMapItem
            {
                Name = "Inline Multiple Editors",
                Controller = "GridInlineEditDemo",
                Action = "MultiEditorsDemo",
                Parent = extraInlineEdit
            });

            items.Add(new SiteMapItem
            {
                Name = "Always Edit",
                Controller = "GridInlineEditDemo",
                Action = "AlwaysEdit",
                Parent = extraInlineEdit
            });
            #endregion


            items.Add(new SiteMapItem { Name = "Frozen Columns", Controller = "GridFrozenColumns", Action = "Index", Parent = grid, Keywords = "freeze pinned right left" });
            items.Add(new SiteMapItem { Name = "Custom Querying", Controller = "GridDemo", Action = "CustomQuerying", Parent = grid, Keywords = "database async await sql" });
            items.Add(new SiteMapItem { Name = "Client Side API", Controller = "GridDemo", Action = "ClientSideApi", Parent = grid, Keywords = "refresh reload" });
            items.Add(new SiteMapItem { Name = "Nesting", Controller = "GridNestingDemo", Action = "Index", Keywords = "master detail hierarchical", Parent = grid });
            items.Add(new SiteMapItem { Name = "Tree Grid", Controller = "TreeGrid", Action = "Index", Parent = grid });
            items.Add(new SiteMapItem { Name = "Header Groups", Controller = "GridMultiRowHeaderGroups", Action = "Index", Keywords = "editable table", Parent = grid });
            items.Add(new SiteMapItem { Name = "Custom Render", Controller = "GridCustomRender", Action = "Index", Keywords = "editable table cards view", Parent = grid });
            items.Add(new SiteMapItem { Name = "Infinite Scrolling", Controller = "GridInfiniteScrollingDemo", Action = "Index", Parent = grid });

            var demos = new SiteMapItem { Name = "Grid Demos", Parent = grid };
            items.Add(demos);

            items.Add(new SiteMapItem { Name = "Grid Crud (Popup)", Controller = "GridCrudDemo", Action = "Index", Keywords = "editing", Parent = demos });
            items.Add(new SiteMapItem { Name = "Master Detail crud", Controller = "MasterDetailCrudDemo", Action = "Index", Parent = demos });
            items.Add(new SiteMapItem { Name = "Grid Client Data", Controller = "GridClientDataDemo", Action = "Index", Parent = demos });
            items.Add(new SiteMapItem { Name = "Scheduler", Controller = "SchedulerDemo", Action = "Index", Parent = demos });
            items.Add(new SiteMapItem { Name = "Grid Mailbox", Controller = "MailboxDemo", Action = "Index", Parent = demos });
            items.Add(new SiteMapItem
            {
                Name = "Grid In Nest Editing",
                Controller = "GridNestingDemo",
                Action = "Index",
                Anchor = "#Grid-nest-editing",
                Keywords = "inline crud",
                Parent = demos
            });
            items.Add(new SiteMapItem { Name = "TreeGrid Crud", Controller = "TreeGrid", Action = "Index", Anchor = "#Tree-Grid-with-CRUD-operations", Parent = demos });
            items.Add(new SiteMapItem { Name = "Grid with checkboxes", Controller = "GridCheckboxesDemo", Action = "Index", Parent = demos });
            items.Add(new SiteMapItem { Name = "Spreadsheet Grid", Controller = "GridSpreadsheetDemo", Action = "Index", Keywords = "inline", Parent = demos });

            items.Add(new SiteMapItem
            {
                Name = "Autocomplete Cells Spreadsheet Grid",
                Controller = "GridSpreadsheetDemo",
                Action = "Autocomplete",
                Keywords = "inline",
                Parent = demos,
                NoMenu = true
            });

            items.Add(new SiteMapItem { Name = "Keyboard Navigation", Controller = "KeyboardNavigationDemo", Action = "Index", Parent = demos });

            var ddl = new SiteMapItem { Name = "DropdownList", Keywords = "odropdown editor binding to parents parameters", Collapsed = false, Controller = "DropdownList", Action = "Index" };
            var multis = new SiteMapItem { Name = "Multiselect", Keywords = "multiple editor", Collapsed = false, Controller = "Multiselect", Action = "Index" };
            items.Add(ddl);
            items.Add(multis);
            items.Add(new SiteMapItem { Name = "Multicheck", Keywords = "editor", Collapsed = false, Controller = "Multicheck", Action = "Index" });
            items.Add(new SiteMapItem { Name = "Combobox", Keywords = "editor", Collapsed = false, Controller = "Combobox", Action = "Index" });
            items.Add(new SiteMapItem { Name = "RadioButtonList", Keywords = "editor", Collapsed = false, Controller = "RadioButtonList", Action = "Index" });
            items.Add(new SiteMapItem { Name = "CheckboxList", Keywords = "editor", Collapsed = false, Controller = "CheckboxList", Action = "Index" });
            items.Add(new SiteMapItem { Name = "ActivePanel", Controller = "ActivePanel", Action = "Index", Keywords = "update div container" });
            items.Add(new SiteMapItem { Name = "Popup", Controller = "PopupDemo", Action = "Index", Keywords = "open hint tooltip hover dialog" });
            items.Add(new SiteMapItem { Name = "PopupForm", Controller = "PopupFormDemo", Action = "Index", Keywords = "open" });
            items.Add(new SiteMapItem { Name = "Dropmenu", Controller = "Dropmenu", Action = "Index" });
            items.Add(new SiteMapItem { Name = "Drag And Drop", Controller = "DragAndDropDemo", Action = "Index", Keywords = "grid reorder rows move movable" });
            items.Add(new SiteMapItem { Name = "Autocomplete", Controller = "AutocompleteDemo", Action = "Index" });
            items.Add(new SiteMapItem { Name = "DateTimePicker", Controller = "DatePickerDemo", Action = "Index", Keywords = "timepicker calendar" });
            items.Add(new SiteMapItem { Name = "TextBox", Controller = "TextBoxDemo", Action = "Index", Keywords = "string format func" });
            items.Add(new SiteMapItem { Name = "NumericInput", Controller = "NumericInput", Action = "Index", Keywords = "numeric decimal" });
            items.Add(new SiteMapItem { Name = "ButtonGroup", Keywords = "editor", Collapsed = false, Controller = "ButtonGroup", Action = "Index" });
            items.Add(new SiteMapItem { Name = "Notification", Controller = "Notification", Action = "Index", Keywords = "notify popup" });
            items.Add(new SiteMapItem { Name = "ColorDropdown", Controller = "ColorDropdown", Action = "Index", Keywords = "picker" });
            items.Add(new SiteMapItem { Name = "Toggle", Controller = "FormInput", Action = "Index", Anchor = "#Toggle-button", Keywords = "switch button" });
            items.Add(new SiteMapItem { Name = "Checkbox", Controller = "FormInput", Action = "Index", Anchor = "#Checkbox", Keywords = "checkbox" });
            items.Add(new SiteMapItem { Name = "Form Inputs", Controller = "FormInput", Action = "Index", Keywords = "ocheckbox ochk otoggle button editors" });
            #region lookups

            var lookup = new SiteMapItem { Name = "Lookup", Collapsed = true, Controller = "LookupDemo", Action = "Index", Keywords = "custom search grid" };
            items.Add(lookup);

            var multilookup = new SiteMapItem { Name = "MultiLookup", Collapsed = true, Controller = "MultiLookupDemo", Action = "Index" };

            items.Add(multilookup);
            #endregion

            items.Add(new SiteMapItem { Name = "Tabs", Controller = "Tabs", Action = "Index", Keywords = "tab strip" });
            items.Add(new SiteMapItem { Name = "Call", Controller = "CallDemo", Action = "Index" });
            items.Add(new SiteMapItem { Name = "Form", Controller = "FormDemo", Action = "Index" });
            items.Add(new SiteMapItem { Name = "Pager", Controller = "PagerDemo", Action = "Index" });

            var ajaxlist = new SiteMapItem { Name = "AjaxList", Controller = "AjaxListDemo", Action = "Index", Collapsed = true };

            items.Add(ajaxlist);
            items.Add(new SiteMapItem { Name = "Custom Item Template", Controller = "AjaxListDemo", Action = "CustomItemTemplate", Parent = ajaxlist });
            items.Add(new SiteMapItem { Name = "Client Side API", Controller = "AjaxListDemo", Action = "ClientSideApi", Parent = ajaxlist });

            var native = new SiteMapItem { Name = "Native Editors", Collapsed = true };
            items.Add(native);
            items.Add(new SiteMapItem { Name = "AjaxDropdown", Controller = "AjaxDropdownDemo", Action = "Index", Parent = native });
            items.Add(new SiteMapItem { Name = "AjaxCheckboxList", Controller = "AjaxCheckboxListDemo", Action = "Index", Parent = native });

            var arl = new SiteMapItem { Name = "AjaxRadioList", Controller = "AjaxRadioListDemo", Action = "Index", Parent = native };
            items.Add(arl);


            var generic = new SiteMapItem { Name = "Generic" };

            items.Add(generic);
            items.Add(new SiteMapItem { Name = "Unobtrusive validation", Controller = "Unobtrusive", Action = "Index", Parent = generic, Keywords = "editors inputs" });
            items.Add(new SiteMapItem { Name = "Disabled", Controller = "Disabled", Action = "Index", Parent = generic, Keywords = "enabled" });
            items.Add(new SiteMapItem { Name = "Rtl Demo", Controller = "RtlDemo", Action = "Index", Parent = generic });
            items.Add(new SiteMapItem { Name = "Grid RTL Support", Controller = "GridDemo", Action = "RTLSupport", Parent = generic });
            items.Add(new SiteMapItem { Name = "List Binding", Controller = "ListBinding", Action = "Index", Parent = generic, Keywords = "Phil Haack multi inline" });
            items.Add(new SiteMapItem { Name = "Attributes Demo", Controller = "AttributesDemo", Action = "Index", Parent = generic });
            items.Add(new SiteMapItem { Name = "Error Handling", Controller = "ErrorHandlingDemo", Action = "Index", Parent = generic });
            items.Add(new SiteMapItem { Name = "Client data demo", Controller = "ClientDataLoadingDemo", Action = "Index", Parent = generic });
            items.Add(new SiteMapItem { Name = "Multilevel cascading", Controller = "MultipleLevelCascadingDemo", Action = "Index", Parent = generic });

            var misc = new SiteMapItem { Name = "Misc" };
            items.Add(misc);
            items.Add(new SiteMapItem { Name = "Gantt chart and grid", Controller = "GanttChart", Action = "Index", Parent = misc, Keywords = "google" });
            items.Add(new SiteMapItem { Name = "Pie chart and grid", Controller = "GridChart", Action = "Pie", Parent = misc, Keywords = "google" });
            items.Add(new SiteMapItem { Name = "Wizard Demo", Controller = "WizardDemo", Action = "Index", Parent = misc });
            items.Add(new SiteMapItem { Name = "CRUD inside Lookup", Controller = "CrudInLookup", Action = "Index", Parent = misc });

            items.Add(new SiteMapItem { Name = "Grid export to excel/pdf/txt", Controller = "GridExportToExcelDemo", Action = "Index", Parent = misc, Keywords = "csv" });

            items.Add(new SiteMapItem { Name = "Cascading Grids", Controller = "CascadingGridDemo", Action = "Index", Parent = misc });

            //Items.Add(new SiteMapItem{Name = "Grid maintain selection", Controller = "GridMaintainSelectionDemo", Action = "Index", Parent = misc});
            items.Add(new SiteMapItem { Name = "Grid Sorting", Controller = "GridDemo", Action = "Sorting", Parent = misc });
            items.Add(new SiteMapItem { Name = "Grid selection", Controller = "GridDemo", Action = "Selection", Parent = misc });
            items.Add(new SiteMapItem { Name = "Grid choose columns", Controller = "GridChooseColumnsDemo", Action = "Index", Parent = misc });

            items.Add(new SiteMapItem { Name = "Knockout.js Demo", Controller = "KnockoutDemo", Action = "Index", Parent = misc });
            items.Add(new SiteMapItem { Name = "Popup mod", Controller = "PopupDemo", Action = "PopupMod", Parent = misc });

            items.Add(new SiteMapItem { Name = "Grid Columns Auto Width", Controller = "GridColumnsAutoWidth", Action = "Index", Parent = misc });
            items.Add(new SiteMapItem { Name = "Grid List Count Column", Controller = "GridWithListCountColumn", Action = "Index", Parent = misc, Keywords = "sort order by" });
            items.Add(new SiteMapItem { Name = "Grid Custom Pager", Controller = "CustomPagerGridDemo", Action = "Index", Parent = misc });
            items.Add(new SiteMapItem { Name = "Grid Custom Loading Animation", Controller = "GridNoRecordsFoundCustomLoadingDemo", Action = "Index", Parent = misc });
            items.Add(new SiteMapItem { Name = "Grid Array DataSource", Controller = "GridArrayDataSource", Action = "Index", Parent = misc, Keywords = "Dynamic columns modify config" });
            items.Add(new SiteMapItem { Name = "aweui", Controller = "AweUi", Action = "Index", Parent = misc, Keywords = "react vue.js client angular" });

            items.Add(new SiteMapItem
            {
                Name = "About",
                Controller = "Home",
                Action = "About",
                NoMenu = true,
                Keywords = "learn documentation video tutorial price question forum buy cost license installation download"
            });
        }
    }
}