using AweCoreDemo.Data;
using AweCoreDemo.Hubs;
using AweCoreDemo.Utils;

using Omu.AwesomeMvc;

var builder = WebApplication.CreateBuilder(args);

if (builder.Environment.IsDevelopment())
{
    setupDevRemoteAccess(builder);
}

var services = builder.Services;

services.AddTransient<DbSeeder>();

// Add framework services.
var mvcBuilder = services.AddControllersWithViews(o =>
{
    o.ModelMetadataDetailsProviders.Add(new AweMetaProvider());
})
.AddJsonOptions(opt =>
{
    opt.JsonSerializerOptions.PropertyNamingPolicy = null;
});
mvcBuilder.AddRazorRuntimeCompilation(); // compile views on change 
services.AddSignalR();
services.Configure<RequestLocalizationOptions>(AppCultureSettings.ConfigureReqLocOpt);

setupDatabase(services);

var app = builder.Build();
app.UseExceptionHandler("/Error");
app.UseStatusCodePagesWithReExecute("/Error/HttpStatus", "?code={0}");


app.UseRequestLocalization(AppCultureSettings.ReqLocOpt);

//app.Use(async (context, next) => {
//    context.Request.
//    await next.Invoke();
//});

app.UseStaticFiles(); // needed for signalrSync.js, when loading signalr.js via ajax

app.UseRouting();
app.UseEndpoints(endpoints =>
{
    endpoints.MapControllerRoute(
        name: "default",
        pattern: "{controller=Home}/{action=Index}/{id?}");

    // signalr
    endpoints.MapHub<SyncHub>("/syncHub");
});

new Worker().Start();
initData(app);
//Settings.UseInlineStyleAndJS = false;

app.Run();

void initData(WebApplication app)
{
    using var scope = app.Services.CreateScope();
    var dbSeeder = scope.ServiceProvider.GetRequiredService<DbSeeder>();    

    dbSeeder.Seed(true); // setting true parameter recreates database   
}

void setupDatabase(IServiceCollection services)
{
    // the Db will be recreated on start by the DbSeeder    
    //var cs = builder.Configuration.GetConnectionString("awecs");
    //services.AddDbContext<MyContext>(o => o.UseSqlServer(cs));
}

void setupDevRemoteAccess(WebApplicationBuilder builder)
{
    if (builder.Environment.IsDevelopment())
    {
        // to access site (running from VS http profile) from a different device on the network
        builder.WebHost
        .UseUrls("http://0.0.0.0:5026;")
        .UseKestrel();
    }
}