using AweCoreDemo.Data;
using AweCoreDemo.Hubs;
using AweCoreDemo.Utils;

using Omu.AwesomeMvc;

var builder = WebApplication.CreateBuilder(args);

var services = builder.Services;


services.AddTransient<DbSeeder>();

// Add framework services.
var mvcBuilder = services.AddControllersWithViews(o =>
{
    o.ModelMetadataDetailsProviders.Add(new AweMetaProvider());
})
.AddJsonOptions(opt =>
{
    opt.JsonSerializerOptions.PropertyNamingPolicy = null;
});
mvcBuilder.AddRazorRuntimeCompilation(); // compile views on change 
services.AddSignalR();
services.Configure<RequestLocalizationOptions>(AppCultureSettings.ConfigureReqLocOpt);

var app = builder.Build();
app.UseExceptionHandler("/Error");
app.UseStatusCodePagesWithReExecute("/Error/HttpStatus", "?code={0}");


app.UseRequestLocalization(AppCultureSettings.ReqLocOpt);

app.UseStaticFiles(); // needed for signalrSync.js, when loading signalr.js via ajax

app.UseRouting();
app.UseEndpoints(endpoints =>
{
    endpoints.MapControllerRoute(
        name: "default",
        pattern: "{controller=Home}/{action=Index}/{id?}");

    // signalr
    endpoints.MapHub<SyncHub>("/syncHub");
});

new Worker().Start();
initData(app);

Settings.StringFilterIgnoreCase = true;

//Settings.UseInlineStyleAndJS = false;

app.Run();

void initData(WebApplication app)
{
    using var scope = app.Services.CreateScope();
    var dbSeeder = scope.ServiceProvider.GetRequiredService<DbSeeder>();    

    dbSeeder.Seed(true); // setting true parameter recreates database   
}