using Microsoft.AspNetCore.Localization;
using System.Globalization;

namespace AweCoreDemo.Utils
{
    public static class AppCultureSettings
    {
        public static string[] SupportedCulturesArr
        {
            get => new string[] { "it", "en", "fr", "de", "pt", "ro", "ru", "es", "zh" };
        }

        public static IList<CultureInfo> SupportedCultures
        {
            get => new List<CultureInfo>
            {
                new CultureInfo("en"),
                new CultureInfo("fr"),
                new CultureInfo("it"),
                new CultureInfo("es"),
                new CultureInfo("de"),
                new CultureInfo("pt"),
                new CultureInfo("ro"),
                new CultureInfo("ru"),
                new CultureInfo("zh"),
            };
        }

        public static RequestLocalizationOptions ReqLocOpt => new RequestLocalizationOptions
        {
            DefaultRequestCulture = new RequestCulture("en"),
            // Formatting numbers, dates, etc.
            SupportedCultures = AppCultureSettings.SupportedCultures,
            // UI strings that we have localized.
            SupportedUICultures = AppCultureSettings.SupportedCultures
        };

        public static void ConfigureReqLocOpt(RequestLocalizationOptions opts)
        {
            opts.DefaultRequestCulture = new RequestCulture("en");
            // Formatting numbers, dates, etc.
            opts.SupportedCultures = AppCultureSettings.SupportedCultures;
            // UI strings that we have localized.
            opts.SupportedUICultures = AppCultureSettings.SupportedCultures;
        }
    }
}