﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using AweMySql.Models;
using Omu.AwesomeMvc;
using System.Linq;
using System.Threading.Tasks;

namespace AweMySql.Controllers
{
    public class DataController : Controller
    {
        readonly MyDbContext myCx;

        public DataController(MyDbContext dbContext)
        {
            myCx = dbContext;
        }

        public async Task<IActionResult> CountryGrid(GridParams g, string search)
        {
            search ??= "";            
            var query = myCx.Countries.Where(o => o.Name.Contains(search));

            var gmb = new GridModelBuilder<Country>(query, g)
            {
                KeyProp = o => o.Id,
                ToArrayAsync = qitems => qitems.ToArrayAsync()
            };

            return Json(await gmb.BuildAsync());
        }

        public async Task<ActionResult> GetCountriesPage(GridParams g, string search)
        {
            search ??= "";
            var query = myCx.Countries.Where(o => o.Name.Contains(search));            

            var gmb = new GridModelBuilder<Country>(g)
            {
                KeyProp = o => o.Id
            };            

            query = gmb.OrderBy(query);

            var count = await query.CountAsync();
            var items = await gmb.GetPage(query).ToArrayAsync();

            return Json(new { count, items });
        }

        public async Task<ActionResult> GetAllCountries()
        {
            var items = await myCx.Countries.ToArrayAsync();

            return Json(items.Select(o => new KeyContent(o.Id, o.Name)));
        }

        public async Task<ActionResult> ChefGrid(GridParams g, string search)
        {
            search ??= "";

            var query = myCx.Chefs.Where(o => o.FirstName.Contains(search));

            var gmb = new GridModelBuilder<Chef>(query, g)
            {
                KeyProp = o => o.Id,
                Map = o => new
                {
                    o.Id,
                    o.FirstName,
                    o.LastName,
                    CountryName = o.Country != null ? o.Country.Name : "",
                    o.CountryId
                },
                ToArrayAsync = qitems => qitems.Include(o => o.Country).ToArrayAsync()
            };

            return Json(await gmb.BuildAsync());
        }
    }
}