﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using MinSetupCore.Data;
using MinSetupCore.Models;
using MinSetupCore.Utils;
using Omu.AwesomeMvc;

namespace MinSetupCore.Controllers.Awesome.Lookup
{
    public class ChefLookupController : Controller
    {
        private readonly MyContext mcx;

        public ChefLookupController(MyContext mcx)
        {
            this.mcx = mcx;
        }

        public async Task<IActionResult> GetItem(int? v)
        {
            Check.NotNull(v, "v");
            var o = await mcx.FindAsync<Chef>(v) ?? new Chef();
            return Json(new KeyContent(o.Id, o.FirstName + " " + o.LastName));
        }

        public async Task<IActionResult> Search(int page, string search = "")
        {
            const int PageSize = 7;

            var list = await mcx.Chefs.Where(f => (f.FirstName + " " + f.LastName).Contains(search)).ToListAsync();
            return Json(new AjaxListResult
            {
                Items = list.Skip((page - 1) * PageSize).Take(PageSize).Select(o => new KeyContent(o.Id, o.FullName)),
                More = list.Count() > page * PageSize
            });
        }
    }
}