﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using MinSetupCore.Data;
using MinSetupCore.Models;
using MinSetupCore.Utils.Awesome;
using Omu.AwesomeMvc;

namespace MinSetupCore.Controllers.Awesome.MultiLookup
{
    public class MealsMultiLookupController : Controller
    {
        private readonly MyContext mcx;

        public MealsMultiLookupController(MyContext mcx)
        {
            this.mcx = mcx;
        }

        public async Task<IActionResult> GetItems(int[] v)
        {
            var items = new List<Meal>();

            if (v != null)
            {
                items = await mcx.Meals
                    .Where(o => v.Contains(o.Id)).ToListAsync();
            }

            return Json(items.Select(meal => new KeyContent(meal.Id, meal.Name)));
        }

        public async Task<IActionResult> Search(int[] selected, int page, string search)
        {
            selected = selected ?? new int[] { };

            var query = mcx.Meals.Filter(search, o => o.Name).Where(o => !selected.Contains(o.Id));

            var res = await EFAweUtil.BuildListAsync(query, page, keyContentFunc: o => new KeyContent(o.Id, o.Name));

            return Json(res);
        }

        public async Task<IActionResult> Selected(int[] selected)
        {
            var items = new List<Meal>();

            if (selected != null)
            {
                items = await mcx.Meals
                    .Where(o => selected.Contains(o.Id)).ToListAsync();
            }

            return Json(new AjaxListResult
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name))
            });
        }
    }
}