using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using MinSetupCore.Data;
using Omu.AwesomeMvc;

var builder = WebApplication.CreateBuilder(args);

var provider = new AweMetaProvider();

// Add services to the container.
var mvcBuilder = builder.Services.AddControllersWithViews(o => o.ModelMetadataDetailsProviders.Add(provider));

mvcBuilder.AddRazorRuntimeCompilation(); // compile views on change 

builder.Services.AddTransient<DbSeeder>();

setupDatabase(builder.Services);

var app = builder.Build();

// Configure the HTTP request pipeline.
if (!app.Environment.IsDevelopment())
{
    app.UseExceptionHandler("/Home/Error");
}
app.UseStaticFiles();

app.UseRouting();

app.UseAuthorization();

app.MapControllerRoute(
    name: "default",
    pattern: "{controller=Home}/{action=Index}/{id?}");

initData(app);

app.Run();


void initData(WebApplication app)
{
    using var scope = app.Services.CreateScope();
    var dbSeeder = scope.ServiceProvider.GetRequiredService<DbSeeder>();
    var mcx = scope.ServiceProvider.GetRequiredService<MyContext>();

    dbSeeder.Seed(true); // setting true parameter recreates database
}

void setupDatabase(IServiceCollection services)
{
    // the Db will be recreated on start by the DbSeeder    
    var cs = builder.Configuration.GetConnectionString("awecs");
    services.AddDbContext<MyContext>(o => o.UseSqlServer(cs));
}