﻿Imports System.Web.Mvc
Imports Omu.AwesomeWebForms
Imports AweWFUtil
Imports AweWFVB.ViewModels.Input
Imports Omu.SampleData
Imports Omu.SampleData.Model

Namespace Controllers
    ''' <summary>
    ''' Note: If your Key Property is not "Id", you need to change the Key = "MyId" in GridGetItems and it in the view
    ''' change the Bind = "MyId" (if you're showing the id column), 
    ''' and for the action columns you can either specify the MyId property (e.g. GridUtils.EditFormatForGrid("DinnersGrid", "MyId"));
    ''' or in MapToGridModel additionally to o.MyId add another property Id = o.MyId
    ''' parameters for Edit, Delete need to remain called "id", that's how they are set in GridUtils.cs (params:{{ id: );
    ''' Edit and Delete post actions must return an object with property "Id" - in aweUtils.js itemEdited and itemDeleted funcs expect it this way;
    ''' </summary>
    Public Class DinnersGridCrudController
        Inherits Controller

        Private Shared Function MapToGridModel(ByVal o As Dinner) As Object
            Return New With {
            o.Id,
            o.Name,
            .Date = o.Date.ToShortDateString(),
            .ChefName = o.Chef.FirstName + " " + o.Chef.LastName,
            .Meals = String.Join(", ", o.Meals.Select(Function(m) m.Name))
            }
        End Function

        Public Function GridGetItems(ByVal g As GridParams, ByVal search As String) As ActionResult
            search = search.ToLowCaseVal()

            Dim items = Db.Dinners.Where(Function(o) o.Name.ToLower().Contains(search)).AsQueryable()
            Return Json(New GridModelBuilder(Of Dinner)(items, g) _
                With {
                   .KeyProp = Function(o) o.Id,
                    .GetItem = Function() Db.Get(Of Dinner)(Convert.ToInt32(g.Key)), ' called by the grid.api.update ( edit popupform success js func )
                   .Map = Function(o) MapToGridModel(o)}.Build())
        End Function

        Public Overloads Function Create() As ActionResult
            Return PartialView()
        End Function

        <HttpPost()>
        Public Overloads Function Create(ByVal input As DinnerInput) As ActionResult
            If Not ModelState.IsValid Then
                Return PartialView(input)
            End If

            Dim d1 = New Dinner() With {
                        .Name = input.Name,
                        .Date = input.Date.Value,
                        .Chef = Db.Get(Of Chef)(input.Chef),
                        .Meals = Db.Meals.Where(Function(o) input.Meals.Contains(o.Id)),
                        .BonusMeal = Db.Get(Of Meal)(input.BonusMealId)}

            Dim dinner = Db.Insert(d1)
            Return Json(MapToGridModel(dinner))
            ' returning grid model, used in grid.api.renderRow
        End Function

        Public Overloads Function Edit(ByVal id As Integer) As ActionResult
            Dim dinner = Db.Get(Of Dinner)(id)
            Dim input = New DinnerInput() With {
                .Id = dinner.Id,
                .Name = dinner.Name,
                .Chef = dinner.Chef.Id,
                .Date = dinner.Date,
                .Meals = dinner.Meals.Select(Function(o) o.Id),
                .BonusMealId = dinner.BonusMeal.Id
            }

            Return PartialView("Create", input)
        End Function

        <HttpPost()>
        Public Overloads Function Edit(ByVal input As DinnerInput) As ActionResult
            If Not ModelState.IsValid Then
                Return PartialView("Create", input)
            End If

            Dim dinner = Db.Get(Of Dinner)(input.Id)
            dinner.Name = input.Name
            dinner.Date = input.Date.Value
            dinner.Chef = Db.Get(Of Chef)(input.Chef)
            dinner.Meals = Db.Meals.Where(Function(m) input.Meals.Contains(m.Id))
            dinner.BonusMeal = Db.Get(Of Meal)(input.BonusMealId)
            Db.Update(dinner)
            ' returning the key to call grid.api.update
            Return Json(New With {dinner.Id})
        End Function

        Public Overloads Function Delete(id As Integer, gridId As String) As ActionResult
            Dim dinner = Db.Get(Of Dinner)(id)
            Return PartialView(New DeleteConfirmInput() With {
                               .Id = id,
                               .GridId = gridId,
                               .Message = String.Format("Are you sure you want To delete dinner <b>{0}</b> ?", dinner.Name)})
        End Function

        <HttpPost()>
        Public Overloads Function Delete(ByVal input As DeleteConfirmInput) As ActionResult
            Db.Delete(Of Dinner)(input.Id)
            Return Json(New With {.Id = input.Id})
        End Function
    End Class
End Namespace