using System.Web;
using Omu.AwesomeMvc;
using System.Reflection;
using System.Text;

namespace Omu.Awem.Helpers
{
    /// <summary>
    /// 
    /// </summary>
    public static class AwesomeHtmlHelperExtensions
    {
        /// <summary>
        /// Initializes awesome and aweui (if present), by generating a script that will
        /// set the defaults for awem.js and jquery.validate (if present); 
        /// sets date format, first day of week, decimal separator, isMobileOrTablet
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="ahtml"></param>
        /// <param name="isMobile"></param>
        /// <returns></returns>
        public static IHtmlString Init<T>(this AwesomeHtmlHelper<T> ahtml, bool? isMobile = null)
        {
            var isMobileOrTablet = (isMobile ?? Autil.IsMobileOrTablet(ahtml)) ? 1 : 0;
            var dateFormat = Autil.CurrentCulture().DateTimeFormat.ShortDatePattern;
            var decimalSep = Autil.CurrentCulture().NumberFormat.NumberDecimalSeparator;

            var sb = new StringBuilder();
            sb.Append(ahtml.InitAwe());
            sb.Append("<script>");
            sb.AppendFormat("awem.isMobileOrTablet = function() {{ return {0}; }};", isMobileOrTablet);
            sb.AppendFormat("awem.fdw = {0};", (int)Autil.CurrentCulture().DateTimeFormat.FirstDayOfWeek);
            sb.AppendFormat("aweUtils.init('{0}', {1}, '{2}');", dateFormat, isMobileOrTablet, decimalSep);
            sb.AppendFormat("if(window.aweui){{aweui.init(); aweui.decimalSep = '{0}';}}", decimalSep);
            sb.Append("awe.mdl = [awem, aweUtils]");
            sb.Append("</script>");

            return new HtmlString(sb.ToString());
        }

        private static string getVersion(int fieldCount)
        {
            var typeInfo = typeof(Grid<>).GetTypeInfo();
            var assembly = typeInfo.Assembly;
            var anm = assembly.GetName();
            return anm.Version.ToString(fieldCount);
        }

        /// <summary>
        /// Init tag, used by aweinit.js
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="ahtml"></param>
        /// <param name="isMobile"></param>
        /// <returns></returns>
        public static IHtmlString InitTag<T>(this AwesomeHtmlHelper<T> ahtml, bool? isMobile = null)
        {
            var isMobileOrTablet = (isMobile ?? Autil.IsMobileOrTablet(ahtml)) ? 1 : 0;
            var dateFormat = Autil.CurrentCulture().DateTimeFormat.ShortDatePattern;
            var decimalSep = Autil.CurrentCulture().NumberFormat.NumberDecimalSeparator;
            var version = getVersion(3);

            var sb = string.Format(
                "<svg class='aweintag o-hid' width='0' height='0' data-ismob='{0}' data-datef='{1}' data-fdw='{2}' data-decsep='{3}' data-version='{4}' data-noinl='{5}'></svg>",
                isMobileOrTablet,
                dateFormat,
                (int)Autil.CurrentCulture().DateTimeFormat.FirstDayOfWeek,
                decimalSep,
                version,
                !Settings.UseInlineStyleAndJS);

            return new HtmlString(sb);
        }

        /// <summary>
        /// Call this helper in head to avoid calling jquery inside the head tag
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="ahtml"></param>
        /// <returns></returns>
        public static IHtmlString AwejQuery<T>(this AwesomeHtmlHelper<T> ahtml)
        {
            var sb = new StringBuilder("<script>");
            sb.Append("var awejqfuncs = [];if (!jQuery) {var jQuery = function (fn) { awejqfuncs.push(fn); };var $ = jQuery;}");
            sb.Append("</script>");
            return new HtmlString(sb.ToString());
        }
    }
}