using Omu.AwesomeMvc;
using System;

namespace Omu.Awem.Helpers
{
    /// <summary>
    /// Awesome mod helpers
    /// </summary>
    public static class AwesomeModHelpers
    {
        #region obsolete
        /// <summary>
        /// Odropdown Awesome Mod 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="ahtml"></param>
        /// <param name="prop"></param>
        /// <param name="setCfg"></param>
        /// <returns></returns>
        [Obsolete("Use DropdownList instead")]
        public static AjaxRadioList<T> Odropdown<T>(this AwesomeHtmlHelper<T> ahtml, string prop, Action<OddCfg> setCfg = null)
        {
            var res = ahtml.AjaxRadioList(prop).Mod("awem.odropdown");
            var odcfg = new OddCfg();

            if (setCfg != null)
            {
                setCfg(odcfg);
                res.Tag(odcfg.ToTag());
            }

            return res;
        }
        #endregion

        /// <summary>
        /// Init popup form
        /// </summary>
        public static IAweBaseHelper InitPopupForm<T>(this AwesomeHtmlHelper<T> ahtml, PopupFormOpt opt)
        {
            var res = ahtml.InitPopupForm().Name(opt.Name);

            if (opt.Width.HasValue)
            {
                res.Width(opt.Width.Value);
            }

            if (opt.Height.HasValue)
            {
                res.Height(opt.Height.Value);
            }

            res
            .MaxWidth(opt.MaxWidth)
            .Title(opt.Title)
            .Modal(opt.Modal)
            .Fullscreen(opt.Fullscreen)
            .Url(opt.Url)

            .Content(opt.Content)
            .PopupClass(opt.PopupClass)
            .Top(opt.Top)
            .Group(opt.Group)
            //.LoadOnce(opt.LoadOnce)
            .Close(opt.OnClose);

            res.OnLoad(opt.OnLoad);

            if (opt.UseDefaultButtons.HasValue)
            {
                res.UseDefaultButtons(opt.UseDefaultButtons.Value);
            }

            res.Success(opt.OnSuccess);

            res.Mod(o =>
            {
                if (opt.Dropdown)
                {
                    o.Dropdown();
                }

                if (opt.OutClickClose)
                {
                    o.OutClickClose();
                }

                if (opt.ShowHeader.HasValue)
                {
                    o.ShowHeader(opt.ShowHeader.Value);
                }

                if (opt.Inline || opt.InlTrg != null)
                {
                    o.Inline(opt.InlTrg);
                }
            });

            if (opt.Parents != null)
            {
                foreach (var parent in opt.Parents)
                {
                    res.Parent(parent.Id, parent.Alias ?? "parent", parent.Context ?? true);
                }
            }

            if (opt.Parameters != null)
            {
                foreach (var p in opt.Parameters)
                {
                    res.Parameter(p.Name, p.Value);
                }
            }

            res.ParameterFunc(opt.ParameterFunc);

            if (opt.Buttons != null)
            {
                foreach (var button in opt.Buttons)
                {
                    res.Button(button.Text, button.Action);
                }
            }

            return res;
        }

        /// <summary>
        /// Init popup
        /// </summary>
        public static IAweBaseHelper InitPopup<T>(this AwesomeHtmlHelper<T> ahtml, PopupOpt opt)
        {
            var res = ahtml.InitPopup().Name(opt.Name);

            if (opt.Width.HasValue)
            {
                res.Width(opt.Width.Value);
            }

            if (opt.Height.HasValue)
            {
                res.Height(opt.Height.Value);
            }

            res.MaxWidth(opt.MaxWidth)
            .Title(opt.Title)
            .Modal(opt.Modal)
            .Fullscreen(opt.Fullscreen)
            .Url(opt.Url)

            .Content(opt.Content)
            .PopupClass(opt.PopupClass)
            .Top(opt.Top)
            .Group(opt.Group)
            .LoadOnce(opt.LoadOnce)
            .Close(opt.OnClose);

            res.Mod(o =>
            {
                if (opt.Dropdown)
                {
                    o.Dropdown();
                }

                if (opt.OutClickClose)
                {
                    o.OutClickClose();
                }

                if (opt.ShowHeader.HasValue)
                {
                    o.ShowHeader(opt.ShowHeader.Value);
                }

                if (opt.Inline || opt.InlTrg != null)
                {
                    o.Inline(opt.InlTrg);
                }
            });

            if (opt.Parents != null)
            {
                foreach (var parent in opt.Parents)
                {
                    res.Parent(parent.Id, parent.Alias ?? "parent", parent.Context ?? true);
                }
            }

            if (opt.Parameters != null)
            {
                foreach (var p in opt.Parameters)
                {
                    res.Parameter(p.Name, p.Value);
                }
            }

            res.ParameterFunc(opt.ParameterFunc);

            if (opt.Buttons != null)
            {
                foreach (var button in opt.Buttons)
                {
                    res.Button(button.Text, button.Action);
                }
            }

            return res;
        }

        /// <summary>
        /// Button
        /// </summary>
        public static IAweBaseHelper Button<T>(this AwesomeHtmlHelper<T> ahtml, ButtonOpt opt)
        {
            var res = ahtml.Button()
                .CssClass(opt.CssClass)
                .OnClick(opt.OnClick)
                .Text(opt.Text)
                .HtmlAttributes(opt.HtmlAttributes);

            if (opt.Enabled.HasValue)
            {
                res.Enabled(opt.Enabled.Value);
            }

            return res;
        }

        /// <summary>
        /// Toggle button
        /// </summary>
        public static IAweValHelper Toggle<T>(this AwesomeHtmlHelper<T> ahtml, ToggleOpt opt)
        {
            var res = ahtml.CheckBox(opt.Name)
                .Otoggl(cfg =>
                cfg.Width(opt.Width)
                .Yes(opt.Yes)
                .No(opt.No));

            if (opt.EmptyFalseVal)
            {
                res.EmptyFalseVal();
            }

            setValHelper(res, opt);

            return res;
        }

        /// <summary>
        /// ColorDropdown
        /// </summary>
        public static IAweValHelper ColorDropdown<T>(this AwesomeHtmlHelper<T> ahtml, ColorDropdownOpt opt)
        {
            var res = ahtml.AjaxRadioList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);
            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.ColorDropdown(d =>
            {
                if (opt.ClearBtn != null)
                {
                    d.ClearBtn(opt.ClearBtn.Value);
                }

                d.AutoSelectFirst(opt.AutoSelectFirst);
            });

            return res;
        }

        /// <summary>
        /// Checkbox
        /// </summary>
        public static IAweValHelper CheckBox<T>(this AwesomeHtmlHelper<T> ahtml, CheckBoxOpt opt)
        {
            var res = ahtml.CheckBox(opt.Name).Ochk();

            res.HtmlAttributes(new { @class = "o-chk-field" });
            if (opt.EmptyFalseVal)
            {
                res.EmptyFalseVal();
            }

            setValHelper(res, opt);

            return res;
        }

        /// <summary>
        /// Autocomplete control
        /// </summary>
        public static IAweValHelper Autocomplete<T>(this AwesomeHtmlHelper<T> ahtml, AutocompleteOpt opt)
        {
            var res = ahtml.Autocomplete(opt.Name);

            if (opt.Url != null)
            {
                res.Url(opt.Url);
            }

            if (opt.DataFunc != null)
            {
                res.DataFunc(opt.DataFunc);
            }

            res.Numeric(opt.Numeric);
            res.Encode(opt.Encode);
            res.Placeholder(opt.Placeholder);
            res.MaxLength(opt.MaxLength);
            res.MinLength(opt.MinLength);
            res.Delay(opt.Delay);
            res.Cache(opt.Cache);
            res.PropId(opt.PropId);
            res.GeneratePropId(opt.GeneratePropId);
            res.ItemFunc(opt.ItemFunc);

            res.CssClass(opt.CssClass);

            setValHelper(res, opt);

            setParentsParms(opt, res);

            //res.ParameterFunc(opt.ParameterFunc);

            return res;
        }

        /// <summary>
        /// Textbox control
        /// </summary>
        public static IAweValHelper TextBox<T>(this AwesomeHtmlHelper<T> ahtml, TextBoxOpt opt)
        {
            var res = ahtml.TextBox(opt.Name);

            if (opt.ClearBtn != null)
            {
                res.ClearButton(opt.ClearBtn.Value);
            }

            res.CssClass(opt.CssClass);
            res.Placeholder(opt.Placeholder);
            setValHelper(res, opt);

            res.FormatFunc(opt.FormatFunc);

            var editorAttrDict = AweHelperUtil.GetAttrDictFromObj(opt.EditorAttr);

            if (opt.Type != null)
            {
                editorAttrDict.Add("type", opt.Type);
            }

            if (editorAttrDict.Count > 0)
            {
                res.EditorHtmlAttributes(editorAttrDict);
            }

            return res;
        }

        /// <summary>
        /// Numeric input control
        /// </summary>
        public static IAweValHelper NumericInput<T>(this AwesomeHtmlHelper<T> ahtml, NumericInputOpt opt)
        {
            var res = ahtml.TextBox(opt.Name);

            if (opt.ClearBtn != null)
            {
                res.ClearButton(opt.ClearBtn.Value);
            }

            res.CssClass(opt.CssClass);
            res.Placeholder(opt.Placeholder);
            setValHelper(res, opt);

            res.FormatFunc(opt.FormatFunc);
            res.EditorHtmlAttributes(null, opt.EditorAttr);
            res.ReadonlyInput(opt.ReadonlyInput);

            res.Numeric(cfg =>
            {
                if (opt.Decimals != null)
                {
                    cfg.Decimals(opt.Decimals.Value);
                }

                if (opt.Min != null)
                {
                    cfg.Min(opt.Min.Value);
                }

                if (opt.Max != null)
                {
                    cfg.Max(opt.Max.Value);
                }

                if (opt.AllowNegative)
                {
                    cfg.AllowNegative();
                }

                if (opt.ShowSpinners.HasValue)
                {
                    cfg.ShowSpinners(opt.ShowSpinners.Value);
                }

                if (opt.Step.HasValue)
                {
                    cfg.Step(opt.Step.Value);
                }

                cfg.NoFocusOnSpin(opt.NoFocusOnSpin);
                cfg.BigSpinButtons(opt.BigSpinButtons);
            });

            return res;
        }

        /// <summary>
        /// DateTimePicker helper
        /// </summary>
        public static IAweValHelper DateTimePicker<T>(this AwesomeHtmlHelper<T> ahtml, DateTimePickerOpt opt)
        {
            var res = (DatePicker<T>)DatePickerBase(ahtml, opt);
            res.ShowSeconds(opt.ShowSeconds);
            return res;
        }

        /// <summary>
        /// DatePicker helper
        /// </summary>
        public static IAweValHelper DatePicker<T>(this AwesomeHtmlHelper<T> ahtml, DatePickerOpt opt)
        {
            var res = (DatePicker<T>)DatePickerBase(ahtml, opt);

            res.SelectRange(opt.SelectRange);

            return res;
        }

        private static IAweValHelper DatePickerBase<T>(AwesomeHtmlHelper<T> ahtml, DatePickerBaseOpt opt)
        {
            var res = ahtml.DatePicker(opt.Name);

            if (opt.ClearBtn != null)
            {
                res.ClearButton(opt.ClearBtn.Value);
            }
            res.CssClass(opt.CssClass);

            if (opt.Placeholder != null)
            {
                res.Placeholder(opt.Placeholder);
            }

            setValHelper(res, opt);

            res.ChangeMonth(opt.ChangeMonth ?? true);
            res.ChangeYear(opt.ChangeYear ?? true);

            res.Opt = opt;

            if (opt.DefaultDate.HasValue)
            {
                res.DefaultDate(opt.DefaultDate.Value);
            }

            if (opt.MaxDate.HasValue)
            {
                res.MaxDate(opt.MaxDate.Value);
            }

            if (opt.MinDate.HasValue)
            {
                res.MinDate(opt.MinDate.Value);
            }

            if (opt.NumberOfMonths.HasValue)
            {
                res.NumberOfMonths(opt.NumberOfMonths.Value);
            }

            if (opt.IsRtl.HasValue)
            {
                res.IsRtl(opt.IsRtl.Value);
            }

            res.DayFunc(opt.DayFunc);
            res.InlineCalendar(opt.InlineCalendar);

            return res;
        }

        /// <summary>
        /// CheckboxList control
        /// </summary>
        public static IAweValHelper CheckBoxList<T>(this AwesomeHtmlHelper<T> ahtml, CheckBoxListOpt opt)
        {
            var res = ahtml.AjaxCheckboxList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);
            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);
            res.SelectAll(opt.SelectAll);

            res.Ochk(d =>
            {
                d.ItemFunc(opt.ItemFunc);
            });
            return res;
        }

        /// <summary>
        /// ButtonGroup control
        /// </summary>        
        public static IAweValHelper ButtonGroup<T>(this AwesomeHtmlHelper<T> ahtml, ButtonGroupOpt opt)
        {
            if (opt.Multiple)
            {
                var r1 = ahtml.AjaxCheckboxList(opt.Name);

                setLoadHelper(r1, opt);

                r1.CssClass(opt.CssClass);

                setValHelper(r1, opt);

                setParentsParms(opt, r1);
                r1.ParameterFunc(opt.ParameterFunc);

                r1.ButtonGroup(o => o.AutoSelectFirst(opt.AutoSelectFirst));

                return r1;
            }

            var res = ahtml.AjaxRadioList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);
            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.ButtonGroup(o => o.AutoSelectFirst(opt.AutoSelectFirst));

            return res;
        }

        /// <summary>
        /// RadioButtonList control
        /// </summary>
        public static IAweValHelper RadioButtonList<T>(this AwesomeHtmlHelper<T> ahtml, RadioButtonListOpt opt)
        {
            var res = ahtml.AjaxRadioList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);
            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Ochk(d =>
            {
                d.ItemFunc(opt.ItemFunc);
            });

            return res;
        }

        /// <summary>
        /// Multicheck control
        /// </summary>
        public static IAweValHelper Multicheck<T>(this AwesomeHtmlHelper<T> ahtml, MulticheckOpt opt)
        {
            var res = ahtml.AjaxCheckboxList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);
            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Multichk(d =>
            {
                if (opt.ClearBtn != null)
                {
                    d.ClearBtn(opt.ClearBtn.Value);
                }

                d.ItemFunc(opt.ItemFunc)
                .NoAutoFocus(opt.NoAutoFocus)
                .FilterFunc(opt.FilterFunc)
                .CaptionFunc(opt.CaptionFunc)
                .AutoSelectFirst(opt.AutoSelectFirst)
                .Submenu(opt.Submenu)
                .CollapseNodes(opt.CollapseNodes)
                .InLabel(opt.InLabel)
                .PopupMinWidth(opt.PopupMinWidth)
                .PopupMaxWidth(opt.PopupMaxWidth)
                .PopupMaxHeight(opt.PopupMaxHeight);

                d.Caption(opt.Caption);

                if (opt.SearchFunc != null)
                {
                    d.SearchFunc(opt.SearchFunc.Name, opt.SearchFunc.Url, opt.SearchFunc.Key, opt.SearchFunc.NoCache);
                }
            });

            return res;
        }

        /// <summary>
        /// DropdownList control
        /// </summary>
        public static IAweValHelper DropdownList<T>(this AwesomeHtmlHelper<T> ahtml, DropdownListOpt opt)
        {
            var res = ahtml.AjaxRadioList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);

            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Odropdown(d =>
            {
                if (opt.ClearBtn != null)
                {
                    d.ClearBtn(opt.ClearBtn.Value);
                }

                d.ItemFunc(opt.ItemFunc)
                .NoAutoFocus(opt.NoAutoFocus)
                .FilterFunc(opt.FilterFunc)
                .CaptionFunc(opt.CaptionFunc)
                .AutoSelectFirst(opt.AutoSelectFirst)
                .Submenu(opt.Submenu)
                .CollapseNodes(opt.CollapseNodes)

                .InLabel(opt.InLabel)
                .OpenOnHover(opt.OpenOnHover)

                .PopupMinWidth(opt.PopupMinWidth)
                .PopupMaxWidth(opt.PopupMaxWidth)
                .PopupMaxHeight(opt.PopupMaxHeight);

                d.Caption(opt.Caption);

                if (opt.AutoSearch.HasValue)
                {
                    d.AutoSearch(opt.AutoSearch.Value);
                }

                if (opt.Favs != null)
                {
                    d.Favs(opt.Favs.Count, opt.Favs.Values, opt.Favs.Frozen);
                }

                if (opt.SearchFunc != null)
                {
                    d.SearchFunc(opt.SearchFunc.Name, opt.SearchFunc.Url, opt.SearchFunc.Key, opt.SearchFunc.NoCache);
                }
            });

            return res;
        }

        /// <summary>
        /// Combobox control
        /// </summary>
        public static IAweValHelper Combobox<T>(this AwesomeHtmlHelper<T> ahtml, ComboboxOpt opt)
        {
            var res = ahtml.AjaxRadioList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);

            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Combobox(d =>
            {
                if (opt.ClearBtn != null)
                {
                    d.ClearBtn(opt.ClearBtn.Value);
                }

                d.ItemFunc(opt.ItemFunc)
                .NoAutoFocus(opt.NoAutoFocus)
                .FilterFunc(opt.FilterFunc)
                .AutoSelectFirst(opt.AutoSelectFirst)
                .Submenu(opt.Submenu)
                .CollapseNodes(opt.CollapseNodes)
                .PopupMinWidth(opt.PopupMinWidth)
                .PopupMaxWidth(opt.PopupMaxWidth)
                .PopupMaxHeight(opt.PopupMaxHeight)
                .UseConVal(opt.UseContentValue)
                .Caption(opt.Caption)
                .SelectOnFocus(opt.SelectOnFocus)
                .OpenOnClickFocus(opt.OpenOnClickFocus);

                if (opt.ShowComboItem.HasValue)
                {
                    d.ShowComboItem(opt.ShowComboItem.Value);
                }

                if (opt.SearchFunc != null)
                {
                    d.SearchFunc(opt.SearchFunc.Name, opt.SearchFunc.Url, opt.SearchFunc.Key, opt.SearchFunc.NoCache);
                }
            });

            return res;
        }

        /// <summary>
        /// MultiLookup Control
        /// </summary>        
        public static MultiLookup<T> MultiLookup<T>(this AwesomeHtmlHelper<T> ahtml, MultiLookupOpt opt)
        {
            var res = ahtml.MultiLookup(opt.Name);

            res.CssClass(opt.CssClass);

            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Controller(opt.Controller);
            res.GetItemsUrl(opt.GetItemsUrl);
            res.PopupUrl(opt.PopupUrl);
            res.SearchFormUrl(opt.SearchFormUrl);
            res.SearchUrl(opt.SearchUrl);
            res.Title(opt.Title);
            res.PopupClass(opt.PopupClass);
            res.ClearButton(opt.ClearButton);
            res.HighlightChange(opt.HightlightChange);
            res.Fullscreen(opt.Fullscreen);
            res.CustomSearch(opt.CustomSearch);

            if (opt.Height.HasValue)
            {
                res.Height(opt.Height.Value);
            }

            if (opt.Width.HasValue)
            {
                res.Width(opt.Width.Value);
            }

            if (opt.MaxWidth.HasValue)
            {
                res.MaxWidth(opt.MaxWidth.Value);

                if (!opt.Width.HasValue)
                {
                    res.Width(opt.MaxWidth.Value);
                }
            }

            if (opt.SyncScript)
            {
                res.SyncScript();
            }

            if (opt.Modal.HasValue)
            {
                res.Modal(opt.Modal.Value);
            }

            res.Mod(o =>
            {
                if (opt.Dropdown.HasValue)
                {
                    o.Dropdown(opt.Dropdown.Value);
                }

                if (opt.OutClickClose.HasValue)
                {
                    o.OutClickClose(opt.OutClickClose.Value);
                }

                if (opt.ShowHeader.HasValue)
                {
                    o.ShowHeader(opt.ShowHeader.Value);
                }
            });

            return res;
        }

        /// <summary>
        /// Lookup control
        /// </summary>
        public static Lookup<T> Lookup<T>(this AwesomeHtmlHelper<T> ahtml, LookupOpt opt)
        {
            var res = ahtml.Lookup(opt.Name);

            res.CssClass(opt.CssClass);

            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Controller(opt.Controller);
            res.GetItemUrl(opt.GetItemUrl);
            res.PopupUrl(opt.PopupUrl);
            res.SearchFormUrl(opt.SearchFormUrl);
            res.SearchUrl(opt.SearchUrl);
            res.Title(opt.Title);
            res.PopupClass(opt.PopupClass);
            res.ClearButton(opt.ClearButton);
            res.HighlightChange(opt.HightlightChange);
            res.Fullscreen(opt.Fullscreen);
            res.CustomSearch(opt.CustomSearch);

            if (opt.Height.HasValue)
            {
                res.Height(opt.Height.Value);
            }

            if (opt.Width.HasValue)
            {
                res.Width(opt.Width.Value);
            }

            if (opt.MaxWidth.HasValue)
            {
                res.MaxWidth(opt.MaxWidth.Value);

                if (!opt.Width.HasValue)
                {
                    res.Width(opt.MaxWidth.Value);
                }
            }

            if (opt.SyncScript)
            {
                res.SyncScript();
            }

            if (opt.Modal.HasValue)
            {
                res.Modal(opt.Modal.Value);
            }

            res.Mod(o =>
            {
                if (opt.Dropdown.HasValue)
                {
                    o.Dropdown(opt.Dropdown.Value);
                }

                if (opt.OutClickClose.HasValue)
                {
                    o.OutClickClose(opt.OutClickClose.Value);
                }

                if (opt.ShowHeader.HasValue)
                {
                    o.ShowHeader(opt.ShowHeader.Value);
                }
            });

            return res;
        }

        /// <summary>
        /// Multiselect control
        /// </summary>
        public static IAweValHelper Multiselect<T>(this AwesomeHtmlHelper<T> ahtml, MultiselectOpt opt)
        {
            var res = ahtml.AjaxCheckboxList(opt.Name);

            setLoadHelper(res, opt);

            res.CssClass(opt.CssClass);

            setValHelper(res, opt);

            setParentsParms(opt, res);
            res.ParameterFunc(opt.ParameterFunc);

            res.Multiselect(d =>
            {
                if (opt.ClearBtn != null)
                {
                    d.ClearBtn(opt.ClearBtn.Value);
                }

                d.ItemFunc(opt.ItemFunc)
                .NoAutoFocus(opt.NoAutoFocus)
                .FilterFunc(opt.FilterFunc)
                .CaptionFunc(opt.CaptionFunc)
                .Submenu(opt.Submenu)
                .NoSelectClose(opt.NoSelectClose)
                .CollapseNodes(opt.CollapseNodes)
                .PopupMinWidth(opt.PopupMinWidth)
                .PopupMaxWidth(opt.PopupMaxWidth)
                .PopupMaxHeight(opt.PopupMaxHeight);

                if (opt.SearchFunc != null)
                {
                    d.SearchFunc(opt.SearchFunc.Name, opt.SearchFunc.Url, opt.SearchFunc.Key, opt.SearchFunc.NoCache);
                }

                if (opt.Combo)
                {
                    d.Combo(opt.ComboGenKey ?? true);
                }
            });

            return res;
        }

        private static void setLoadHelper<TH>(ILoad<TH> helper, DataEditorOpt opt)
        {
            if (opt.Load.HasValue)
            {
                helper.Load(opt.Load.Value);
            }

            if (opt.Url != null)
            {
                helper.Url(opt.Url);
            }

            if (opt.DataFunc != null)
            {
                helper.DataFunc(opt.DataFunc);
            }
        }

        private static void setValHelper<TH>(IValueHelper<TH> helper, EditorOpt opt)
        {
            if (opt.Value != null)
            {
                helper.Value(opt.Value);
            }

            if (opt.Enabled.HasValue)
            {
                helper.Enabled(opt.Enabled.Value);
            }

            if (opt.SyncScript)
            {
                helper.SyncScript();
            }

            if (opt.Prefix != null)
            {
                helper.Pref = opt.Prefix;
            }

            if (opt.Svalue != null)
            {
                helper.Sval = opt.Svalue;
            }

            helper.HtmlAttributes(opt.FieldAttr, opt.InputAttr);
        }

        private static void setParentsParms<THelper>(IParentParamOpt opt, IAwesomeHelperParent<THelper> helper)
        {
            if (opt.Parents != null)
            {
                foreach (var parent in opt.Parents)
                {
                    helper.Parent(parent.Id, parent.Alias ?? "parent", parent.LoadOnChange ?? true, parent.Context ?? true);
                }
            }

            if (opt.Parameters != null)
            {
                foreach (var p in opt.Parameters)
                {
                    helper.Parameter(p.Name, p.Value);
                }
            }
        }
    }
}