using Omu.AwesomeMvc;
using System;
using System.Collections.Generic;

namespace Omu.Awem.Helpers
{
    /// <summary>
    /// Column mods config
    /// </summary>
    public class ColumnModCfg : IInlColBuilder, IEvalGridColumnCfg
    {
        internal readonly ColumnModTag Tag = new ColumnModTag();

        private readonly Column column;

        /// <summary>
        /// 
        /// </summary>
        public object EvalCfg(GridColumnEvalContext cx)
        {
            if (filterElmFuncs != null)
            {
                foreach (var ff in filterElmFuncs)
                {
                    Tag.Filters.Add(ff(cx));
                }
            }

            if (inlElmFuncs != null)
            {
                foreach (var ief in inlElmFuncs)
                {
                    Tag.Format.Add(ief(cx));
                }
            }

            if (freeFuncs != null)
            {
                foreach (var ff in freeFuncs)
                {
                    ff(cx);
                }
            }

            if (Tag.Autohide > 0)
            {
                if (!cx.Mods.Contains("awem.gridColAutohide"))
                {
                    cx.Mods.Add("awem.gridColAutohide");
                }
            }

            return Tag;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="column"></param>
        public ColumnModCfg(Column column)
        {
            this.column = column;
            Tag.Format = new List<InlElm>();
            Tag.Filters = new List<FilterInlElm>();
        }

        /// <returns></returns>
        /// <summary>
        /// hide column on small screens, or when resizing the browser (should be used with columns selector mod)
        /// </summary>
        /// <param name="order">hide order ( greater number - last to be hidden, 0 - no autohide, lesser number - first to be hidden ) </param>
        /// <returns></returns>
        public ColumnModCfg Autohide(int order = 1)
        {
            Tag.Autohide = order;
            return this;
        }

        /// <summary>
        /// disable autohide for this column, use when setting autohide for all columns using grid ColumnsAutohide grid mod
        /// </summary>
        /// <returns></returns>
        public ColumnModCfg NoAutohide()
        {
            Tag.Autohide = 0;
            return this;
        }

        /// <summary>
        /// use to disable hiding of the column from the columns selector, it will also disable autohiding
        /// </summary>
        /// <returns></returns>
        public ColumnModCfg Nohide()
        {
            Tag.Nohide = true;
            return this;
        }

        /// <summary>
        /// define a js function that will set the inline format of the column; the func will receive the grid row model as the first parameter,
        /// and an array of all the inline elements set for this column
        /// </summary>
        /// <param name="func">js func</param>
        /// <returns></returns>
        public ColumnModCfg InlineFunc(string func)
        {
            Tag.FormatFunc = func;
            return this;
        }

        /// <summary>
        /// column caption used in columns selector
        /// </summary>
        /// <param name="caption"></param>
        /// <returns></returns>
        public ColumnModCfg Caption(string caption)
        {
            Tag.Caption = caption;
            return this;
        }

        /// <summary>
        /// adds inline element
        /// </summary>
        /// <param name="el"></param>
        public ColumnModCfg AddInline(InlElm el)
        {
            //Tag.Format.Add(el);
            AddInlElmFunc(cx => el);
            return this;
        }

        /// <summary>
        /// add filter format
        /// </summary>
        /// <param name="el"></param>
        public ColumnModCfg AddFilter(FilterInlElm el)
        {
            Tag.Filters.Add(el);
            //AddFilterElmFunc(cx => el);
            return this;
        }

        private List<Func<GridColumnEvalContext, FilterInlElm>> filterElmFuncs;

        private List<Func<GridColumnEvalContext, InlElm>> inlElmFuncs;

        private List<Action<GridColumnEvalContext>> freeFuncs;

        /// <summary>
        /// 
        /// </summary>        
        public void AddGridFunc(Action<GridColumnEvalContext> func)
        {
            if (freeFuncs == null)
            {
                freeFuncs = new List<Action<GridColumnEvalContext>>();
            }

            freeFuncs.Add(func);
        }

        /// <summary>
        /// Add func that returns a grid FilterElm
        /// </summary>
        /// <param name="func"></param>
        /// <returns></returns>
        public ColumnModCfg AddFilterElmFunc(Func<GridColumnEvalContext, FilterInlElm> func)
        {
            if (filterElmFuncs == null)
            {
                filterElmFuncs = new List<Func<GridColumnEvalContext, FilterInlElm>>();
            }

            filterElmFuncs.Add(func);

            return this;
        }

        /// <summary>
        /// Add func that return a grid InlElm
        /// </summary>
        /// <param name="func"></param>
        /// <returns></returns>
        public ColumnModCfg AddInlElmFunc(Func<GridColumnEvalContext, InlElm> func)
        {
            if (inlElmFuncs == null)
            {
                inlElmFuncs = new List<Func<GridColumnEvalContext, InlElm>>();
            }

            inlElmFuncs.Add(func);

            return this;
        }

        /// <summary>
        /// 
        /// </summary>
        public Column Column { get { return column; } }
    }
}