using Omu.Awem.Utils;
using Omu.AwesomeMvc;
using System;

namespace Omu.Awem.Helpers
{
    /// <summary>
    /// 
    /// </summary>
    public static class GridFilterExtensions
    {

        private static string getName(Column column)
        {
            return column.Bind ?? Guid.NewGuid().ToString();
        }

        /// <summary>
        /// Awesome control filter
        /// </summary>
        public static Column Flt(this Column column, IAweValHelper helper, string valProp = null)
        {
            return column.Mod(o => o.Filter(helper, valProp));
        }

        /// <summary>
        /// Filter format
        /// </summary>
        public static Column FltFormat(
            this Column column,
            string format)
        {
            return column.Mod(o => o.AddFilter(new FilterInlElm { format = format }));
        }

        /// <summary>
        /// Grid column filter
        /// </summary>
        /// <param name="column">Grid column</param>
        /// <param name="valProp">Grid row model property name to get value for filter</param>
        /// <param name="type">filter type</param>
        /// <param name="clearBtn">clear button</param>
        /// <param name="format">html format</param>
        /// <param name="jsFunc">js Func</param>
        /// <param name="jsFuncArg"></param>
        /// <returns></returns>
        public static Column Flt(
            this Column column,
            string valProp = null,
            FilterType type = FilterType.Default,
            bool? clearBtn = null,
            string format = null,
            string jsFunc = null,
            string jsFuncArg = null)
        {
            return column.Mod(o => o.Filter(format, jsFunc, jsFuncArg, clearBtn, valProp, type));
        }

        private static string clrAllBtn(string id)
        {
            return
                "<div class='awe-field o-cmped o-op'>" +
                "<button id='" + id +
                "' type='button' class='awe-btn o-clrAll awe-clrbtn'><i class='awe-icon awe-icon-x'></i></button>" +
                "<script>awe.fbtn({ id: '" + id + "'});</script>" +
                "</div>";

        }

        /// <summary>
        /// Filter clear button
        /// </summary>        
        public static Column FltClearBtn(this Column column)
        {
            var name = getName(column);


            return column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return new FilterInlElm { format = clrAllBtn(name + "clrAll") };
                });
            });
        }

        /// <summary>
        /// DropdownList filter
        /// </summary>
        public static Column FltDropdownList(this Column column, DropdownListOpt opt = null)
        {
            opt = opt ?? new DropdownListOpt();
            var name = getName(column);

            opt.Name = opt.Name ?? name;
            opt.FilterData();

            if (opt.ClearBtn == null)
            {
                opt.ClearBtn = true;
            }

            column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().DropdownList(opt));
                });
            });

            return column;
        }

        /// <summary>
        /// Multiselect filter
        /// </summary>
        public static Column FltMultiselect(this Column column, MultiselectOpt opt = null)
        {
            opt = opt ?? new MultiselectOpt();
            var name = getName(column);

            opt.Name = opt.Name ?? name;
            opt.FilterData();

            column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().Multiselect(opt));
                });
            });

            return column;
        }

        /// <summary>
        /// Textbox string filter
        /// </summary>
        public static Column FltString(this Column column, TextBoxOpt opt = null)
        {
            opt = opt ?? new TextBoxOpt();
            var name = getName(column);

            opt.Name = opt.Name ?? name;

            if (opt.Placeholder == null)
            {
                opt.Placeholder = InternalUtils.SplitByCapLetter(name);
            }

            if (opt.ClearBtn == null)
            {
                opt.ClearBtn = true;
            }

            opt.EditorAttr = new { enterkeyhint = "search" };

            column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().TextBox(opt));
                });
                //.AddFilterElmFunc(cx => new FilterElm { format = clrAll });
            });

            return column;
        }

        /// <summary>
        /// Numeric textbox filter
        /// </summary>
        public static Column FltNumeric(this Column column, NumericInputOpt opt = null)
        {
            opt = opt ?? new NumericInputOpt();
            var name = getName(column);

            opt.Name = opt.Name ?? name;

            if (opt.Placeholder == null)
            {
                opt.Placeholder = InternalUtils.SplitByCapLetter(name);
            }

            if (opt.ClearBtn == null)
            {
                opt.ClearBtn = true;
            }

            column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().NumericInput(opt));
                });
            });

            return column;
        }

        /// <summary>
        /// Numeric textbox filter
        /// </summary>
        public static Column FltNumericOp(this Column column, NumericInputOpt opt = null)
        {
            opt = opt ?? new NumericInputOpt();
            var name = getName(column);

            opt.Name = opt.Name ?? name;

            if (opt.Placeholder == null)
            {
                opt.Placeholder = InternalUtils.SplitByCapLetter(name);
            }

            if (opt.EditorAttr == null)
            {
                opt.EditorAttr = new { enterkeyhint = "search" };
            }

            column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().NumericInput(opt));
                })
                .AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().DropdownList(new DropdownListOpt { Name = name + "Op", CssClass = "o-op", DataFunc = "awem.opData" }));
                })
                .AddFilterElmFunc(cx => new FilterInlElm { format = clrAllBtn(name + "clrAll") });
            });

            return column;
        }

        /// <summary>
        /// DateOp filter
        /// </summary>
        public static Column FltDateOp(this Column column)
        {
            var name = getName(column);

            column.Mod(columnCfg =>
            {
                columnCfg.AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().DatePicker(new DatePickerOpt
                    {
                        Name = name,
                        Placeholder = name,
                        InputAttr = new { enterkeyhint = "search" }
                    }));
                })
                .AddFilterElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().DropdownList(new DropdownListOpt() { Name = name + "Op", CssClass = "o-op", DataFunc = "awem.opData" }));
                })
                .AddFilterElmFunc(cx => new FilterInlElm { format = clrAllBtn(name + "clrAll") });
            });

            return column;
        }
    }
}