using System;
using System.Collections.Generic;
using System.Linq;
using AwesomeMvcDemo.Data;
using AwesomeMvcDemo.Models;
using System.Web.Mvc;

using Omu.AwesomeMvc;

namespace AwesomeMvcDemo.Controllers.Awesome.MultiLookup
{
    public class MealsCustomSearchMultiLookupController : Controller
    {
        // when CustomSearch(true) this action will be rendered instead of the default search
        public ActionResult SearchForm()
        {
            return PartialView();
        }

        public ActionResult GetItems(IEnumerable<int> v)
        {
            var items = new List<Meal>();

            if (v != null)
            {
                items = Db.Meals
                    .Where(o => v.Contains(o.Id)).ToList();
            }

            return Json(items.Select(meal => new KeyContent(meal.Id, meal.Name)));
        }

        public ActionResult Search(int[] selected, int[] categories, int page, string search = "")
        {
            selected = selected ?? new int[] { };
            categories = categories ?? new int[] { };

            var query = Db.Meals
                .Where(o => o.Name.Contains(search)
                        && !selected.Contains(o.Id))
                .AsQueryable();

            if (categories.Any())
            {
                query = query.Where(o => categories.Contains(o.Category.Id));
            }

            var res = AweUtil.BuildList(query, page, keyContentFunc: o => new KeyContent(o.Id, o.Name));

            return Json(res);
        }

        public ActionResult Selected(int[] selected)
        {
            var items = new List<Meal>();

            if (selected != null)
            {
                items = Db.Meals
                    .Where(o => selected.Contains(o.Id)).ToList();
            }

            return Json(new AjaxListResult
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name))
            });
        }
    }
}