using System;
using System.Collections.Generic;
using System.Linq;
using AwesomeMvcDemo.Data;
using AwesomeMvcDemo.ViewModels;
using System.Web.Mvc;


using Omu.Awem.Utils;
using Omu.AwesomeMvc;

namespace AwesomeMvcDemo.Controllers.Demos.Grid
{
    public class GridArrayDataSourceController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult GridGetData(GridParams g, FilterPrm fprm, bool? init)
        {
            var initData = getLunchesTwoDimensArrayData(); // two-dimensional array data, 1st row has the header            
            // we can sort and page the data before passing it to the GridModelBuilder            
            // using gmb.GetPage, gmb.OrderBy (example CustomQueryingController.GetEfAsyncItems)
            // or manually making use of g.Page, g.PageSize, g.SortNames, g.SortDirections

            var header = initData[0];

            var query = initData.Skip(1); // skip header

            // Note: in the view (GridCfg.cshml) Column.Bind is set to the data's column index value
            // below we usually use Convert.ToInt32(bind) to get the column index

            #region filter data
            if (fprm != null && fprm.Forder != null)
            {
                foreach (var bind in fprm.Forder)
                {
                    var index = Convert.ToInt32(bind);
                    var fltVal = Request.Form[bind];
                    var name = header[index].ToString();

                    if (name == "Price")
                    {
                        var intval = Convert.ToInt32(fltVal);
                        query = query.Where(row => object.Equals(row[index], intval));
                    }
                    else
                    {
                        query = query.Where(row => row[index].ToString().ToLower().Contains(fltVal.ToString().ToLower()));
                    }
                }
            }
            #endregion

            // transform data to GridArrayRow[]
            var data = query.ToArray();
            var items = new List<GridArrayRow>();
            for (var i = 0; i < data.Length; i++)
            {
                items.Add(new GridArrayRow { Id = data[i][0], Values = data[i] });
            }

            // build grid model
            var gmb = new GridModelBuilder<GridArrayRow>(items.AsQueryable(), g);

            // define get col value by index, used for grouping
            gmb.GetColValFunc = (bind, row) =>
            {
                var index = Convert.ToInt32(bind);
                return new[] { row.Values[index] };
            };

            // define orderby for array, used by sorting, grouping
            gmb.OrderByFunc = ArrayRowOrderBy;

            var model = gmb.BuildModel();

            // update grid cfg only on first request, GridCfg helper has .Parameter("init", true)
            // the helper in Index.cshtml doesn't have init param, so it will enter here 1st time
            if (init != true)
            {
                model.UpdateCfg = this.RenderPartialView("GridCfg", header);
            }

            return Json(model.ToDto());
        }

        public ActionResult SelGridGetData(GridParams g, FilterPrm fprm, string dataType, string cfgDataType)
        {
            var initData = new object[][] { new object[] { } };
            if (dataType == "Meals") initData = getMealsTwoDimensArrayData();
            if (dataType == "Lunches") initData = getLunchesTwoDimensArrayData();
            if (dataType == "Categories") initData = getCategoriesTwoDimensArrayData();

            var headers = initData[0].Select(o => o.ToString()).ToArray();

            var query = initData.Skip(1); // skip header

            #region filter data
            if (dataType != cfgDataType)
            {
                fprm = null; // ignore previous filters when changing data type
            }

            if (fprm != null && fprm.Forder != null)
            {
                foreach (var bind in fprm.Forder)
                {
                    var index = Convert.ToInt32(bind);
                    var fltVal = Request.Form[bind];
                    var name = headers[index].ToString();

                    if (name == "Price")
                    {
                        var intval = Convert.ToInt32(fltVal);
                        query = query.Where(row => object.Equals(row[index], intval));
                    }
                    else
                    {
                        query = query.Where(row => row[index].ToString().ToLower().Contains(fltVal.ToString().ToLower()));
                    }
                }
            }
            #endregion

            // transform data to GridArrayRow[]
            var data = query.ToArray();
            var items = new List<GridArrayRow>();
            for (var i = 0; i < data.Length; i++)
            {
                items.Add(new GridArrayRow { Id = data[i][0], Values = data[i] });
            }

            // build grid model
            var gmb = new GridModelBuilder<GridArrayRow>(items.AsQueryable(), g);

            // define get col value by index, used for grouping
            gmb.GetColValFunc = (bind, row) =>
            {
                var index = Convert.ToInt32(bind);
                return new[] { row.Values[index] };
            };

            // define orderby for array, used by sorting, grouping
            gmb.OrderByFunc = ArrayRowOrderBy;

            var model = gmb.BuildModel();

            // when dataType dropdown value changed update the grid
            if (cfgDataType != dataType)
            {
                model.UpdateCfg = this.RenderPartialView("SelGridCfg", new SelGridCfg { Headers = headers, DataType = dataType });
            }

            return Json(model.ToDto());
        }

        #region get array data
        /// <summary>
        /// get lunches sample data
        /// </summary>        
        private object[][] getLunchesTwoDimensArrayData()
        {
            var initData = new List<object[]>
                {
                    new[] { "Id", "Name", "Meal", "Price" }
                };

            foreach (var lunch in Db.Lunches.Take(100))
            {
                initData.Add(new object[] { lunch.Id, lunch.Person, lunch.Food.Name, lunch.Price });
            }

            return initData.ToArray();
        }

        /// <summary>
        /// get meals sample data
        /// </summary>        
        private object[][] getMealsTwoDimensArrayData()
        {
            var initData = new List<object[]>
                {
                    new[] { "Id", "Name", "Category", "Description" }
                };

            foreach (var m in Db.Meals)
            {
                initData.Add(new object[] { m.Id, m.Name, m.Category.Name, m.Description });
            }

            return initData.ToArray();
        }

        /// <summary>
        /// get meals sample data
        /// </summary>        
        private object[][] getCategoriesTwoDimensArrayData()
        {
            var initData = new List<object[]>
                {
                    new[] { "Id", "Name" }
                };

            foreach (var o in Db.Categories)
            {
                initData.Add(new object[] { o.Id, o.Name });
            }

            return initData.ToArray();
        }
        #endregion

        private IQueryable<GridArrayRow> ArrayRowOrderBy(IQueryable<GridArrayRow> query, string[][] rules)
        {
            const string ASC = "asc";
            IOrderedQueryable<GridArrayRow> resq = null;

            for (int i = 0; i < rules.Length; i++)
            {
                string[] rule = rules[i];

                var index = Convert.ToInt32(rule[0]);
                var isAsc = rule[1] == ASC;

                if (i == 0)
                {
                    if (isAsc)
                    {
                        resq = query.OrderBy(row => row.Values[index]);
                    }
                    else
                    {
                        resq = query.OrderByDescending(row => row.Values[index]);
                    }
                }
                else
                {
                    if (isAsc)
                    {
                        resq = resq.ThenBy(row => row.Values[index]);
                    }
                    else
                    {
                        resq = resq.ThenByDescending(row => row.Values[index]);
                    }
                }
            }

            return resq ?? query;
        }
    }

    public class SelGridCfg
    {
        public string[] Headers { get; set; }

        public string DataType { get; set; }
    }
}