using System;
using System.Linq;
using AwesomeMvcDemo.Data;
using AwesomeMvcDemo.Models;
using AwesomeMvcDemo.Utils.Awesome;
using AwesomeMvcDemo.ViewModels.Input;
using System.Web.Mvc;

using Omu.AwesomeMvc;

namespace AwesomeMvcDemo.Controllers.Demos.Misc
{
    public class CrudInLookupController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Index(LookupCrudDemoInput input)
        {
            return View(input);
        }

        public ActionResult DinnerLookup()
        {
            return PartialView();
        }

        public ActionResult DinnerMultiLookup()
        {
            return PartialView();
        }

        private static object MapToGridModel(Dinner o)
        {
            return
                new
                {
                    o.Id,
                    o.Name,
                    Date = o.Date.ToShortDateString(),
                    ChefName = o.Chef.FullName,
                    Meals = string.Join(", ", o.Meals.Select(m => m.Name))
                };
        }

        public ActionResult DinnersGridSearch(GridParams g, int[] selected, string search)
        {
            selected = selected ?? new int[] { };

            var query = Db.Dinners
                .Where(o => !selected.Contains(o.Id))
                .AsQueryable();

            query = query.Filter(search, o => o.Name);

            var gmb = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel
            };

            return Json(gmb.Build());
        }

        public ActionResult DinnersGridSelected(GridParams g, int[] selected)
        {
            selected = selected ?? new int[] { };

            var query = Db.Dinners
                .Where(o => selected.Contains(o.Id))
                .AsQueryable();

            var gmb = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel
            };

            return Json(gmb.Build());
        }
    }
}