using AwesomeMvcDemo.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace AwesomeMvcDemo.Data
{
    /// <summary>
    /// mock Db context
    /// </summary>
    public class MyContext : IDisposable
    {
        public IQueryable<Category> Categories { get { return Db.Categories.AsQueryable(); } }
        public IQueryable<Meal> Meals { get { return Db.Meals.AsQueryable(); } }
        public IQueryable<Chef> Chefs { get { return Db.Chefs.AsQueryable(); } }
        public IQueryable<TreeNode> TreeNodes { get { return Db.TreeNodes.AsQueryable(); } }
        public IQueryable<Dinner> Dinners { get { return Db.Dinners.AsQueryable(); } }
        public IQueryable<Lunch> Lunches { get { return Db.Lunches.AsQueryable(); } }
        public IQueryable<Country> Countries { get { return Db.Countries.AsQueryable(); } }
        public IQueryable<Food> Foods { get { return Db.Foods.AsQueryable(); } }
        public IQueryable<Message> Messages { get { return Db.Messages.AsQueryable(); } }
        public IQueryable<ParentMeal> ParentMeals { get { return Db.ParentMeals.AsQueryable(); } }
        public IQueryable<Meeting> Meetings { get { return Db.Meetings.AsQueryable(); } }
        public IQueryable<Restaurant> Restaurants { get { return Db.Restaurants.AsQueryable(); } }
        public IQueryable<RestaurantAddress> RestaurantAddresses { get { return Db.RestaurantAddresses.AsQueryable(); } }
        public IQueryable<Spreadsheet> Spreadsheets { get { return Db.Spreadsheets.AsQueryable(); } }

        public void Dispose()
        {
            // added so we could use using
        }

        public Task<T> FindAsync<T>(object key) where T : Entity
        {
            return Task.FromResult(Db.Get<T>((int)key));
        }

        public void Add<T>(T entity) where T : Entity
        {
            Db.Add(entity);
        }

        public Task SaveChangesAsync()
        {
            return Task.FromResult(0);
        }

        public void Remove<T>(T entity) where T : Entity
        {
            Db.Remove(entity);
        }
    }

    /// <summary>
    /// extensions used to imitate EF code 
    /// (used in ASP.net MVC 5 demos)
    /// </summary>
    public static class MockEfExtensions
    {
        public static Task<decimal> AverageAsync<T>(this IQueryable<T> items, Expression<Func<T, decimal>> selector)
        {
            return Task.FromResult(items.Average(selector));
        }

        public static IQueryable<T> Include<T>(this IEnumerable<T> items, Expression<Func<T, object>> _)
        {
            return items.AsQueryable();
        }

        public static Task<T> FirstOrDefaultAsync<T>(this IQueryable<T> items, Expression<Func<T, bool>> predicate)
        {
            var res = items.FirstOrDefault(predicate);

            return Task.FromResult(res);
        }

        public static Task<T> SingleAsync<T>(this IQueryable<T> items, Expression<Func<T, bool>> predicate)
        {
            var res = items.Single(predicate);

            return Task.FromResult(res);
        }

        public static Task<T> FirstAsync<T>(this IQueryable<T> items, Expression<Func<T, bool>> predicate = null)
        {
            var res = predicate == null ? items.First() : items.First(predicate);

            return Task.FromResult(res);
        }

        public static Task<T[]> ToArrayAsync<T>(this IEnumerable<T> items)
        {
            return Task.FromResult(items.ToArray());
        }

        public static Task<List<T>> ToListAsync<T>(this IEnumerable<T> items)
        {
            return Task.FromResult(items.ToList());
        }

        public static Task<int> CountAsync<T>(this IQueryable<T> items)
        {
            return Task.FromResult(items.Count());
        }
    }
}