(function ($, ovld) {
    const defaults = { autorun: 1 };
    const attachedFlag = 'ovldunobs';

    function bind({ selector } = { selector: 'form' }) {
        $(selector).each(function () {
            const form = $(this);
            if (form.data(attachedFlag)) return;
            form.data(attachedFlag, 1);

            const api = ovld.bind({
                subev: 'submit',
                selector,
                getRules: () => getRules(form),
                msgCont: function ({ input, name }) {
                    return $(input).closest(selector).find(`[data-valmsg-for="${name}"]`);
                }
            });

            var baseDestroy = api.destroy;
            
            api.destroy = function () {
                baseDestroy();
                $(selector).each(function () {
                    $(this).data(attachedFlag, null);
                });
            }

            return api;
        });
    } 

    function getRules(form) {
        const rules = {};

        form = $(form);

        function addRule(key, rule) {
            if (!rules[key]) rules[key] = [];
            rules[key].push(rule);
        }

        var dval = 'data-val-';
        var dmaxlen = `${dval}maxlength`;
        var dminlen = `${dval}minlength`;
        var dmaxlenmax = dmaxlen + '-max';
        var dminlenmin = dminlen + '-min';

        form.find(`[${dmaxlenmax}]`).each(function () {
            var max = $(this).attr(dmaxlenmax);
            addRule($(this).attr('name'), { chk: ovld.rules.maxlen(max), msg: $(this).attr(dmaxlen) });
        });

        form.find(`[${dminlenmin}]`).each(function () {
            var min = $(this).attr(dminlenmin);
            addRule($(this).attr('name'), { chk: ovld.rules.minlen(min), msg: $(this).attr(dminlen) });
        });

        form.find(`[${dval}length]`).each(function () {
            var inp = $(this);
            var min = inp.attr(`${dval}length-min`);
            var max = inp.attr(`${dval}length-max`);
            addRule(
                inp.attr('name'),
                {
                    chk: ovld.rules.len({ min, max }),
                    msg: inp.attr(`${dval}length`)
                });
        });

        form.find(`[${dval}required]`).each(function () {
            var name = $(this).attr('name');
            addRule(name, { chk: ovld.rules.req, msg: $(this).attr(`${dval}required`) });
        });

        return rules;
    }

    function validate({ cont }) {        
        return ovld.validate({ cont, rules: getRules(cont) });
    }

    function setDefaults(opt) {
        defaults = Object.assign({}, defaults, opt);
    }

    // doc ready
    $(function () {        
        if (defaults.autorun) {
            bind();
        }        
    });

    const unobtrusive = {
        bind,
        setDefaults,
        validate
    };

    ovld.unobtrusive = unobtrusive;
}(jQuery, ovld));