using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web.Mvc;
using AwesomeWebFormsDemo.Data;
using AwesomeWebFormsDemo.Models;
using AwesomeWebFormsDemo.Utils;
using AwesomeWebFormsDemo.ViewModels.Display;
using NPOI.SS.Formula.Functions;
using Omu.AwemWebForms.Helpers;
using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Controllers.Awesome
{
    public class DataController : Controller
    {
        public ActionResult GetFoods()
        {
            var items = Db.Foods;

            var lst = new List<MealDisplay> { new MealDisplay("", "any", "pasta.png") };
            lst.AddRange(items.Select(o => new MealDisplay(o.Id, o.Name, o.Pic)));

            return Json(lst);
        }

        /*begin*/
        public ActionResult GetCategories()
        {
            return Json(Db.Categories.Select(o => new KeyContent(o.Id, o.Name)));
        }

        public ActionResult GetMeals(int[] categories)
        {
            categories = categories ?? new int[] { };
            var items = Db.Meals.Where(o => categories.Contains(o.Category.Id))
                .Select(o => new KeyContent(o.Id, o.Name));

            return Json(items);
        }
        /*end*/

        public ActionResult GetAllMeals()
        {
            var items = Db.Meals.Select(o => new KeyContent(o.Id, o.Name));
            return Json(items);
        }

        public ActionResult GetChefs()
        {
            var items = Db.Chefs.Select(o => new KeyContent(o.Id, o.FirstName + " " + o.LastName));
            return Json(items);
        }

        /*beginamg*/
        public ActionResult GetAllMealsGrouped()
        {
            var res = new List<Oitem>();
            var groups = Db.Meals.GroupBy(o => o.Category);
            foreach (var g in groups)
            {
                res.Add(new Oitem
                {
                    Content = g.Key.Name,
                    Class = "o-itm o-itmg",
                    NonVal = true,
                    Items = g.Select(meal => new Oitem(meal.Id, meal.Name))
                });
            }

            return Json(res);
        }
        /*endamg*/

        /*beginsbm*/
        public ActionResult GetMealsTreeImg()
        {
            var url = Url.Content(DemoUtils.MealsUrl);

            var res = new List<MealOitem>();
            var groups = Db.Meals.GroupBy(o => o.Category);
            foreach (var g in groups)
            {
                res.Add(new MealOitem
                {
                    Content = g.Key.Name,
                    Class = "o-itm",
                    NonVal = true,
                    Items = g.Select(meal => new MealOitem { Key = meal.Id, Content = meal.Name, url = url + meal.Name + ".jpg" })
                });
            }

            return Json(res);
        }
        /*endsbm*/

        /*begintree*/
        public ActionResult GetTreeNodesLazy(int? key)
        {
            var nodes = key.HasValue ?
                GetNodes(Db.TreeNodes.Where(o => o.Id == key)).Take(10) :
                GetNodes(Db.TreeNodes.Where(o => o.Parent == null).Take(10), lazy: true);

            return Json(nodes);
        }

        public ActionResult SearchTree(string term)
        {
            term = (term ?? string.Empty).ToLower();
            var result = Db.TreeNodes.Where(o => o.Name.ToLower().Contains(term)).Take(3).ToList();
            var loopres = result.ToList();

            foreach (var treeNode in loopres)
            {
                AddParentsTo(result, treeNode);
            }

            var roots = result.Where(o => o.Parent == null).ToList();

            var nodes = GetNodes(roots);
            return Json(nodes);
        }

        private IEnumerable<Oitem> GetNodes(IEnumerable<TreeNode> nodes, int lvl = 0, bool lazy = false)
        {
            var list = new List<Oitem>();
            foreach (var node in nodes)
            {
                var oitem = new Oitem { Key = node.Id.ToString(), Content = node.Name };
                list.Add(oitem);
                var children = Db.TreeNodes.Where(o => o.Parent == node);
                if (children.Any())
                {
                    if (lazy && lvl > 0)
                    {
                        oitem.Lazy = true;
                    }
                    else
                    {
                        oitem.Items = GetNodes(Db.TreeNodes.Where(o => o.Parent == node), lvl + 1, lazy);
                    }
                }
            }

            return list;
        }

        private void AddParentsTo(ICollection<TreeNode> result, TreeNode node)
        {
            if (node.Parent != null)
            {
                if (!result.Contains(node.Parent))
                {
                    result.Add(node.Parent);
                    AddParentsTo(result, node.Parent);
                }
            }
        }
        /*endtree*/

        /*beginrso*/
        public ActionResult GetMealsInit(int? v)
        {
            var items = Db.Meals.Take(3).ToList();
            var selected = Db.Meals.SingleOrDefault(o => o.Id == v);

            if (selected != null && !items.Contains(selected))
            {
                items.Add(selected);
            }

            return Json(items.Select(o => new KeyContent(o.Id, o.Name)));
        }

        public ActionResult SearchMeals(string term = "")
        {
            var items = Db.Meals
                .Where(o => o.Name.IndexOf(term, StringComparison.OrdinalIgnoreCase) >= 0)
                .Take(10)
                .Select(o => new KeyContent(o.Id, o.Name));

            return Json(items);
        }
        /*endrso*/

        /*beginimg*/
        public ActionResult GetMealsImg()
        {
            var url = Url.Content(DemoUtils.MealsUrl);
            var items = Db.Meals
                .Select(o => new MealDisplay(o.Id, o.Name, url + o.Name + ".jpg"));
            return Json(items);
        }
        /*endimg*/

        /*beginaimg*/
        public ActionResult GetMealsImgAutoc(string v)
        {
            v = (v ?? string.Empty).ToLower();
            var url = Url.Content(DemoUtils.MealsUrl);
            var items = Db.Meals
                .Where(o => o.Name.ToLower().Contains(v))
                .Select(o => new MealDisplay(o.Id, o.Name, url + o.Name + ".jpg"));

            return Json(items);
        }
        /*endaimg*/

        public ActionResult GetMealsGroupedImg()
        {
            var url = Url.Content(DemoUtils.MealsUrl);

            var res = new List<MealOitem>();
            var groups = Db.Meals.GroupBy(o => o.Category);
            foreach (var g in groups)
            {
                res.Add(new MealOitem
                {
                    Content = g.Key.Name,
                    Class = "o-itm o-itmg",
                    NonVal = true,
                    Items = g.Select(meal => new MealOitem { Key = meal.Id, Content = meal.Name, url = url + meal.Name + ".jpg" })
                });
            }

            return Json(res);
        }        

        public ActionResult GetCountries()
        {
            var items = new List<KeyContent> { new KeyContent(string.Empty, "any country") };
            items.AddRange(Db.Countries.Select(o => new KeyContent(o.Id, o.Name)));

            return Json(items);
        }

        /*begingmsv*/
        public ActionResult GetMealsSetValue(int[] categories)
        {
            categories = categories ?? new int[] { };

            var items = Db.Meals.Where(o => categories.Contains(o.Category.Id)).ToList();

            object value = null;
            if (items.Any())
            {
                value = new[] { items.Skip(1).First().Id };
            }

            return Json(new AweItems
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name)),
                Value = value
            });
        }
        /*endgmsv*/

        /*beginsv*/
        public ActionResult GetMealsSetValue2(int[] categories)
        {
            categories = categories ?? new int[] { };

            var items = Db.Meals.Where(o => categories.Contains(o.Category.Id)).ToList();

            object value = null;
            if (items.Any())
            {
                value = items.Skip(1).First().Id;
            }

            return Json(new AweItems
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name)),
                Value = value
            });
        }

        public ActionResult GetOrgSetValue()
        {
            var items = Db.Organisations.ToList();

            object value = null;
            if (items.Any())
            {
                value = items.Skip(2).First().Id;
            }

            return Json(new AweItems
            {
                Items = items.Select(o => new KeyContent(o.Id, o.Name)),
                Value = value
            });
        }
        /*endsv*/

        /*beginmlc*/
        public ActionResult GetNumbers(int[] parent)
        {
            parent = parent ?? new int[] { };

            var items = new[] { 3, 7, 10 };

            return Json(items.Select(o => o + parent.Sum()).Select(o => new KeyContent(o, o.ToString(CultureInfo.InvariantCulture))));
        }

        public ActionResult GetWords(string parent)
        {
            var items = new[] { "The", "brown", "cow", "is eating", "green", "grass" };

            return Json(items.Select(o => parent + " " + o).Select(o => new KeyContent(o, o)));
        }
        /*endmlc*/

        public ActionResult GetCategoriesFirstOption()
        {
            var list = new List<KeyContent> { new KeyContent("", "please select") };
            list.AddRange(Db.Categories.Select(o => new KeyContent(o.Id, o.Name)));
            return Json(list);
        }

        public ActionResult GetMealsList()
        {
            return Json(Db.Meals);
        }

        public ActionResult MealsGrid(GridParams g, string name)
        {
            name = (name ?? string.Empty).ToLower();
            var query = Db.Meals.Where(o => o.Name.ToLower().Contains(name)).AsQueryable();
            // there's no need to call tolower when using a real db

            return Json(new GridModelBuilder<Meal>(query, g) { Key = "Id" }.Build());
        }

        public ActionResult MealsGridSearch(GridParams g, int[] selected, string name)
        {
            name = (name ?? string.Empty).ToLower();
            selected = selected ?? new int[] { };

            var query = Db.Meals.Where(o => o.Name.ToLower().Contains(name) && !selected.Contains(o.Id)).AsQueryable();
            // there's no need to call tolower when using a real db

            return Json(new GridModelBuilder<Meal>(query, g) { Key = "Id" }.Build());
        }

        public ActionResult MealsGridSelected(GridParams g, int[] selected)
        {
            var sel = (selected ?? new int[] { }).ToList();
            var items = Db.Meals.Where(o => sel.Contains(o.Id)).ToList()
                                .OrderBy(o => sel.IndexOf(o.Id));

            return Json(new GridModelBuilder<Meal>(items.AsQueryable(), g) { Key = "Id", DefaultKeySort = Sort.None }.Build());
        }

        public ActionResult GetDinnerItem(int? v)
        {
            Check.NotNull(v, "v");

            var o = Db.Dinners.Single(f => f.Id == v);

            return Json(new KeyContent(o.Id, o.Name));
        }

        public ActionResult GetDinnerItems(int[] v)
        {
            v = v ?? new int[] { };
            return Json(Db.Dinners.Where(o => v.Contains(o.Id))
                .Select(o => new KeyContent(o.Id, o.Name)));
        }

        public ActionResult GetMenuNodes()
        {
            var root = Url.Content("~/");

            var menuNodes =
                MySiteMap.GetAll().Select(
                    o =>
                        new
                        {
                            o.Id,
                            o.Name,
                            o.Keywords,
                            ParentId = o.Parent?.Id ?? 0,
                            Url = o.Page != null ? root + o.Page + o.Anchor : null,
                            o.Collapsed,
                            o.NoMenu,
                            Action = o.Page
                        });

            return Json(menuNodes);
        }

        public ActionResult LunchGrid(GridParams g)
        {
            var list = Db.Lunches.AsQueryable();

            return Json(new GridModelBuilder<Lunch>(list, g)
            {
                Key = "Id", // needed for Entity Framework | nesting | tree | api
                Map = o => new
                {
                    o.Id,
                    o.Person,
                    FoodName = o.Food.Name,
                    o.Location,
                    o.Price,
                    Date = o.Date.ToShortDateString(),
                    CountryName = o.Country.Name,
                    ChefFName = o.Chef.FirstName,
                    ChefLName = o.Chef.LastName
                }
            }.Build());
        }

        /*begincasg*/
        public ActionResult CategoriesGrid(GridParams g)
        {
            return Json(new GridModelBuilder<Category>(Db.Categories.AsQueryable(), g)
            {
                Key = "Id"
            }.Build());
        }

        public ActionResult ChildMealsGrid(GridParams g, int[] categories)
        {
            categories = categories ?? new int[] { };

            return Json(new GridModelBuilder<Meal>(Db.Meals
                .Where(o => categories.Contains(o.Category.Id)).AsQueryable(), g).Build());
        }
        /*endcasg*/

        public ActionResult GetLunches()
        {
            return Json(Db.Lunches.Take(200).Select(o => new
            {
                o.Id,
                o.Person,
                FoodName = o.Food.Name,
                o.Location,
                CountryName = o.Country.Name,
                ChefName = o.Chef.FirstName + " " + o.Chef.LastName,
                o.Date,
                DateStr = o.Date.ToShortDateString()
            }));
        }

        /*beginenum*/
        public ActionResult GetWeatherEnumItems()
        {
            var type = typeof(WeatherType);
            var items = Enum.GetValues(type).Cast<int>().Select(o => new KeyContent(o, SplitByCapLetter(Enum.GetName(type, o)))).ToList();

            // remove if not needed or if used with odropdown/ajaxradiolist
            items.Insert(0, new KeyContent(string.Empty, "please select"));

            return Json(items);
        }

        /// <summary>
        /// from HiThere to Hi There
        /// </summary>
        private string SplitByCapLetter(string s)
        {
            var r = new Regex(@"
                (?<=[A-Z])(?=[A-Z][a-z]) |
                 (?<=[^A-Z])(?=[A-Z]) |
                 (?<=[A-Za-z])(?=[^A-Za-z])", RegexOptions.IgnorePatternWhitespace);

            return r.Replace(s, " ");
        }
        /*endenum*/

        public ActionResult Rend(int v)
        {
            var views = new[]
            {
                "GridFilterRowServerData",                
                "GridInlineCrud",
                "GridInlineEditParent",
                "GridCrud",
                "CardsAndItems",
                "DropmenuItems",
                "AutocInTxta",
                "Cascade",
                "GridCardsView",
                "GridInlineCrudCustomRender",
                "LookupCrud"
            };

            if (!Request.IsAjaxRequest()) throw new AwesomeException("ajax only");

            if (views.Length <= v) return Content("end");

            return PartialView("Demos/" + views[v]);
        }
    }
}