using System;
using System.Linq;
using AwesomeWebFormsDemo.Data;
using AwesomeWebFormsDemo.Models;
using AwesomeWebFormsDemo.ViewModels.Input;
using System.Web.Mvc;


using Omu.AwemWebForms.Utils;
using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Controllers.Demos.Grid
{
    public class GridInlineEditDemoController : Controller
    {
        private object MapToGridModel(Dinner o)
        {
            return new
            {
                o.Id,
                o.Name,
                Date = o.Date.ToShortDateString(),
                ChefName = o.Chef.FullName,
                Meals = string.Join(", ", o.Meals.Select(m => m.Name)),
                BonusMeal = o.BonusMeal.Name,
                o.Organic,
                DispOrganic = o.Organic ? "Yes" : "No",

                // below properties used for inline editing only
                MealsIds = o.Meals.Select(m => m.Id).ToArray(),
                ChefId = o.Chef.Id,
                BonusMealId = o.BonusMeal.Id,

                // for conditional demo
                Editable = o.Meals.Count() > 1,
                DateReadOnly = o.Date.Year < 2015
            };
        }

        public ActionResult GridGetItems(GridParams g, string search = "")
        {
            var query = Db.Dinners.Where(o => o.Name.Contains(search)).AsQueryable();

            var model = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel,
            };

            return Json(model.Build());            
        }

        [HttpPost]
        public ActionResult Save(DinnerInput input)
        {
            // custom validation example
            if (ModelState.IsValid && input.Name.Contains("asdf"))
            {
                ModelState.AddModelError("Name", "Name can't contain asdf");
            }

            if (ModelState.IsValid)
            {
                var isCreate = !input.Id.HasValue;

                var ent = isCreate ? new Dinner() :
                    Db.Dinners.FirstOrDefault(o => o.Id == input.Id);

                if (ent == null)
                {
                    throw new Exception("Item doesn't exist anymore, id:" + input.Id);
                }

                ent.Name = input.Name;
                ent.Date = input.Date.Value;
                ent.Chef = Db.Find<Chef>(input.ChefId);

                // ToList req for EF
                ent.Meals = Db.Meals
                    .Where(o => input.MealsIds.Contains(o.Id)).ToList();

                ent.BonusMeal = Db.Find<Meal>(input.BonusMealId);
                ent.Organic = input.Organic ?? false;

                if (isCreate)
                {
                    Db.Add(ent);
                }

                return Json(new { Item = MapToGridModel(ent) });
            }

            return Json(ModelState.GetErrorsInline());
        }

        public ActionResult Delete(int id)
        {
            var dinner = Db.Find<Dinner>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "dinner",
                Name = dinner.Name
            });
        }

        [HttpPost]
        public ActionResult Delete(DeleteConfirmInput input)
        {
            Db.Remove(Db.Find<Dinner>(input.Id));

            // the PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }

        public ActionResult Index()
        {
            return View();
        }

        public ActionResult ConditionalDemo()
        {
            return View();
        }

        public ActionResult MultiEditorsDemo()
        {
            return View();
        }

        public ActionResult ClientSave()
        {
            return View();
        }

        public ActionResult AlwaysEdit()
        {
            return View();
        }
    }
}