using System;
using System.Linq;
using AwesomeWebFormsDemo.Data;
using AwesomeWebFormsDemo.Models;
using AwesomeWebFormsDemo.ViewModels.Input;
using System.Web.Mvc;


using Omu.AwemWebForms.Utils;
using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Controllers.Demos.Grid
{
    public class GridInlineEditParentController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(ParentMeal o)
        {
            return new
            {
                o.Id,

                // for display
                CategoriesDisplay = string.Join(", ", o.Categories.Select(c => c.Name)),
                MealName = o.Meal.Name,
                Meals1Names = string.Join(", ", o.Meals1.Select(m => m.Name)),

                // for inline editing helpers to get value
                Categories = o.Categories.Select(c => c.Id),
                MealId = o.Meal.Id,
                Meals1Ids = o.Meals1.Select(m => m.Id).ToArray()
            };
        }

        public ActionResult CategoryContent(int[] category, string rowKey)
        {
            category = category ?? new int[] { };
            var meals = Db.Meals.Where(o => category.Contains(o.Category.Id)).ToList();

            ViewData["rowKey"] = rowKey;

            return PartialView(meals);
        }

        public ActionResult GridGetItems(GridParams g)
        {
            var query = Db.ParentMeals.AsQueryable();

            var gmb = new GridModelBuilder<ParentMeal>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,
            };

            return Json(gmb.Build());
        }

        [HttpPost]
        public ActionResult Save(ParentMealInput input)
        {
            if (ModelState.IsValid)
            {
                var isCreate = !input.Id.HasValue;

                var ent = isCreate ? new ParentMeal() :
                    Db.ParentMeals.First(o => o.Id == input.Id);

                ent.Categories = Db.Categories
                    .Where(o => input.Categories.Contains(o.Id)).ToList();

                ent.Meal = Db.Find<Meal>(input.MealId);

                // ToList req for EF
                ent.Meals1 = Db.Meals
                    .Where(o => input.Meals1Ids.Contains(o.Id)).ToList();

                if (isCreate)
                {
                    Db.Add(ent);
                }

                return Json(new { });
            }

            return Json(ModelState.GetErrorsInline());
        }

        public ActionResult Delete(int id)
        {
            // check for presence
            Db.Find<ParentMeal>(id);

            return PartialView(new DeleteConfirmInput
            {                
                Id = id
            });
        }

        [HttpPost]
        public ActionResult Delete(DeleteConfirmInput input)
        {
            Db.Remove(Db.Find<ParentMeal>(input.Id));

            return Json(new { input.Id });
        }
    }
}