using System;
using System.Linq;
using AwesomeWebFormsDemo.Data;
using AwesomeWebFormsDemo.Models;
using AwesomeWebFormsDemo.ViewModels.Input;
using System.Web.Mvc;

using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Controllers.Demos.Grid.MasterDetailCrud
{
    public class MasterDetailCrudDemoController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult RestaurantGridGetItems(GridParams g)
        {
            var query = Db.Restaurants.Where(o => o.IsCreated).AsQueryable();

            var gmb = new GridModelBuilder<Restaurant>(query, g)
            {
                KeyProp = o => o.Id,
                GetItem = () => Db.Get<Restaurant>(Convert.ToInt32(g.Key))
            };

            return Json(gmb.Build());
        }

        public ActionResult Create()
        {
            // needed so we could add addresses even before the restaurant is created/saved
            var rest = new Restaurant();

            Db.Add(rest);


            return PartialView(new RestaurantInput { Id = rest.Id });
        }

        [HttpPost]
        public ActionResult Create(RestaurantInput input)
        {
            if (!ModelState.IsValid)
            {
                return PartialView(input);
            }

            var restaurant = Db.Find<Restaurant>(input.Id);
            restaurant.Name = input.Name;
            restaurant.IsCreated = true;



            return Json(new { input.Id });
        }

        public ActionResult Edit(int id)
        {
            var rest = Db.Find<Restaurant>(id);
            return PartialView("Create", new RestaurantInput { Id = id, Name = rest.Name });
        }

        [HttpPost]
        public ActionResult Edit(RestaurantInput input)
        {
            if (!ModelState.IsValid)
            {
                return PartialView("Create", input);
            }

            var rest = Db.Find<Restaurant>(input.Id);

            rest.Name = input.Name;

            return Json(new { rest.Id });
        }

        public ActionResult Delete(int id)
        {
            var restaurant = Db.Find<Restaurant>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "restaurant",
                Name = restaurant.Name
            });
        }

        [HttpPost]
        public ActionResult Delete(DeleteConfirmInput input)
        {
            Db.Remove(Db.Find<Restaurant>(input.Id));


            return Json(new { input.Id });
        }
    }
}