using System;
using System.Web;
using System.Web.Mvc;
using System.Web.UI;
using AwesomeWebFormsDemo.Utils;
using NPOI.HSSF.Record;
using Omu.AwemWebForms.Helpers;
using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Helpers.Awesome
{
    public static class PageCrudHelpers
    {
        private static UrlHelper GetUrlHelper(Page page)
        {
            return page.Url();
        }

        public static IHtmlString InitCrudForGridNest(this Page page, string gridId, string crudController)
        {
            var url = GetUrlHelper(page);
            
            var result =
                page.Awe()
                    .InitPopupForm()
                    .Name("create" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Create", crudController))
                    .Mod(o => o.Inline().ShowHeader(false))
                    .Success("aweUtils.refreshGrid('" + gridId + "')")
                    .ToString()
                + page.Awe()
                    .InitPopupForm()
                    .Name("edit" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Edit", crudController))
                    .Mod(o => o.Inline().ShowHeader(false))
                    .Success("aweUtils.refreshGrid('" + gridId + "')")
                + page.Awe()
                    .InitPopupForm()
                    .Name("delete" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Delete", crudController))
                    .Mod(o => o.Inline().ShowHeader(false))
                    .Success("aweUtils.refreshGridOnDel('" + gridId + "')");

            return new MvcHtmlString(result);
        }

        public static IHtmlString InitDeletePopupForGrid(
            this Page page,
            string gridId,
            string crudController = null,
            string action = "Delete")
        {
            var url = GetUrlHelper(page);

            var result =
                page.Awe()
                    .InitPopupForm()
                    .Name("delete" + gridId)
                    .Group(gridId)
                    .Url(url.Action(action, crudController))
                    .Success("aweUtils.itemDeleted('" + gridId + "')")
                    .OnLoad("aweUtils.delConfirmLoad('" + gridId + "')") // calls grid.api.select and animates the row
                    .Height(200)
                    .Modal()
                    .ToString();

            return new MvcHtmlString(result);
        }

        /*beging*/
        /// <summary>
        /// initialize PopupForms for grid crud
        /// </summary>
        /// <param name="page"></param>
        /// <param name="gridId"></param>
        /// <param name="crudController">controller containing the crud actions</param>
        /// <param name="createPopupHeight">height of the create/edit popup</param>
        /// <param name="maxWidth"> max popup width</param>
        /// <param name="reload">reload grid after save/delete action success</param>
        /// <param name="area"></param>
        /// <param name="inlineContId">html id for the inline popup container</param>
        public static IHtmlString InitCrudPopupsForGrid(
            this Page page,
            string gridId,
            string crudController,
            int createPopupHeight = 430,
            int maxWidth = 0,
            bool reload = true,
            string area = null,
            string inlineContId = null,
            Action<PopupFormOpt, string> cfg = null)
        {
            var url = GetUrlHelper(page);

            var refreshGrid = "refreshGrid";
            var refreshOnDel = "refreshGridOnDel";

            var format = "aweUtils.{0}('" + gridId + "')";

            var createFunc = string.Format(format, reload ? refreshGrid : "itemCreated");
            var editFunc = string.Format(format, reload ? refreshGrid : "itemEdited");
            var delFunc = string.Format(format, reload ? refreshOnDel : "itemDeleted");

            var delConfirmFunc = string.Format(format, "delConfirmLoad");
            var showHeader = true;

            if (inlineContId != null)
            {
                showHeader = false;
            }

            var opt = new PopupFormOpt
            {
                Name = "create" + gridId,
                Group = gridId,
                Height = createPopupHeight,
                MaxWidth = maxWidth,
                Url = url.Action("Create", crudController, new { area }),
                Title = "Create item",
                Modal = true,
                OnSuccess = createFunc,
                ShowHeader = showHeader,
                InlTrg = inlineContId
            };

            if (cfg != null)
            {
                cfg(opt, "create");
            }

            var create = page.Awe()
                .InitPopupForm(opt);

            opt = new PopupFormOpt
            {
                Name = "edit" + gridId,
                Group = gridId,
                Height = createPopupHeight,
                MaxWidth = maxWidth,
                Url = url.Action("Edit", crudController, new { area }),
                Title = "Edit item",
                Modal = true,
                OnSuccess = editFunc,
                ShowHeader = showHeader,
                InlTrg = inlineContId
            };

            if (cfg != null)
            {
                cfg(opt, "edit");
            }

            var edit = page.Awe()
                .InitPopupForm(opt);

            opt = new PopupFormOpt
            {
                Name = "delete" + gridId,
                Group = gridId,
                Url = url.Action("Delete", crudController, new { area }),
                Title = "Delete item",
                OnSuccess = delFunc,
                OnLoad = delConfirmFunc, // calls grid.api.select and animates the row
                Height = 200,
                Modal = true,
                ShowHeader = showHeader,
                InlTrg = inlineContId
            };

            if (cfg != null)
            {
                cfg(opt, "delete");
            }

            var delete = page.Awe()
                  .InitPopupForm(opt);

            var result = create.ToString() + edit + delete;

            return new MvcHtmlString(result);
        }

        public static IHtmlString InitCrudPopupsForAjaxList(
           this Page page,
           string ajaxListId,
           string controller,
           string popupName)
        {
            var url = GetUrlHelper(page);

            var result =
                page.Awe()
                    .InitPopupForm()
                    .Name("create" + popupName)
                    .Url(url.Action("Create", controller))
                    .Height(430)
                    .Success("aweUtils.itemCreatedAlTbl('" + ajaxListId + "')")
                    .Group(ajaxListId)
                    .Title("create item")
                    .ToString()

                + page.Awe()
                      .InitPopupForm()
                      .Name("edit" + popupName)
                      .Url(url.Action("Edit", controller))
                      .Height(430)
                      .Success("aweUtils.itemEditedAl('" + ajaxListId + "')")
                      .Group(ajaxListId)
                      .Title("edit item")

                + page.Awe()
                      .InitPopupForm()
                      .Name("delete" + popupName)
                      .Url(url.Action("Delete", controller))
                      .Success("aweUtils.itemDeletedAl('" + ajaxListId + "')")
                      .Group(ajaxListId)
                      .OkText("Yes")
                      .CancelText("No")
                      .Height(200)
                      .Title("delete item");

            return new MvcHtmlString(result);
        }
    }
}