using AwesomeWebFormsDemo.Utils;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.UI;

namespace AwesomeWebFormsDemo.Helpers
{
    public static class WfShowCodeHelpers
    {
        /// <summary>
        /// get the string value of the code between the /*begin*/ and /*end*/ comment blocks  
        /// </summary>
        /// <param name="page"></param>
        /// <param name="path"> file location path</param>
        /// <param name="k"></param>
        /// <returns></returns>
        public static IHtmlString Csrc(this Page page, string path, object k = null)
        {
            var startWord = "/*begin" + k + "*/";
            var endWord = "/*end" + k + "*/";

            var newpath = page.ServerMapPath() + path;
            var lines = System.IO.File.ReadAllLines(newpath);
            var code = StrUtil.GetCode(lines, startWord, endWord);
            return new MvcHtmlString(StrUtil.ParseStrToCode(code, path));
        }

        /// <summary>
        /// get the string value of the view code located between the begin+key and end+key comment blocks
        /// </summary>
        /// <param name="page"></param>
        /// <param name="path"></param>
        /// <param name="k">key</param>
        /// <param name="wrap">wrap with div class='code'</param>
        /// <returns></returns>
        public static IHtmlString Source(this Page page, string path, object k = null, bool wrap = false)
        {
            var key = k?.ToString() ?? "";

            if (!path.EndsWith(".aspx") && !path.EndsWith(".ascx"))
            {
                path = "\\Views\\" + path;
            }

            var newpath = page.ServerMapPath() + path;

            var lines = System.IO.File.ReadAllLines(newpath);


            var code = path.EndsWith(".cshtml") ?
                StrUtil.GetCode(lines, "@*begin" + key + "*@", "@*end" + key + "*@") :
                StrUtil.GetCode(lines, "<%--begin" + key + "--%>", "<%--end" + key + "--%>");
            var result = StrUtil.ParseStrToCode(code, path);
            if (wrap) result = "<div class='code'>" + result + "</div>";

            return new MvcHtmlString(result);
        }

        /// <summary>
        /// get the string value of the controller code between the /*begin(key)*/ and /*end(key)*/ comment blocks 
        /// </summary>
        /// <param name="page"></param>
        /// <param name="path"></param>
        /// <param name="k">key</param>
        /// <returns></returns>
        public static IHtmlString Csource(this Page page, string path, object k = null)
        {
            var startWord = "/*begin" + k + "*/";
            var endWord = "/*end" + k + "*/";
            
            var newpath = page.ServerMapPath() + "\\Controllers\\" + path;
            var lines = System.IO.File.ReadAllLines(newpath);

            var code = StrUtil.GetCode(lines, startWord, endWord);

            return new MvcHtmlString(StrUtil.ParseStrToCode(code, path));
        }

        /// <summary>
        /// get the string value of the view code located between the begin+key and end+key comment blocks
        /// </summary>
        /// <param name="html"></param>
        /// <param name="path"></param>
        /// <param name="k">key</param>
        /// <param name="wrap">wrap with div class='code'</param>
        /// <returns></returns>
        public static IHtmlString PageSource(this HtmlHelper html, string path, object k = null, bool wrap = false)
        {
            var key = k == null ? "" : k.ToString();
            
            if (!path.EndsWith(".aspx") && !path.EndsWith(".ascx"))
            {
                path = "\\Views\\" + path;
            }
            
            var newpath = html.ServerMapPath() + path;
            var lines = System.IO.File.ReadAllLines(newpath);

            var code = path.EndsWith(".cshtml") ? StrUtil.GetCode(lines, "@*begin" + key + "*@", "@*end" + key + "*@") : 
                StrUtil.GetCode(lines, "<%--begin" + key + "--%>", "<%--end" + key + "--%>");
            var result = StrUtil.ParseStrToCode(code, path);
            if (wrap) result = "<div class='code'>" + result + "</div>";

            return new MvcHtmlString(result);
        }

    }
}