using Microsoft.AspNetCore.Html;
using Microsoft.AspNetCore.Mvc.Rendering;
using System.Globalization;
using System.Text;

namespace AweuiAspCore.Utils
{
    public static class JsUtil
    {
        private static CultureInfo GetCulture()
        {
            return CultureInfo.CurrentCulture;
        }

        /// <summary>
        /// Set the defaults for awem.js and jquery.validate (if present) by calling aweUtils.init; 
        /// sets date format, first day of week, decimal separator, isMobileOrTablet
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="ahtml"></param>
        /// <param name="isMobile"></param>
        /// <returns></returns>
        public static IHtmlContent InitAweui<T>(this IHtmlHelper<T> html, bool? isMobile = null)
        {
            var isMobileOrTablet = (isMobile ?? IsMobileOrTablet(html)) ? 1 : 0;
            var dateFormat = GetDateFormat();
            var decimalSep = GetDecimalSeparator();

            var sb = new StringBuilder("<script>");
            sb.AppendFormat("awem.isMobileOrTablet = function() {{ return {0}; }};", isMobileOrTablet);
            sb.AppendFormat("awem.fdw = {0};", (int)GetCulture().DateTimeFormat.FirstDayOfWeek);
            sb.AppendFormat("aweUtils.init('{0}', {1}, '{2}');", dateFormat, isMobileOrTablet, decimalSep);            
            sb.AppendFormat("aweui && aweui.init(),aweui.decimalSep = '{0}';", decimalSep);
            sb.Append("</script>");

            return new HtmlString(sb.ToString());
        }

        public static int FirstDayOfWeek()
        {
            return (int)GetCulture().DateTimeFormat.FirstDayOfWeek;
        }

        public static string GetDecimalSeparator()
        {
            return GetCulture().NumberFormat.NumberDecimalSeparator;
        }

        public static string GetDateFormat()
        {
            var format = GetCulture().DateTimeFormat.ShortDatePattern;
            var result = format;

            return result;
        }

        public static bool IsMobileOrTablet(IHtmlHelper html)
        {
            return MobileUtils.IsMobileOrTablet(html.ViewContext.HttpContext.Request);
        }
    }
}