//import {awedb} from './awedb.js';
//import { awef, awem, aweUtils, aweui } from '../aweui/all.js';
//import {aweq} from '../infra/aweq';
//import {awevld, modelState} from '../infra/awevld';
//import * as $ from 'jquery';

var dinnersInlCrudCon = function () {
    function map(o) {
        return {
            id: o.id,
            name: o.name,
            date: awem.dateToStr( o.date),
            chefName: o.chef.firstName + ' ' + o.chef.lastName,
            meals: o.meals ? o.meals.map(function (m) { return m.name }).join(', ') : null,
            bonusMeal: o.bonusMeal.name,
            organic: o.organic,
            dispOrganic: o.organic ? aweui.dict.Yes : aweui.dict.No,

            // below properties used for inline editing only
            mealsIds: o.meals ? o.meals.map(function (m) { return m.id; }) : null,
            chefId: o.chef.id,
            bonusMealId: o.bonusMeal.id,

            // for conditional demo
            editable: o.date < new Date(2012, 3, 1),
            dateReadonly: o.date.getDate() < 15
        };
    }

    function mapToEnt(din, p) {
        din.name = p.name;
        din.date = awem.toDate(p.date);
        din.chef = aweq.getById(awedb.chefs, p.chef);
        din.meals = aweq.getByIds(awedb.meals, p.meals);
        din.bonusMeal = aweq.getById(awedb.meals, p.bonusMealId);
        din.organic = p.organic === 'true';
        return din;
    }

    function validate(sp) {
        var din = aweUtils.getParams(sp);
        var ms = modelState(din);
        var v = awevld(ms);

        v.req('name');
        v.req('date');
        v.req('chef');
        v.req('meals');
        v.req('bonusMealId', 'bonus meal');

        return ms;
    }

    return {
        map: map,
        grid: function (sgp) {
            var gp = aweUtils.getGridParams(sgp);
            var list = awedb.dinners;

            if (gp.search) {
                var words = gp.search.split(" ");
                var regs = $.map(words, function (w) { return new RegExp(w, "i"); });
                var regslen = regs.length;

                list = aweq.where(list, function (o) {
                    var matches = 0;
                    awef.loop(regs, function (reg) {
                        if (reg.test(o.name) || reg.test(o.chef.firstName) || reg.test(o.chef.lastName)) matches++;
                    });

                    return regslen === matches;
                });
            }

            return aweUtils.gridModelBuilder(
                {
                    gp: gp,
                    items: list,
                    key: "id",
                    map: dinnersInlCrudCon.map,
                    getItem: function () {
                        return aweq.getById(awedb.dinners, gp.key);
                    }
                });
        },
        create: function (sp) {
            var vm = aweUtils.getParams(sp);
            var ms = validate(sp);
            if (!ms.isValid()) {
                return aweui.getErrors(ms.errors);
            }

            var din = mapToEnt({}, vm);
            awedb.add(awedb.dinners, din);

            return aweui.inlRes(map(din));
        },
        edit: function (sp) {
            var p = aweUtils.getParams(sp);
            var ms = validate(sp);
            if (!ms.isValid()) {
                return aweui.getErrors(ms.errors);
            }

            var din = aweq.getById(awedb.dinners, p.id);

            mapToEnt(din, p);

            return aweui.inlRes(map(din));
        },
        initRowVld: function (row, td) {
            aweUtils.initInstVld(row, validate);
        },
        confirmDelete: function (sp) {
            var p = aweUtils.getParams(sp);
            var din = aweq.getById(awedb.dinners, p.id);

            if (!din) throw new Error('this item doesn\'t exist anymore');

            // key hidden input is also used by aweUtils.delConfirmLoad to highlight the grid row
            return '<form>Are you sure you want to delete <b> ' + din.name +
                '</b> ?<input type="hidden" name="key" value="' + p.id + '">' +
                '</form>';
        },
        del: function (sp) {
            var p = aweUtils.getParams(sp);

            // p.key is the value we set in the hidden input in confirmDelete
            aweq.removeById(awedb.dinners, p.key);
            return { id: p.key };
        }
    };
}();

//export {dinnersInlCrudCon};