using Core.Model;
using Microsoft.EntityFrameworkCore;

namespace Data
{
    public class ProDbContext : DbContext
    {
        public DbSet<Country> Countries { get; set; }

        public DbSet<Chef> Chefs { get; set; }

        public DbSet<Meal> Meals { get; set; }

        public DbSet<Feedback> Feedbacks { get; set; }

        public DbSet<User> Users { get; set; }

        public DbSet<Role> Roles { get; set; }

        // unused but EF will use this Property Name for Table name
        public DbSet<UserRole> UserRoles { get; set; }

        public DbSet<Dinner> Dinners { get; set; }

        // EF will use this Property Name for Table name
        public DbSet<DinnerMeal> DinnerMeals { get; set; }

        public ProDbContext(DbContextOptions<ProDbContext> dbContextOptions) : base(dbContextOptions)
        {
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            // https://docs.microsoft.com/en-us/ef/core/modeling/relationships

            // many to many for Dinners and Meals
            modelBuilder.Entity<DinnerMeal>()
                .HasKey(t => new { t.DinnerId, t.MealId});

            modelBuilder.Entity<DinnerMeal>()
                .HasOne(pt => pt.Dinner)
                .WithMany(p => p.DinnerMeals)
                .HasForeignKey(pt => pt.DinnerId);

            modelBuilder.Entity<DinnerMeal>()
                .HasOne(pt => pt.Meal)
                .WithMany(t => t.DinnerMeals)
                .HasForeignKey(pt => pt.MealId);

            // many to many for Users and Roles
            modelBuilder.Entity<UserRole>()
                .HasKey(t => new { t.UserId, t.RoleId});

            modelBuilder.Entity<UserRole>()
                .HasOne(pt => pt.User)
                .WithMany(p => p.UserRoles)
                .HasForeignKey(pt => pt.UserId);

            modelBuilder.Entity<UserRole>()
                .HasOne(pt => pt.Role)
                .WithMany(t => t.UserRoles)
                .HasForeignKey(pt => pt.RoleId);
        }
    }
}