using System.Linq;
using System.Threading.Tasks;
using Core.Model;
using Data;
using Microsoft.EntityFrameworkCore;
using Service.Infra;

namespace Service
{
    public class UserService
    {
        private readonly IHasher hasher;
        private readonly ProDbContext cx;

        public UserService(IHasher hasher, ProDbContext cx)
        {
            this.cx = cx;
            this.hasher = hasher;
        }

        public async Task<User> GetUserWithRoleAsync(int id)
        {
            return await GetUsersWithRoles().SingleAsync(o => o.Id == id);
        }

        public IQueryable<User> GetUsersWithRoles()
        {
            return cx.Set<User>().Include(o => o.UserRoles).ThenInclude(o => o.Role);
        }

        public async Task<User> Create(User user)
        {
            user.Password = hasher.Encrypt(user.Password);
            cx.Users.Add(user);
            await cx.SaveChangesAsync();
            
            return user;
        }

        public async Task ChangePassword(int userId, string password)
        {
            var user = cx.Users.Find(userId);
            user.Password = hasher.Encrypt(password);
            await cx.SaveChangesAsync();
        }

        public async Task EditRoles(int userId, int[] roles)
        {
            var ent = await cx.FindAsync<User>(userId);
            await cx.Entry(ent).Collection(o => o.UserRoles).LoadAsync();
            
            ent.UserRoles = roles.Select(o => new UserRole { UserId = userId, RoleId = o }).ToList();

            await cx.SaveChangesAsync();
        }

        public async Task<User> GetUser(string login, string password)
        {
            var user = await cx.Set<User>().Where(o => o.Login == login && !o.IsDeleted)
                .Include(o => o.UserRoles)
                .ThenInclude(o => o.Role)
                .SingleOrDefaultAsync();

            if (user == null) return null;
            if (!hasher.CompareStringToHash(password, user.Password)) return null;
            return user;
        }
    }
}