using System.Collections.Generic;
using System.Security.Claims;
using System.Threading.Tasks;
using Core.Model;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Mvc;
using Service;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    // https://docs.microsoft.com/en-us/aspnet/core/security/authentication/cookie?view=aspnetcore-3.1
    public class AccountController : Controller
    {
        private readonly UserService userSvc;

        public AccountController(UserService userSvc)
        {
            this.userSvc = userSvc;
        }

        public new async Task<IActionResult> SignOut()
        {
            await HttpContext.SignOutAsync();
            return RedirectToAction("Index", "Home");
        }

        public IActionResult SignIn(string returnUrl)
        {
            return View();
        }

        [HttpPost]
        public async Task<IActionResult> SignIn(SignInInput input)
        {
            if (!ModelState.IsValid)
            {
                input.Login = null;
                input.Password = null;

                return View();
            }

            User user;

            // Warning: done, for learning purposes only
            if (input.Login == "o" && input.Password == "1")
            {
                user = new User
                {
                    Login = "o",
                    UserRoles = new List<UserRole>
                    { 
                        new UserRole
                        {
                            Role = new Role { Name = "admin" }
                        }
                    }

                };
            }
            else
            {
                user = await userSvc.GetUser(input.Login, input.Password);
            }

            if (user == null)
            {
                ModelState.AddModelError("", "Login or password incorrect, try Login: o and Password: 1");
                return View();
            }

            await HttpContext.SignOutAsync();

            var claims = new List<Claim>
            {
                new Claim(ClaimTypes.Name, user.Login),
            };

            foreach (var ur in user.UserRoles)
            {
                claims.Add(new Claim(ClaimTypes.Role, ur.Role.Name));
            }

            var claimsIdentity = new ClaimsIdentity(
                claims, CookieAuthenticationDefaults.AuthenticationScheme);

            var authProperties = new AuthenticationProperties
            {
                //AllowRefresh = <bool>,
                // Refreshing the authentication session should be allowed.

                //ExpiresUtc = DateTimeOffset.UtcNow.AddMinutes(10),
                // The time at which the authentication ticket expires. A 
                // value set here overrides the ExpireTimeSpan option of 
                // CookieAuthenticationOptions set with AddCookie.

                IsPersistent = input.Remember,
                // Whether the authentication session is persisted across 
                // multiple requests. When used with cookies, controls
                // whether the cookie's lifetime is absolute (matching the
                // lifetime of the authentication ticket) or session-based.

                //IssuedUtc = <DateTimeOffset>,
                // The time at which the authentication ticket was issued.

                RedirectUri = input.ReturnUrl
                // The full path or absolute URI to be used as an http 
                // redirect response value.
            };

            await HttpContext.SignInAsync(new ClaimsPrincipal(claimsIdentity), authProperties);

            if (input.ReturnUrl != null) return Redirect(input.ReturnUrl);
            return RedirectToAction("Index", "Home");
        }
    }
}