using Microsoft.AspNetCore.Mvc;
using Core.Model;
using Omu.AwesomeMvc;
using Service;
using Microsoft.EntityFrameworkCore;
using WebUI.Utils.Awe;

namespace WebUI.Controllers.Awesome
{
    public class MealsMultiLookupController : Controller
    {
        private readonly CrudService crudSvc;

        public MealsMultiLookupController(CrudService crudSvc)
        {
            this.crudSvc = crudSvc;
        }

        public async Task<IActionResult> GetItems(int[] v)
        {
            var query = crudSvc.GetQuery<Meal>();

            if (v != null)
            {
                query = query.Where(o => v.Contains(o.Id));
            }

            var res = await EFAweUtil.BuildListAsync(
                query,
                setContentFunc: (items) => this.RenderPartialView("MealsMultiLookupCaption", items));

            return Json(res);
        }

        public async Task<IActionResult> Search(int[] selected, int page, string search)
        {
            selected = selected ?? Array.Empty<int>();

            var query = crudSvc.GetQuery<Meal>()
                .Filter(search, o => o.Name)
                .Where(o => !selected.Contains(o.Id));

            var res = await EFAweUtil.BuildListAsync(
                query, 
                page, 
                keyContentFunc: o => new KeyContent(o.Id, o.Name),
                setContentFunc: (items) => this.RenderPartialView("MealsMultiLookupItems", items));

            return Json(res);
        }

        public async Task<IActionResult> Selected(int[] selected)
        {
            var query = crudSvc.GetQuery<Meal>();

            if (selected != null)
            {
                query = query.Where(o => selected.Contains(o.Id));
            }

            var res = await EFAweUtil.BuildListAsync(
                query,
                setContentFunc: (items) => this.RenderPartialView("MealsMultiLookupItems", items));

            return Json(res);
        }
    }
}