using System;
using System.Threading.Tasks;
using Core.Model;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Omu.AwesomeMvc;
using Service;
using WebUI.Utils;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    public class FeedbackController : Controller
    {
        private readonly CrudService svc;

        public FeedbackController(CrudService svc)
        {
            this.svc = svc;
        }

        public ActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Feedback o)
        {
            return new { o.Id, o.Comments, Editable = HttpContext.Session.GetInt32(StrConst.LastFeedback) == o.Id };
        }

        public async Task<IActionResult> GridGetItems(GridParams g)
        {
            var query = svc.GetEntQuery<Feedback>();

            var gmb = new GridModelBuilder<Feedback>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,
                ToArrayAsync = qitems => qitems.ToArrayAsync(),

                GetItemAsync = () => svc.GetAsync<Feedback>(Convert.ToInt32(g.Key))
            };

            return Json(await gmb.BuildAsync());
        }

        public IActionResult Create()
        {
            // make sure to use "return PartialView" for PopupForm/Popup views 
            // this will ignore _viewstart.cshtml so that you don't use the _Layout.cshtml and reload all the scripts
            return PartialView();
        }

        [HttpPost]
        public async Task<IActionResult> Create(FeedbackInput input)
        {
            if (!ModelState.IsValid) return PartialView(input);


            var ent = await svc.AddAsync(new Feedback
            {
                Comments = HtmlUtil.SanitizeHtml(input.Comments)
            });
            
            HttpContext.Session.SetInt32(StrConst.LastFeedback, ent.Id);

            // the create PopupForm's success function aweUtils.itemCreated expects the grid row model obj, to render and append the new row
            return Json(MapToGridModel(ent));
        }

        public async Task<IActionResult> Edit(int id)
        {
            Check.IdHasVal(id);

            var ent = await svc.GetAsync<Feedback>(id);

            var input = new FeedbackInput
            {
                Id = id, 
                Comments = HtmlUtil.SanitizeHtml(ent.Comments)
            };

            return PartialView("Create", input);
        }

        [HttpPost]
        public async Task<IActionResult> Edit(FeedbackInput input)
        {
            if (HttpContext.Session.GetInt32(StrConst.LastFeedback) != input.Id)
            {
                throw new Exception("can't edit this feedback anymore");
            }

            if (!ModelState.IsValid) return PartialView("Create", input);

            var ent = await svc.GetAsync<Feedback>(input.Id.Value);
            
            ent.Comments = HtmlUtil.SanitizeHtml(input.Comments);

            await svc.Update(ent);

            // the edit PopupForm's success function aweUtils.itemEdited expects an obj with "Id" property
            return Json(new { ent.Id });
        }
    }
}