using Data;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using Omu.AwesomeMvc;
using Service;
using Service.Infra;
using Service.Infra.Cache;
using WebUI;
using WebUI.Resources;
using WebUI.Utils;

var builder = WebApplication.CreateBuilder(args);
var services = builder.Services;

configureServices(services);


setupDatabase(services);

var app = builder.Build();
app.UseExceptionHandler("/Error");
app.UseStatusCodePagesWithReExecute("/Error/HttpStatus", "?code={0}");

new AwesomeConfig().Configure(app.Services);


app.UseSession();
app.UseStaticFiles();
app.UseRequestLocalization(AppCultureSettings.ReqLocOpt);

app.UseAuthentication();

app.UseRouting();
app.UseAuthorization(); // this has to be between UserRouting and MapControllerRoute
app.MapControllerRoute(
        name: "default",
        pattern: "{controller=Home}/{action=Index}/{id?}");

Globals.PicturesPath = Path.Combine(app.Environment.WebRootPath, "pictures");

Settings.CheckboxModFunc = box => Omu.Awem.Helpers.AwesomeModExtensions.Ochk(box);

app.Run();

void setupDatabase(IServiceCollection services)
{
    // sql server db
    var cs = builder.Configuration.GetConnectionString("DefaultConnection");
    services.AddDbContext<ProDbContext>(o => o.UseSqlServer(cs));
}

void configureServices(IServiceCollection services)
{
    services.AddAuthentication(CookieAuthenticationDefaults.AuthenticationScheme)
        .AddCookie(opt => opt.LoginPath = "/Account/SignIn");
    
    services.AddMemoryCache();
    services.AddSession(options =>
    {
        // Set a short timeout for easy testing.
        //options.IdleTimeout = TimeSpan.FromSeconds(10);
        options.Cookie.HttpOnly = true;
        // Make the session cookie essential
        options.Cookie.IsEssential = true;
    });    

    var provider = new AweMetaProvider();
    
    var mvcBuilder = services.AddControllersWithViews(o =>
    {
        o.ModelMetadataDetailsProviders.Add(provider);
    })
    .AddDataAnnotationsLocalization(options =>
    {
        options.DataAnnotationLocalizerProvider = (type, factory) =>
            factory.Create(typeof(Mui));
    });

    mvcBuilder.AddRazorRuntimeCompilation(); // compile views on change 

    services.AddHttpContextAccessor();
    services.AddLocalization(opts => { opts.ResourcesPath = "Resources"; });
    services.AddSingleton<SharedViewLocalizer>();
    services.Configure<RequestLocalizationOptions>(AppCultureSettings.ConfigureReqLocOpt);

    services.AddScoped<ICacheStorage, CacheStorage>();
    services.AddScoped<ICacheManager, CacheManager>();

    services.AddScoped<CrudService>();
    services.AddScoped<DinnerService>();
    services.AddScoped<ChefService>();
    services.AddScoped<MealService>();
    services.AddScoped<UserService>();
    services.AddScoped<IHasher, Hasher>();

    services.AddHostedService<ConsumeScopedServiceHostedService>();
    services.AddScoped<IScopedProcessingService, ScopedProcessingService>();
}