using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using Microsoft.AspNetCore.Html;
using Microsoft.AspNetCore.Localization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.AspNetCore.Mvc.Routing;
using Newtonsoft.Json;
using Omu.Awem.Helpers;
using Omu.AwesomeMvc;
using WebUI.ViewModels.Settings;

namespace WebUI.Utils
{
    public static class ClientUtil
    {
        private static readonly string[] themes = { "mat", "mui", "wui", "bts", "met", "val", "zen", "gtx", "gui", "gui3", "start", "black-cab" };

        // cultures that we have an awedict.culture.js file for
        private static readonly string[] cultures = { "en", "fr", "it", "de", "es", "ro", "ru" };

        public static bool IsAdmin(this ClaimsPrincipal user)
        {
            return user.IsInRole("admin");
        }

        public static string GetClientDict()
        {
            var localizer = AwesomeConfig.Localizer;

            var obj = new
            {
                Cancel = localizer["Cancel"].Value,
                ChangePicture = localizer["change picture"].Value,
                Stuff = localizer["Stuff"].Value,
                About = localizer["About"].Value
            };

            return JsonConvert.SerializeObject(obj);
        }
        
        public static string GetMenu(IHtmlHelper<dynamic> html)
        {
            var localizer = AwesomeConfig.Localizer;
            var url = GetUrlHelper(html);

            var stuff = new[]
            {
                new KeyContent(url.Action("Index","Dinner"), localizer["Dinner"]),
                new KeyContent(url.Action("Index","Chef"), "Chefs"),
                new KeyContent(url.Action("Index","Country"), localizer["Country"]),
                new KeyContent(url.Action("Index","Feedback"), "Feedback"),
            };

            var menu = new List<Oitem>
            {
                new Oitem(url.Action("Index", "Home"), localizer["Home"]),
                new Oitem(url.Action("Index", "Meal"), localizer["Meals"])
            };
            
            if (html.ViewContext.HttpContext.User.IsAdmin())
            {
                menu.Add(new Oitem(url.Action("Index", "User"), "Users"));
            }

            return JsonConvert.SerializeObject(new
            {
                menu,
                stuff
            });
        }

        public static SettingsVal GetSettings(IHtmlHelper<dynamic> html)
        {
            html.ViewContext.HttpContext.Request.Cookies.TryGetValue("theme", out string theme);
            return new SettingsVal
            {
                Theme = !themes.Contains(theme) ? "mat" : theme
            };
        }

        public static string GetCulture(this IHtmlHelper<dynamic> html)
        {
            var requestCulture = html.ViewContext.HttpContext.Features.Get<IRequestCultureFeature>();
            var rc = requestCulture.RequestCulture.Culture.Name;

            return cultures.Contains(rc) ? rc : cultures[0];
        }

        private static IUrlHelper GetUrlHelper<T>(IHtmlHelper<T> html)
        {
            return ((IUrlHelperFactory)html.ViewContext.HttpContext.RequestServices.GetService(typeof(IUrlHelperFactory))).GetUrlHelper(html.ViewContext);
        }

    }
}