using Core.Model;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace WebUI.Utils
{
    /// <summary>
    /// https://learn.microsoft.com/en-us/aspnet/core/fundamentals/host/hosted-services?view=aspnetcore-7.0&tabs=visual-studio
    /// </summary>
    public class ConsumeScopedServiceHostedService : BackgroundService
    {        
        public ConsumeScopedServiceHostedService(IServiceProvider services)
        {
            Services = services;         
        }

        public IServiceProvider Services { get; }

        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            await DoWork(stoppingToken);
        }

        private async Task DoWork(CancellationToken stoppingToken)
        {
            using (var scope = Services.CreateScope())
            {
                var scopedProcessingService =
                    scope.ServiceProvider
                        .GetRequiredService<IScopedProcessingService>();

                await scopedProcessingService.DoWork(stoppingToken);
            }
        }

        public override async Task StopAsync(CancellationToken stoppingToken)
        {
            await base.StopAsync(stoppingToken);
        }
    }

    internal interface IScopedProcessingService
    {
        Task DoWork(CancellationToken stoppingToken);
    }

    internal class ScopedProcessingService : IScopedProcessingService
    {
        public ScopedProcessingService(IServiceProvider services)
        {
            Services = services;
        }

        public IServiceProvider Services { get; }

        public async Task DoWork(CancellationToken stoppingToken)
        {
            while (!stoppingToken.IsCancellationRequested)
            {
                var crudSrv = Services.GetRequiredService<CrudService>();

                async Task RestoreAll(IEnumerable<DelEntity> query)
                {
                    DelEntity ent = null;
                    foreach (var item in query)
                    {
                        item.IsDeleted = false;
                        ent = item;
                    }

                    if (ent != null)
                    {
                        await crudSrv.Update(ent);
                    }
                }

                try
                {
                    var dinners = await crudSrv.GetQuery<Dinner>(true)
                    .Where(o => o.DinnerMeals.Count > 2 && o.IsDeleted).ToListAsync();

                    await RestoreAll(dinners);

                    var meals = crudSrv.GetQuery<Meal>(true).Where(o => o.Picture != null && o.IsDeleted && o.Name.Length > 3);

                    await RestoreAll(meals);

                    var chefs = crudSrv.GetQuery<Chef>(true).Where(o => o.IsDeleted && o.LastName.Length > 3);

                    await RestoreAll(chefs);

                    var countries = crudSrv.GetQuery<Country>().Where(o => o.IsDeleted && o.Name.Length > 3);

                    await RestoreAll(countries);
                }
                catch (Exception ex)
                {
                    // log exception
                }
                
                await Task.Delay(10*60*1000, stoppingToken);
            }            
        }        
    }
}