using Microsoft.AspNetCore.Mvc;
using Omu.AwesomeMvc;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Core.Repo;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.StoreAwesome.WebUI.Utils.Attributes;

namespace Omu.StoreAwesome.WebUI.Core.Controllers
{
    [Admin]
    public class AdminContactUsMessagesController : Controller
    {
        private GenericBaseRepo<ContactUsMessage> repo;

        public AdminContactUsMessagesController(GenericBaseRepo<ContactUsMessage> repo)
        {        
            this.repo = repo;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(ContactUsMessage o)
        {
            var msg = o.Message;
            if (msg.Length > 50)
            {
                msg = msg.Substring(0, 50) + "...";
            }

            return new
            {
                o.Id,
                o.Email,
                Message = msg,
                o.IsRead,
                IsReadDisplay = o.IsRead ? "Yes" : "No",
                IsReadClass = o.IsRead ? string.Empty : "notRead",
                DateCreated = o.DateCreated.ToShortDateString()
            };
        }

        public async Task<IActionResult> GridGetData(GridParams g)
        {
            var query = repo.GetAll().AsQueryable();

            var model = new GridModelBuilder<ContactUsMessage>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel
            };

            return Json(await model.EFBuildAsync());
        }

        public async Task<IActionResult> Details(int id)
        {
            var prod = await repo.FindAsync(id);

            return PartialView(prod);
        }

        [HttpPost]
        public async Task<IActionResult> Delete(IEnumerable<int> ids)
        {
            await repo.BatchRemoveAsync(ids.Cast<object>());

            return Json(new { });
        }

        [HttpPost]
        public async Task<IActionResult> MarkRead(int[] ids)
        {
            await repo.BatchUpdateAsync(ids.Cast<object>(), o =>
            {
                o.IsRead = true;
            });            

            return Json(new { });
        }

        [HttpPost]
        public async Task<IActionResult> MarkUnread(int[] ids)
        {
            await repo.BatchUpdateAsync(ids.Cast<object>(), o =>
            {
                o.IsRead = false;
            });

            return Json(new { });            
        }

        //public async Task<IActionResult> TestAdd()
        //{
        //    await repo.AddAsync(new ContactUsMessage { 
        //        DateCreated = DateTime.Now,
        //        Email = "test@email.com",
        //        Message = "testing message"
        //    });

        //    return Json(new { });
        //}
    }
}