using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Mvc;
using System.Security.Claims;
using Omu.StoreAwesome.WebUI.ViewModels.Input;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Utils.Attributes;
using Omu.StoreAwesome.WebUI.Core.Service;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class AdminController : Controller
    {
        AccountService accountService;

        public AdminController(AccountService accountService)
        {
            this.accountService = accountService;
        }

        [Admin]
        public IActionResult Panel()
        {
            return View();
        }

        [Admin]
        public IActionResult Advanced()
        {
            return View();
        }

        public IActionResult SignIn()
        {
            if (User.Identity.IsAuthenticated && User.IsInRole("admin"))
            {
                return RedirectToAction("Panel");
            }

            return View();
        }

        public new async Task<IActionResult> SignOut()
        {
            await HttpContext.SignOutAsync();
            return RedirectToAction("Index", "Home");
        }

        [HttpPost]
        public async Task<IActionResult> SignIn(SignInInput input)
        {
            if (!ModelState.IsValid)
            {
                return View();
            }

            var user = await accountService.AuthUser(input.Login, input.Password);

            //TODO: Warning: done, for learning/demo purposes only
            if (input.Login == "admin" && input.Password == "1")
            {
                user = new User
                {
                    Login = input.Login,
                    UserRoles = new List<UserRole>
                    {
                        new UserRole
                        {
                            Role = new Role { Name = "admin" }
                        }
                    }
                };
            }

            if (user == null)
            {
                ModelState.AddModelError("", "Login or password incorrect, try Login: admin and Password: 1");
                return View();
            }

            await HttpContext.SignOutAsync();

            var claims = new List<Claim>
            {
                new Claim(ClaimTypes.Name, user.Login),
            };

            foreach (var ur in user.UserRoles)
            {
                claims.Add(new Claim(ClaimTypes.Role, ur.Role.Name));
            }

            var claimsIdentity = new ClaimsIdentity(
                claims, CookieAuthenticationDefaults.AuthenticationScheme);

            var authProperties = new AuthenticationProperties
            {
                //AllowRefresh = <bool>,
                // Refreshing the authentication session should be allowed.

                //ExpiresUtc = DateTimeOffset.UtcNow.AddMinutes(10),
                // The time at which the authentication ticket expires. A 
                // value set here overrides the ExpireTimeSpan option of 
                // CookieAuthenticationOptions set with AddCookie.

                IsPersistent = input.Remember,
                // Whether the authentication session is persisted across 
                // multiple requests. When used with cookies, controls
                // whether the cookie's lifetime is absolute (matching the
                // lifetime of the authentication ticket) or session-based.

                //IssuedUtc = <DateTimeOffset>,
                // The time at which the authentication ticket was issued.

                RedirectUri = input.ReturnUrl
                // The full path or absolute URI to be used as an http 
                // redirect response value.
            };

            await HttpContext.SignInAsync(new ClaimsPrincipal(claimsIdentity), authProperties);

            if (input.ReturnUrl != null) return Redirect(input.ReturnUrl);
            return RedirectToAction("Panel", "Admin");
        }
    }
}