using Microsoft.AspNetCore.Mvc;
using Serilog;
using System.Text.Json;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.StoreAwesome.WebUI.Utils.Attributes;
using Omu.StoreAwesome.WebUI.Utils.Logs;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    [Admin]
    public class AdminLogsController : Controller
    {
        public ActionResult Index()
        {
            var lrepo = new LogsRepo(Globals.LogsPath);
            var dates = lrepo.GetLoggedDates();

            ViewData["dates"] = dates;

            return View();
        }

        public IActionResult GetLogs(DateTimeOffset date, int page = 1, int pageSize = 100)
        {            
            var lrepo = new LogsRepo(Globals.LogsPath);            

            var dates = lrepo.GetLoggedDates();
            
            var logDay = dates.FirstOrDefault(o => o.CreationDate.Date == date.Date);

            if (logDay == null)
            {
                return Json(new object[] { });
            }
            
            var logs = lrepo.GetLogsFor(logDay, pageSize, page);

            var res = new List<LogItem>();
            foreach (var item in logs)
            {
                if (string.IsNullOrWhiteSpace(item)) continue;

                try
                {
                    res.Add(JsonSerializer.Deserialize<LogItem>(item));
                }
                catch (Exception ex)
                {
                    Log.Error(ex, ex.Message);
                }
            }

            return Json(res);
        }

        public IActionResult Test()
        {
            Log.Information("info test log recorded");
            Log.Error(new Exception(), "error test log recorded");
            return Content(Globals.LogsPath);
        }
    }

    public class LogItem
    {
        public DateTimeOffset Timestamp { get; set; }

        public string Level { get; set; }

        public string SpanId { get; set; }

        public string TraceId { get; set; }

        public string MessageTemplate { get; set; }

        public object Properties { get; set; }
    }
}