using Microsoft.AspNetCore.Mvc;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.AwesomeMvc;
using Omu.StoreAwesome.WebUI.Utils.Attributes;
using Microsoft.EntityFrameworkCore;
using Omu.StoreAwesome.WebUI.Core.Repo;
using Omu.StoreAwesome.WebUI.Core.Data;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    [Admin]
    public class AdminOrdersController : Controller
    {
        private readonly GenericBaseRepo<Order> repo;        

        public AdminOrdersController(GenericBaseRepo<Order> repo)
        {            
            this.repo = repo;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Order o)
        {
            return new
            {
                o.Id,
                Date = o.DateCreated.ToShortDateString(),
                o.Email,
                SubTotal = UiUtils.FormatPrice(o.SubTotal),
                o.FirstName,
                o.LastName,
                o.City,
                State = MemDb.GetStateName(o.State),
                o.StreetAddress
            };
        }

        public async Task<IActionResult> GridGetData(GridParams g)
        {            
            var query = repo.GetAll()
                .AsQueryable();

            var model = new GridModelBuilder<Order>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel,
            };

            return Json(await model.EFBuildAsync());
        }

        public async Task<IActionResult> Details(int id)
        {
            var order = await repo.FindAsync(id);

            return PartialView(order);
        }

        public async Task<IActionResult> OrderItemsGridGetData(GridParams g, int orderId)
        {            
            var query = repo.GetAll<OrderItem>()
            //.Include(o => o.Product)
            .Where(ordItem => (int)ordItem.OrderId == orderId)
            .AsQueryable();

            var model = new GridModelBuilder<OrderItem>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = o => new { 
                    o.Id, 
                    o.Quantity, 
                    UnitPrice = UiUtils.FormatPrice(o.UnitPrice),               
                    Product = o.ProductName                    
                }
            };

            return Json(await model.EFBuildAsync());
        }
    }
}