using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Omu.Awem.Helpers;
using Omu.StoreAwesome.WebUI.Core.Service;
using Omu.StoreAwesome.WebUI.Utils;
using System.Linq;
using System.Security.Policy;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class CartController : Controller
    {
        private readonly ProductService prodSrv;

        public CartController(ProductService prodSrv)
        {
            this.prodSrv = prodSrv;
        }

        public IActionResult Index()
        {
            return View();
        }

        public IActionResult OpenCart()
        {
            return PartialView();
        }

        public async Task<IActionResult> GetItems(OrderItem[] oitems)
        {
            var res = new List<object>();
            var removed = new List<object>();

            decimal cartTotal = 0;
            int count = 0;
            
            var ids = oitems.Select(o => o.ProductId).ToArray();
            
            var prods = await prodSrv.GetActiveProductsQuery().Where(p => ids.Contains((int)p.Id)).ToArrayAsync();

            foreach(var prod in prods)
            {
                var oitem = oitems.First(o => o.ProductId == prod.Id);

                var itemTotal = oitem.Quantity * prod.Price;
                res.Add(new
                {
                    prod.Id,
                    prod.Name,
                    Picture = UiUtils.ProdFirstThumbPartPath(prod),
                    prod.Price,
                    PriceStr = UiUtils.FormatPrice(prod.Price),
                    UrlName = prod.UrlFriendlyName(),
                    prod.Version,
                    Quantity = oitem.Quantity,
                    Total = itemTotal
                });

                count += oitem.Quantity;
                cartTotal += itemTotal;
            }
            
            removed.AddRange(ids.Except(prods.Select(o => (int)o.Id)).Cast<object>());

            return Json(
                new
                {
                    Items = res,
                    Total = UiUtils.FormatPrice(cartTotal),
                    Count = count,
                    Removed = removed
                });
        }

        public class OrderItem
        {
            public int ProductId { get; set; }

            public int Quantity { get; set; }
        }
    }
}