using Microsoft.AspNetCore.Mvc;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel;
using Omu.StoreAwesome.WebUI.Utils.Attributes;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.StoreAwesome.WebUI.Core.Service;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    [Admin]
    public class ChangePasswordController : Controller
    {
        AccountService accountService;

        public ChangePasswordController(AccountService accountService)
        {
            this.accountService = accountService;
        }

        public IActionResult Index()
        {
            return PartialView();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<IActionResult> Index(ChangePasswordInput input)
        {
            if (!ModelState.IsValid) return PartialView();

            if (input.ConfirmNewPassword != input.NewPassword)
            {
                ModelState.AddModelError("", "New Password doesn't match Confirm password");
            }

            var res = await accountService.ChangePassword(input.Login, input.CurrentPassword, input.NewPassword);

            if (!res)
            {
                ModelState.AddModelError("CurrentPassword", "Login and password mismatch");
            }

            if (!ModelState.IsValid)
            {
                return PartialView(input);
            }

            return Json(new { });
        }
    }

    [PropertiesMustMatch("NewPassword", "ConfirmNewPassword", ErrorMessage = "The password and confirmation password do not match.")]
    public class ChangePasswordInput
    {
        [Required]        
        public string Login { get; set; }

        [Required]
        [DisplayName("Current Password")]
        [UIHint("Password")]
        public string CurrentPassword { get; set; }

        [Required]
        [MinLength(8, ErrorMessage = "Must be at least 8 characters")]
        [MaxLength(20, ErrorMessage = "Maximum 20 characters")]
        [RegularExpression(RegexRules.PwdRule, ErrorMessage = "The following characters are allowed in your password: a-z, A-Z, 0-9, and common punctuation characters.")]
        [DisplayName("New Password")]
        [UIHint("Password")]
        public string NewPassword { get; set; }

        [Required]
        [DisplayName("Confirm New Password")]
        [UIHint("Password")]
        public string ConfirmNewPassword { get; set; }
    }
}