using Microsoft.AspNetCore.Mvc;
using System.ComponentModel.DataAnnotations;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.StoreAwesome.WebUI.Utils.Attributes;
using Omu.StoreAwesome.WebUI.Core.Repo;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class ContactUsController : Controller
    {        
        private readonly GenericBaseRepo<ContactUsMessage> repo;

        public ContactUsController(GenericBaseRepo<ContactUsMessage> repo)
        {
            this.repo = repo;
        }

        public IActionResult ContactUsPart()
        {
            return PartialView();
        }

        [HttpPost]
        [RecaptchaValidator]
        [ValidateAntiForgeryToken]
        public async Task<IActionResult> ContactUsPart(ContactUsMessageInput input, bool captchaValid)
        {
            if (!ModelState.IsValid) return PartialView(input);

            if (!captchaValid)
            {
                ModelState.AddModelError("captcha", "security check not valid, please try again");
                return PartialView(input);
            }

            await saveAsync(input);            

            return PartialView("ThankYouPart");
        }

        private async Task saveAsync(ContactUsMessageInput input)
        {
            await repo.AddAsync(new ContactUsMessage { 
                Email = input.Email,
                Message = input.Message,
                DateCreated = DateTime.UtcNow
            });            
        }
    }

    public class ContactUsMessageInput
    {
        [Required]
        [RegularExpression(RegexRules.Email, ErrorMessage = "Incorrect email format")]
        [StringLength(100)]
        public string Email { get; set; }

        [Required]
        [StringLength(700, MinimumLength = 30, ErrorMessage = "The Message length should be between 30 and 700 characters")]
        public string Message { get; set; }
    }
}