using Microsoft.AspNetCore.Mvc;
using MongoDB.Driver;
using Omu.AwesomeMvc;
using Omu.StoreAwesome.WebUI.Core.Data;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Core.Service;
using Omu.StoreAwesome.WebUI.Utils;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class DataController : Controller
    {        
        private readonly ProductService prodSrv;

        public DataController(ProductService productService)
        {     
            this.prodSrv = productService;
        }

        public async Task<IActionResult> ProductsAutoccomplete(string term = "") // v is the entered text
        {
            term = term.Trim();
            
            var terms = UiUtils.GetKeywords(term);

            var prods = await prodSrv.GetProductsByWords(terms);
            
            return Json(prods.Select(o => new { k = o.Id, c = o.Name, title = UiUtils.UrlFriendly(o.Name) }));
        }

        public async Task<IActionResult> ProductsGrid(GridParams g, string search)
        {
            var query = prodSrv.GetActiveProductsQuery();

            query = query.FilterProductBySearch(search);

            query = query.OrderByDescending(o => o.Priority).ThenByDescending(o => o.Id);
            var gmb = new GridModelBuilder<Product>(query, g)
            {
                KeyProp = o => o.Id,
                DefaultKeySort = Sort.None,
                Map = p =>
                {
                    var pricew = Math.Truncate(p.Price);
                    var pricef = (p.Price % 1 * 100).ToString("00");

                    return new
                    {
                        p.Id,
                        p.Name,
                        Picture = UiUtils.ProdFirstThumbPartPath(p),
                        p.Price,
                        Pricef = pricef,
                        Pricew = pricew,
                        UrlName = p.UrlFriendlyName()
                    };
                }
            };

            return Json(await gmb.EFBuildAsync());
        }

        public IActionResult GetStates()
        {
            var res = MemDb.States.Select(o => new KeyContent(o.Abbreviation, o.Name));

            return Json(res);
        }
    }
}