using Microsoft.AspNetCore.Diagnostics;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using System.Diagnostics;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Models;
using Omu.StoreAwesome.WebUI.Utils;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class HomeController : Controller
    {
        public IActionResult Index()
        {            
            return View();
        }

        public IActionResult Search(string k)
        {
            return View("Index");
        }

        public IActionResult Privacy()
        {
            return View();
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }       
    }

    public class ErrorController : Controller
    {
        public IActionResult Index(Exception error)
        {
            SetMessage(error);

            if (Response.StatusCode == 400)
            {
                var feature = Request.HttpContext.Features.Get<IStatusCodeReExecuteFeature>();
                var path = feature.OriginalPath;                
            }

            if (Request.IsAjaxRequest())
            {
                Response.StatusCode = 500;               

                return Content("Sorry, an unexpected error occurred");
            }

            return View("Error");
        }

        public IActionResult Master(Exception error)
        {
            SetMessage(error);

            return View();
        }

        private IActionResult HttpError404()
        {
            Response.StatusCode = 404;

            if (Request.IsAjaxRequest())
            {
                return Content("404 not found");
            }

            return View("PageNotFound");
        }

        private IActionResult HttpError505()
        {
            return Content("http505 error, try again later");
        }

        private void SetMessage(Exception error)
        {
            ViewData["message"] = Message(error);
        }

        private string Message(Exception ex)
        {
            if (ex == null) return "";
            return ex.Message + "\n" + Message(ex.InnerException);
        }

        public IActionResult HttpStatus(int? code)
        {
            var feature = Request.HttpContext.Features.Get<IStatusCodeReExecuteFeature>();
            var path = feature.OriginalPath;

            if (code == 404)
            {
                Response.StatusCode = 404;

                if (Request.IsAjaxRequest() || path.EndsWith(".map"))
                {
                    return Content("404 not found");
                }

                return HttpError404();
            }
            else if (code == 400)
            {
                Response.StatusCode = 400;

                return Content("400 bad request");
            }
            else if (code == 505)
            {
                if (Request.IsAjaxRequest())
                {
                    return Content("please try again later");
                }

                return HttpError505();
            }

            return Content("");
        }
    }
}