using Microsoft.AspNetCore.Mvc;
using Omu.StoreAwesome.WebUI.Core.Service;
using Omu.StoreAwesome.WebUI.Utils;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class ProductController : Controller
    {
        private readonly ProductService prodSrv;
        
        public ProductController(ProductService prodSrv)
        {
            this.prodSrv = prodSrv;
        }

        public async Task<IActionResult> OpenProduct(int? id, string title)
        {
            var prod = await prodSrv.GetActiveProduct(id);

            if (id is null || prod is null)
            {
                throw new Exception("Product not found");
            }

            var realTitle = UiUtils.UrlFriendly(prod.Name);

            if (realTitle != title)
            {
                return RedirectToAction("OpenProduct", new { id, title = realTitle });
            }

            return View(prod);
        }

        public async Task<IActionResult> GetRichDescModel(int id)
        {
            var prod = await prodSrv.GetActiveProduct(id);

            return Json(prod.RichDescription);
        }
    }
}