using Microsoft.EntityFrameworkCore;
using MongoDB.Driver;

namespace Omu.StoreAwesome.WebUI.Core.Data.Mongo
{
    public class MongoDbFactory
    {
        private MongoDatabaseSettings dbSettings;

        public MongoDbFactory(MongoDatabaseSettings gds)
        {
            dbSettings = gds;
        }

        public IMongoCollection<T> GetCollection<T>()
        {
            return GetDatabase().GetCollection<T>(typeof(T).Name + "s");
        }

        public IMongoDatabase GetDatabase()
        {
            var db = new MongoClient(dbSettings.ConnectionString).GetDatabase(dbSettings.DatabaseName);
            return db;
        }

        public DbContextOptions<MongoDbContext> GetOptions()
        {
            var client = new MongoClient(dbSettings.ConnectionString);
            var db = client.GetDatabase(dbSettings.DatabaseName);

            return new DbContextOptionsBuilder<MongoDbContext>()
                .UseMongoDB(db.Client, db.DatabaseNamespace.DatabaseName)
                .EnableSensitiveDataLogging()
                .Options;
        }

        public MongoDbContext GetContext()
        {
            var client = new MongoClient(dbSettings.ConnectionString);
            var db = client.GetDatabase(dbSettings.DatabaseName);
            return new MongoDbContext(new DbContextOptionsBuilder<MongoDbContext>().UseMongoDB(db.Client, db.DatabaseNamespace.DatabaseName).Options);
        }
    }
}