using MongoDB.Bson;
using MongoDB.Bson.Serialization.Attributes;

namespace Omu.StoreAwesome.WebUI.Core.Model
{
    public class Entity
    {
        public int? Id { get; set; }
        
        /// <summary>
        /// mongodb id
        /// </summary>
        //public ObjectId MongoId { get; set; }
    }    

    public interface IPictureEntity
    {
        public int? Id { get; set; }
        public string Picture { get; set; }
    }

    [BsonIgnoreExtraElements]
    public class Product : Entity, IPictureEntity
    {
        public string Name { get; set; }

        /// <summary>
        /// used for sorting
        /// </summary>
        public int Priority { get; set; }

        public bool IsActive { get; set; }

        public string Description { get; set; }

        public string Picture { get; set; }

        public decimal Price { get; set; }

        public int Version { get; set; }

        public string RichDescription { get; set; }
    }

    public class Order : Entity
    {
        public required string FirstName { get; set; }
        public required string LastName { get; set; }
        public required string Email { get; set; }
        public string StreetAddress { get; set; }
        public string City { get; set; }
        public string State { get; set; }
        public string ZipCode { get; set; }
        public decimal SubTotal { get; set; }
        public DateTime DateCreated { get; set; }

        public ICollection<OrderItem> OrderItems { get; set; }
    }

    public class OrderItem : Entity
    {
        public int? OrderId { get; set; }
        public Order Order { get; set; }

        public int? ProductId { get; set; }
        public Product Product { get; set; }

        public int Quantity { get; set; }
        public decimal UnitPrice { get; set; }
        public string ProductName { get; set; }
    }

    public class ContactUsMessage : Entity
    {
        public string Email { get; set; }

        public string Message { get; set; }

        public DateTime DateCreated { get; set; }

        public bool IsRead { get; set; }
    }

    public class AppInfo : Entity
    {
        public int ProductsListVersion { get; set; }
    }

    public class MongoIds
    {
        public int Id { get; set; }
        public int Value { get; set; }
    }
}