using Microsoft.EntityFrameworkCore;
using Omu.StoreAwesome.WebUI.Core.Model;

namespace Omu.StoreAwesome.WebUI.Core.Repo
{
    public abstract class GenericBaseRepo<T> where T : Entity
    {
        protected virtual DbContext DbCx { get; set; }

        public virtual async Task<T> AddAsync(T entity)
        {
            DbCx.Add(entity);
            await DbCx.SaveChangesAsync();
            return entity;
        }

        public IQueryable<TModel> GetAll<TModel>() where TModel : Entity
        {
            return DbCx.Set<TModel>();
        }

        public IQueryable<T> GetAll()
        {
            return DbCx.Set<T>();
        }

        public virtual async Task<T> FindAsync(object id)
        {
            if (id is int)
            {
                return await DbCx.Set<T>().FirstOrDefaultAsync(o => (int)o.Id == (int)id);
            }

            return await DbCx.FindAsync<T>(id);
        }

        public async Task<T> UpdateAsync(object id, Action<T> updateAction)
        {
            var prod = await FindAsync(id);

            updateAction(prod);
            await DbCx.SaveChangesAsync();

            return prod;
        }

        public async Task Remove(object id)
        {
            var ent = await FindAsync(id);
            DbCx.Remove(ent);
            await DbCx.SaveChangesAsync();
        }

        public virtual async Task BatchUpdateAsync(IEnumerable<object> ids, Action<T> updateAction)
        {
            foreach (var id in ids)
            {
                var prod = await FindAsync(id);

                updateAction(prod);
            }

            await DbCx.SaveChangesAsync();
        }

        public virtual async Task BatchRemoveAsync(IEnumerable<object> ids)
        {
            foreach (var id in ids)
            {
                var prod = await FindAsync(id);

                DbCx.Remove(prod);
            }

            await DbCx.SaveChangesAsync();
        }
    }
}