using Microsoft.EntityFrameworkCore;
using MongoDB.Driver;
using Omu.StoreAwesome.WebUI.Core.Data.Mongo;
using Omu.StoreAwesome.WebUI.Core.Model;

namespace Omu.StoreAwesome.WebUI.Core.Repo
{
    public class GenericMongoRepo<T> : GenericBaseRepo<T> where T : Entity
    {
        private readonly MongoDbFactory mongoDbContextFactory;
        private MongoDbContext mongoDbCx;

        public GenericMongoRepo(MongoDbContext mcx, MongoDbFactory mongoDbContextFactory)
        {
            mongoDbCx = mcx;
            this.mongoDbContextFactory = mongoDbContextFactory;
        }

        protected override DbContext DbCx { get => mongoDbCx; }

        private async Task<MongoIds> reserveIds(int count = 1)
        {
            var idsColl = mongoDbContextFactory.GetCollection<MongoIds>();

            var filter = Builders<MongoIds>.Filter
                .Eq(o => o.Id, 1);

            var update = Builders<MongoIds>.Update.Inc(o => o.Value, count);

            var opt = new FindOneAndUpdateOptions<MongoIds, MongoIds> { ReturnDocument = ReturnDocument.After, IsUpsert = true };

            var res = await idsColl.FindOneAndUpdateAsync(filter, update, opt);
            return res;
        }

        public override async Task<T> AddAsync(T entity)
        {
            var mongoIds = await reserveIds();
            entity.Id = mongoIds.Value;

            if (entity is Order order)
            {
                int? count = order.OrderItems?.Count;

                if (count > 0)
                {
                    var orderId = (await reserveIds(count.Value)).Value;

                    foreach (var ordItem in order.OrderItems.Reverse())
                    {
                        ordItem.Id = orderId;
                        orderId--;

                        ordItem.OrderId = orderId;
                        ordItem.Order = order;
                    }
                }
            }

            DbCx.Add(entity);
            await DbCx.SaveChangesAsync();
            return entity;
        }

        public async Task InitIds()
        {
            var idsColl = mongoDbContextFactory.GetCollection<MongoIds>();
            var filter = Builders<MongoIds>.Filter
                .Eq(o => o.Id, 1);

            var updateSetInitValue = Builders<MongoIds>.Update.Max(o => o.Value, 100);

            var opt = new FindOneAndUpdateOptions<MongoIds, MongoIds> { ReturnDocument = ReturnDocument.After, IsUpsert = true };

            await idsColl.FindOneAndUpdateAsync(filter, updateSetInitValue, opt);
        }
    }
}