using Microsoft.EntityFrameworkCore;
using MongoDB.Driver;
using Omu.StoreAwesome.WebUI.Core.Data.Mongo;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Utils;
using System.Linq.Expressions;

namespace Omu.StoreAwesome.WebUI.Core.Repo
{
    public class ProdMongoRepo : IProdRepo
    {
        private readonly MongoDbFactory mongoDbContextFactory;

        public ProdMongoRepo(MongoDbFactory mongoDbContextFactory)
        {
            this.mongoDbContextFactory = mongoDbContextFactory;
        }

        public async Task<IEnumerable<Product>> GetProductsByWords(string[] words)
        {
            if (words == null || words.Length == 0) return Enumerable.Empty<Product>();
            words = words.Select(o => o.ToLower()).ToArray();

            var coll = mongoDbContextFactory.GetCollection<Product>();

            Expression<Func<Product, string>> propExpr = o => o.Name;

            var filter = Builders<Product>.Filter
                .Where(ExprUtil.ContainsWordsToLowerAndExpr(words, propExpr));

            var res = await coll.Find(filter).Limit(5).ToListAsync();

            return res;
        }
    }
}