using Omu.StoreAwesome.WebUI.Utils.Attributes;
using Omu.StoreAwesome.WebUI.Core.Model;
using Microsoft.EntityFrameworkCore;
using Omu.StoreAwesome.WebUI.Core.Repo;

namespace Omu.StoreAwesome.WebUI.Core.Service
{
    public class AccountService
    {
        private GenericBaseRepo<User> userRepo;

        private Hasher hasher = new Hasher();

        public AccountService(GenericBaseRepo<User> userRepo)
        {
            this.userRepo = userRepo;
        }

        public async Task<User> AuthUser(string login, string pass)
        {
            var user = await userRepo.GetAll().SingleOrDefaultAsync(o => o.Login == login);
            if (user is not null && hasher.CompareStringToHash(pass, user.Password))
            {
                user.UserRoles = new List<UserRole>
                    {
                        new UserRole
                        {
                            Role = new Role { Name = "admin" }
                        }
                    };
            }

            return user;
        }

        public async Task<bool> ChangePassword(string login, string currentPassword, string pass)
        {
            // TODO: note embeded admin user for demo
            if (login == "admin")
            {
                var admin = await userRepo.GetAll().SingleOrDefaultAsync(o => o.Login == "admin");

                if (admin is null)
                {
                    admin = new User { Login = "admin", Password = hasher.Encrypt("1") };
                    await userRepo.AddAsync(admin);
                }
            }

            var user = await AuthUser(login, currentPassword);

            if (user is not null)
            {
                await userRepo.UpdateAsync(user.Id, u =>
                {
                    u.Password = hasher.Encrypt(pass);
                });

                return true;
            }

            return false;
        }
    }
}