using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Core.Repo;
using Omu.StoreAwesome.WebUI.Models;
using Omu.StoreAwesome.WebUI.Utils;

namespace Omu.StoreAwesome.WebUI.Core.Service
{
    public class ProductService
    {
        private IProdRepo prodRepo;

        private readonly GenericBaseRepo<Product> repo;

        private readonly IAppInfoRepo appInfoRepo;

        public ProductService(IProdRepo prodRepo, IAppInfoRepo appInfoRepo, GenericBaseRepo<Product> repo)
        {
            this.prodRepo = prodRepo;
            this.appInfoRepo = appInfoRepo;
            this.repo = repo;
        }

        public async Task<int> GetProductsListVersion()
        {
            return (await appInfoRepo.GetAsync()).ProductsListVersion;
        }

        public async Task<Product> UpdateAsync(object id, Action<Product> updateAction)
        {
            var prod = await repo.UpdateAsync(id, prod =>
            {
                updateAction(prod);
                prod.Version++;
            });

            await appInfoRepo.IncProductsListVersion();

            return prod;
        }

        public async Task<Product> Save(ProductInput input)
        {
            var isEdit = input.Id.HasValue;

            var prod = isEdit ? await repo.FindAsync(input.Id) : new Product();

            prod.Name = input.Name;
            prod.Price = input.Price;
            prod.Description = input.Description;
            prod.IsActive = input.IsActive;
            prod.Priority = input.Priority;

            if (isEdit)
            {
                // edit
                prod.Version++;
            }
            else
            {
                // create
                prod.Version = 1;
                prod = await repo.AddAsync(prod);
            }

            var appInfo = await appInfoRepo.IncProductsListVersion();

            Globals.ProductsListVersion = appInfo.ProductsListVersion;

            return prod;
        }

        public async Task<Product> FindAsync(object id)
        {
            return await repo.FindAsync(id);
        }

        public IQueryable<Product> GetActiveProductsQuery()
        {
            return repo.GetAll<Product>().Where(o => o.IsActive);
        }

        public IQueryable<Product> GetProducts()
        {
            return repo.GetAll<Product>();
        }

        public async Task<Product> GetActiveProduct(object id)
        {
            var prod = await repo.FindAsync(id);
            if (prod != null && !prod.IsActive)
            {
                return null;
            }

            return prod;
        }

        public async Task<IEnumerable<Product>> GetProductsByWords(string[] words)
        {
            return await prodRepo.GetProductsByWords(words);
        }
    }
}