using Microsoft.AspNetCore.Html;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.AspNetCore.Mvc.Routing;
using Microsoft.AspNetCore.Mvc;
using Omu.StoreAwesome.WebUI.Models;

namespace Omu.StoreAwesome.WebUI.Helpers
{
    public static class Helpers
    {
        private static IUrlHelper GetUrlHelper(IHtmlHelper html)
        {
            return ((IUrlHelperFactory)html.ViewContext.HttpContext.RequestServices
                .GetRequiredService(typeof(IUrlHelperFactory)))
                .GetUrlHelper(html.ViewContext);
        }

        public static IHtmlContent ProductBreadCrumb(this IHtmlHelper html)
        {
            var url = GetUrlHelper(html);
            var qs = html.ViewContext.HttpContext.Request.QueryString;
            
            var res = $"<a class='pd-breadc' href='{url.Action("Index", "Home")}{qs}'>‹ Back to results</a>";

            return new HtmlString(res);

        }

        public static IHtmlContent BreadCrumbs(this IHtmlHelper html)
        {
            var action = html.ViewContext.RouteData.Values["Action"].ToString();
            var controller = html.ViewContext.RouteData.Values["Controller"].ToString();

            var currentNode = AdminSiteMap.GetAll().FirstOrDefault(o => o.Action == action && o.Controller == controller);
            if (currentNode is null) return new HtmlString(string.Empty);

            var result = new List<string>();

            addNode(currentNode, result, html);

            if (currentNode.Parent is not null)
            {
                var node = currentNode.Parent;
                var url = GetUrlHelper(html);

                result.Add($"<a class='awe-btn' href='{url.Action(node.Action, node.Controller)}'>Back</a>");
            }
                        
            if (!result.Any())
            {
                return null;
            }

            result.Reverse();
            return new HtmlString(result.First() + " " + string.Join(" / ", result.Skip(1)));
        }

        private static void addNode(SiteMapItem node, List<string> prevRes, IHtmlHelper html)
        {
            var url = GetUrlHelper(html);
            if (node.IsText)
            {
                prevRes.Add($"<span>{node.Name}</span>");
            }
            else if (node.RenderOnlyWhenInChain && prevRes.Count == 0)
            {
                return;
            }
            else
            {
                prevRes.Add($"<a href='{url.Action(node.Action, node.Controller)}'>{node.Name}</a>");
            }

            if (node.Parent is not null)
            {
                addNode(node.Parent, prevRes, html);
            }
        }
    }
}