using Microsoft.AspNetCore.Mvc.Filters;
using Newtonsoft.Json;

namespace Omu.StoreAwesome.WebUI.Utils.Attributes
{
    public class RecaptchaValidatorAttribute : ActionFilterAttribute
    {
        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            var result = false;
            var kname = "g-recaptcha-response";

            if (filterContext.HttpContext.Request.Form.ContainsKey(kname))
            {
                var response = filterContext.HttpContext.Request.Form[kname];

                var remoteip = filterContext.HttpContext.Connection.RemoteIpAddress;

                using var client = new HttpClient();

                var reply = client.GetStringAsync(
                    string.Format("https://www.google.com/recaptcha/api/siteverify?secret={0}&response={1}&remoteip={2}",
                        Globals.Config.CaptchaSecret, response, remoteip)).Result;

                var captchaResponse = JsonConvert.DeserializeObject<CaptchaResponse>(reply);
                result = captchaResponse.Success;

            }

            filterContext.ActionArguments["captchaValid"] = result;

            base.OnActionExecuting(filterContext);
        }
    }

    public class CaptchaResponse
    {
        [JsonProperty("success")]
        public bool Success { get; set; }

        [JsonProperty("error-codes")]
        public IList<string> ErrorCodes { get; set; }
    }
}