using Microsoft.EntityFrameworkCore;
using Omu.AwesomeMvc;
using Omu.StoreAwesome.WebUI.Core.Data;
using Omu.StoreAwesome.WebUI.Core.Model;

namespace Omu.StoreAwesome.WebUI.Utils
{
    public static class EFAweUtil
    {
        public static async Task DeleteAsync<T>(this SqlDbContext mcx, object key) where T : Entity
        {
            var entity = await mcx.FindAsync<T>(key);

            if (entity == null)
            {
                return;
            }

            mcx.Remove(entity);

            await mcx.SaveChangesAsync();
        }

        public static async Task<GridModelDto<T>> EFBuildAsync<T>(this GridModelBuilder<T> gmb) where T : Entity
        {
            var model = await BuildModelAsync(gmb);

            return model.ToDto();
        }

        public static async Task<GridModel<T>> BuildModelAsync<T>(GridModelBuilder<T> gmb) where T : Entity
        {
            var query = gmb.Query;

            // usual request
            if (gmb.GridParams.Key == null)
            {
                gmb.ItemsCount = await query.CountAsync();

                gmb.PageItems = await gmb.GetPage(gmb.OrderBy(query)).ToArrayAsync();
            }

            // api.update
            else if (gmb.GetItemAsync == null)
            {
                // GridParams.Key is set on: client api.update calls, expand lazy node                
                var key = Convert.ToInt32(gmb.GridParams.Key);

                // * in case of a TreeGrid query might contain only the roots and not the key, 
                // specify the gmb.GetItemAsync before before calling EFBuildAsync in that case

                var res = await query.Where(o => o.Id == key).ToListAsync();
                gmb.PageItems = res;
                gmb.ItemsCount = res.Count();
            }

            return await gmb.BuildModelAsync();
        }

        /// <summary>
        /// Build AjaxListResult (used by the Lookup helpers also)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="query">data query</param>
        /// <param name="keyContentFunc">from item to KeyContent function</param>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <returns></returns>
        public static async Task<AjaxListResult> BuildListAsync<T>(
            IQueryable<T> query,
            int? page = null,
            int pageSize = 10,
            Func<T, KeyContent> keyContentFunc = null,
            Func<T[], string> setContentFunc = null)
        {
            var count = await query.CountAsync();

            var items = await (page == null ? query : AweUtil.GetPage(query, page.Value, pageSize))
                .ToArrayAsync();

            var res = new AjaxListResult();

            if (page != null)
            {
                res.More = count > page.Value * pageSize;
            }

            if (setContentFunc != null)
            {
                res.Content = setContentFunc(items);
            }
            else
            {
                res.Items = items.Select(keyContentFunc);
            }

            return res;
        }
    }
}