using Microsoft.AspNetCore.Mvc;

namespace Omu.StoreAwesome.WebUI.Utils.Logs
{
    public class LogsRepo
    {
        private string logsDirPath;

        public LogsRepo(string logsDirPath)
        {
            this.logsDirPath = logsDirPath;
        }

        public IEnumerable<LogDay> GetLoggedDates()
        {
            var res = new List<LogDay>();

            var dirInfo = new DirectoryInfo(logsDirPath);

            if (!dirInfo.Exists) return res;

            var files = dirInfo.GetFiles();

            var groups = dirInfo.GetFiles()
                .Where(o => o.Name.StartsWith("log") && o.Extension == ".json")
                .GroupBy(o => o.CreationTime.Date);

            foreach (var group in groups)
            {
                res.Add(new LogDay
                {
                    CreationDate = group.Key,
                    Files = group
                    .Select(o => new LogFile
                    {
                        FullName = o.FullName,
                        LastWriteTime = o.LastWriteTime
                    })
                });
            }

            return res;
        }

        public IEnumerable<string> GetLogsFor(LogDay logDay, int pageSize, int page)
        {
            var skip = (page - 1) * pageSize;
            var lpageInfo = new LogPageInfo { Skip = skip, Take = pageSize };

            var result = new List<string>();
            foreach (var logFile in logDay.Files)
            {
                if (lpageInfo.Take == 0)
                {
                    break;
                }

                getLogs(logFile.FullName, result, lpageInfo);
            }

            return result;
        }

        void getLogs(string path, List<string> result, LogPageInfo lpageInfo)
        {
            string[] lines;
            using (FileStream fs = new FileStream(path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
            {
                using (StreamReader sr = new StreamReader(fs))
                {
                    string all = sr.ReadToEnd();
                    lines = all.Split('\n');
                }
            }

            if (lines.Length > lpageInfo.Skip)
            {
                lines = lines.Skip(lpageInfo.Skip).ToArray();

                if (lines.Length < lpageInfo.Take)
                {
                    result.AddRange(lines);
                    lpageInfo.Take -= lines.Length;
                }
                else
                {
                    result.AddRange(lines.Take(lpageInfo.Take));
                    lpageInfo.Take = 0;
                }
            }
            else
            {
                lpageInfo.Skip -= lines.Length;
            }
        }

        public class LogPageInfo
        {
            public int Skip { get; set; }

            public int Take { get; set; }
        }

        public class LogDay
        {
            public DateTime CreationDate { get; set; }

            public IEnumerable<LogFile> Files { get; set; }

        }

        public class LogFile
        {
            public string FullName { get; set; }

            public DateTime LastWriteTime { get; set; }
        }
    }
}