var richEditor = function (mainOpt) {
    var mainNode = mainOpt.root;

    var nodeRefs = {};

    function pnlBtnClick() {
        var btn = $(this);
        var act = btn.data('act');

        if (act == 'addText') {
            mainNode.children.push({ type: 'text', text: '' });
        }
        else if (act == 'addRichText') {
            mainNode.children.push({ type: 'richText', text: '' });
        }
        else if (act == 'addImage') {
            openPicturesPicker(function (name) { mainNode.children.push({ type: 'img', opt: { name: name } }); });
        }
        else if (act == 'addYTHtml') {
            mainNode.children.push({ type: 'htmlyt', text: '' });
        }

        renderEditor();
        renderPreview();
    }

    function openPicturesPicker(action) {
        awe.open(
            {
                id: 'picturesPickerPopup',
                setCont: function (sp, o) {
                    o.scon.html($('#picturesPicker'));
                },
                close: function () {
                    $('#picturesPickerCont').append($('#picturesPicker'));
                },
                title: 'select a picture',
                modal: true,
                onSelectAction: action
            });
    }

    function initSelectPicture() {
        $(document).on('click', '.btnSelectPic', function () {
            var name = $(this).closest('.awe-row').data('k');
            var popup = $(this).closest('.awe-popup');
            var o = popup.data('o');
            o.onSelectAction(name);

            renderEditor();
            renderPreview();
            popup.data('api').close();
        });
    }

    function initBtnPnl() {
        $('#btnPanel').on('click', 'button', pnlBtnClick);
    }

    function bindEditorToPreview() {
        $('#editor').on('keyup change', 'textarea', function (e) {            
            var input = $(this);
            
            var n = $(this).closest('.edCompCont').data('ref');
            var node = nodeRefs[n];
            node.text = input.val();

            renderPreview();
        }).on('summernote.change', 'textarea', function () {
            var input = $(this);
            var n = $(this).closest('.edCompCont').data('ref');
            var node = nodeRefs[n];
            node.text = input.val();

            renderPreview();
        });

        $('#editor')
            .on('click', '.addProdImage', function () {
                var cont = $(this).closest('.edCompCont');
                openPicturesPicker(function (name) {
                    var url = sitePictures.prodPicUrl(mainOpt.productId, name);
                    var txt = cont.find('textarea.value');
                    var id = txt.attr('id');
                    console.log(url, id);

                    setTimeout(function () {
                        $('#' + id).summernote("insertImage", url);
                    });

                });
            })
            .on('click', '.moveUp', function () {                
                var cont = $(this).closest('.edCompCont');
                var n = cont.data('ref');
                var node = nodeRefs[n];
                var parent = nodeRefs['parent_' + n];
                var arr = parent.children;
                awef.loop(arr, function (inode, i) {
                    if (inode == node) {
                        if (!i) return false;
                        arr[i] = arr[i - 1];
                        arr[i - 1] = node;
                        return false;
                    }
                });

                renderEditor();
                renderPreview();
            })
            .on('click', '.moveDown', function () {
                var cont = $(this).closest('.edCompCont');
                var n = cont.data('ref');
                var node = nodeRefs[n];
                var parent = nodeRefs['parent_' + n];
                var arr = parent.children;
                awef.loop(arr, function (inode, i) {
                    if (inode == node) {
                        if (i == arr.length - 1) return false;
                        arr[i] = arr[i + 1];
                        arr[i + 1] = node;
                        return false;
                    }
                });

                renderEditor();
                renderPreview();
            })
            .on('click', '.delete', function () {
                var cont = $(this).closest('.edCompCont');
                var n = cont.data('ref');
                var node = nodeRefs[n];
                console.log(node);
                var parent = nodeRefs['parent_' + n];

                cont.addClass('highlight');
                awe.flash(cont);

                awe.open(
                    {
                        setCont: function (sp, o) {
                            var cont = $('<div style="padding: 0 3em;">Are you sure you want to delete this ?</div>');
                            o.scon.html(cont);
                        },
                        btns: [
                            {
                                text: 'Yes',
                                ok: 1, // add okbtn class
                                action: function () {
                                    $(this).data('api').close();

                                    parent.children = parent.children.filter(function (elm) { return elm != node; });

                                    renderEditor();
                                    renderPreview();
                                }
                            },
                            {
                                text: 'No',
                                action: function () {
                                    cont.removeClass('highlight');
                                    $(this).data('api').close();
                                }
                            }],
                        height: 200,
                        width: 700,
                        title: 'please confirm',
                        modal: true
                    });
            });
    }

    function renderEditor() {
        nodeRefs = {};
        $('#editor').html(renderNode({ node: mainNode, edit: true, n: 1 }));

        $('#editor .richText textarea').each(function (_, elm) {
            $(elm).summernote({
                toolbar: [
                    ['style', ['style']],
                    ['font', ['bold', 'underline', 'clear']],
                    ['fontname', ['fontname']],
                    ['color', ['color']],
                    ['para', ['ul', 'ol', 'paragraph']],
                    ['table', ['table']],
                    ['insert', ['link', 'picture', 'video', 'hr']],
                    ['view', ['fullscreen', 'codeview', 'help']],
                ],
            });
        });
    }

    function renderPreview() {
        $('#preview').html(renderNode({ node: mainNode }));
    }

    function renderNode(opt) {
        var node = opt.node;        
        var rendMainOpt = { productId: mainOpt.productId };

        if (opt.edit) {
            rendMainOpt.renderers = editRends;
        }

        return richTextRenderer(rendMainOpt).renderNode({ node: node, n: 1 });
    }

    var editRends = {
        'cont': function (opt) {
            var node = opt.node;
            var res = '<div class="edcont">'

            if (node.children) {
                awef.loop(node.children, function (childNode) {
                    res += opt.renderNode({ node: childNode, parent: node, n: ++opt.n });
                });
            }

            res += '</div>';

            return res;
        },
        'text': function (opt) {            
            var node = opt.node;            
            var res = '<div class="edCompCont" data-ref="' + opt.n + '">';
            res = addStandardTools(res);
            res += '<textarea>' + node.text + '</textarea>';            
            res += '</div>';
            
            setRefs(opt);
            return res;            
        },
        'richText': function (opt) {
            var n = opt.n;
            var node = opt.node;            
            var res = '<div class="edCompCont richText" data-ref="' + n + '">';
            res = addStandardTools(res, btn({ content: 'add image', class: 'addProdImage' }));
            res += '<textarea id="richText' + n + '" class="value">' + node.text + '</textarea>';            
            res += '</div>';
            setRefs(opt);
            return res;
        },
        'img': function (opt) {
            var n = opt.n;
            var node = opt.node;
            var url = sitePictures.prodPicUrl(mainOpt.productId, node.opt.name);

            var res = '<div class="edCompCont img" data-ref="' + n + '">';
            res = addStandardTools(res);
            res += '<img src="' + url + '" />';            
            res += '</div>';
            setRefs(opt);            
            return res;
        },
        'htmlyt': function (opt) {
            var node = opt.node;
            var res = '<div class="edCompCont htmlCont" data-ref="' + opt.n + '">';
            res = addStandardTools(res);
            res += '<textarea>' + node.text + '</textarea>';
            res += '</div>';

            setRefs(opt);
            return res;
        },
    };    

    function setRefs(opt) {
        nodeRefs[opt.n] = opt.node;
        nodeRefs['parent_' + opt.n] = opt.parent;
    }

    function addStandardTools(res, addBtns) {
        res += '<div class="edToolbar">';
        res += btn({ content: 'delete', class: 'delete' });
        res += btn({ content: '<i class="gg-arrow-up"></i>', class: 'moveUp' });
        res += btn({ content: '<i class="gg-arrow-down"></i>', class: 'moveDown' });

        if (addBtns) {
            res += addBtns;
        }

        res += '</div>';
        return res;
    }

    function btn(opt) {
        return '<button type="button" class="awe-btn ' + opt.class + '">' + opt.content + '</button>';
    }

    function init() {
        renderEditor();
        renderPreview();
        bindEditorToPreview();
        initBtnPnl();
        initSelectPicture();
    }

    return {
        init: init
    };
};