var cart = function () {
    var version = 2;
    function getParams() {
        var items = getCartSortedItemsForSrv();

        var res = [];

        awef.loop(items, function (item, i) {
            res.push({ name: "[" + i + "].productId", value: item.id });
            res.push({ name: "[" + i + "].quantity", value: item.q });
        });

        return res;
    }

    function add(id, q) {
        q = parseInt(q) || 1;
        var c = loadCartFromStorage();
        var item = c.items[id];
        if (!item) {
            item = { q: q, id: id };
            c.items[id] = item;
        }
        else {
            item.q += q;
        }

        item.order = c.order;

        c.order++;

        save(c);
        cartChange();
    };

    function clear() {
        save(newCart());
        cartChange();
    }

    function remove(id) {
        var c = loadCartFromStorage();
        delete c.items[id];

        save(c);
        cartChange({ remove: 1 });
    }

    function save(c) {
        localStorage.setItem("cart", JSON.stringify(c));
    }

    function newCart() {
        return { items: {}, order: 1, vers: version };
    }

    function loadCartFromStorage() {
        var c = localStorage.getItem("cart");
        if (c) {
            c = JSON.parse(c);
        }

        if (!c || !c.items || c.vers < version) {
            c = newCart();
        }

        return c;
    }

    function getCount() {
        var c = loadCartFromStorage();

        var sum = 0;
        awef.loop(Object.values(c.items), function (itm) {
            sum += itm.q;
        });

        return sum;
    }

    function cartChange(opt) {
        res.onCartChange && res.onCartChange(opt);
    }

    function getTotal() {
        var c = loadCartFromStorage();

        var total = 0;

        if (!itemsHaveData(c)) {
            return total;
        }

        awef.loop(Object.values(c.items), function (item) {

            total += item.ditm.Price * item.q;
        });

        return total;
    }

    function itemsHaveData(c) {
        var res = 1;
        awef.loop(Object.values(c.items), function (item) {
            if (!item.ditm) {
                res = 0;
                return false; // break
            }
        });

        return res;
    }

    function getDataFromUrl() {
        return $.get('/cart/getItems', getParams())
            .then(function (res) {
                var c = loadCartFromStorage();

                var removed = [];
                if (res.Removed && res.Removed.length) {
                    awef.loop(res.Removed, function (remId) {
                        var remItem = c.items[remId];
                        remItem.Removed = 1;
                        removed.push(remItem);
                    });
                }

                var newItems = {};
                var maxOrder = 0;
                awef.loop(res.Items, function (ritm, i) {
                    newItems[ritm.Id] = {
                        id: ritm.Id,
                        ditm: ritm,
                        order: i,
                        q: ritm.Quantity
                    };
                    maxOrder = i;
                });

                c.order = maxOrder + 1;
                c.items = newItems;
                c.removed = removed;

                save(c);

                return res;
            });
    }

    function getCartSortedItemsForSrv(cc) {
        var c1 = cc || loadCartFromStorage();
        var items = Object.values(c1.items);
        items.sort(function (a, b) { return a.order - b.order; });
        return items;
    }

    function getCartSortedItems(cc) {
        var c1 = cc || loadCartFromStorage();
        var items = Object.values(c1.items);
        items.sort(function (a, b) { return a.order - b.order; }).reverse();

        if (c1.removed) {
            items = c1.removed.concat(items);
        }

        return items;
    }

    function loadData(forceLoad, initResFunc) {
        if (!cart.lastDataLoad) forceLoad = 1;

        var c = loadCartFromStorage();
        if (itemsHaveData(c) && !forceLoad) {
            return getCartSortedItems(c);
        }

        return $.when(getDataFromUrl())
            .then(function (res) {
                cart.lastDataLoad = Date.now();
                initResFunc && initResFunc(res);
                return getCartSortedItems();
            });
    }

    var res = {
        clear: clear,
        getCount: getCount,
        add: add,
        remove: remove,
        lastDataLoad: null,
        load: loadCartFromStorage,
        loadData: loadData,
        getItemsParams: getParams,
        getTotal: getTotal,
        update: function (id, q) {
            var c = loadCartFromStorage();
            var item = c.items[id];
            item.q = parseInt(q);
            save(c);

            cartChange();
        }
    };

    return res;
}();

var site = function () {
    var cartPopCfg;

    function updateCartCount() {
        var elm = $('.cartCount');
        var count = cart.getCount();
        elm.html(count);
    }

    function subtotalFormat(count, total) {
        return 'Subtotal (' + count + ' items): $' + total
    }


    function addToCart(id, q) {
        cart.add(id, q);

        if (cartPopCfg) {
            if (!cartPopCfg.cx.isOpen) {
                cartPopCfg.cx.api.open();
            }

            return;
        }

        openCartPopup($('#btnCart'));
    }

    function initCornerCartHover() {
        if (window.noCornerCartHover) return;

        var btnCart = $('#btnCart');

        // open on cart hover
        var hoverApi = awem.hovPopup({
            hover: btnCart,
            name: 'cartPopup',
            stop: function () {
                if (cartPopCfg && cartPopCfg.cx.isOpen) {
                    return true;
                }
            },
            open: function (opener) {
                var popup = openCartPopup(opener, hoverApi);

                popup.on('click', function () {
                    hoverApi.stopClose();
                });

                return popup;
            }
        });
    }

    function openCartPopup(opener, hoverApi) {
        var popup = awe.open({
            id: 'cartPopup',
            opener: opener,
            dropdown: 1,
            noFocusFirst: 1,
            marg: 3, // top opener marg
            //url: openCartUrl,
            setCont: function (sp, o) {
                o.scon.append($('#cartCont').show());
                initCart();
            },
            loadOnce: 1,
            popupClass: 'o-phov cornerCartPop',
            beforeClose: function (opt) {
                if (!opt.initialEvent) return;
                var trg = $(opt.initialEvent.target);

                if (trg.is('.addToCartAct')) {
                    hoverApi && hoverApi.stopClose();
                    opt.noclose = 1;
                }
            }
        });

        return popup;
    }


    function getCartData(sgp, forceLoad, initResFunc) {
        var gp = aweUtils.getGridParams(sgp);
        var opt = {
            gp: gp,
            key: "Id",
            map: function (citm) {
                var itm = citm.ditm;

                return {
                    Id: itm.Id,
                    Name: itm.Name,
                    Picture: itm.Picture,
                    Quantity: citm.q,
                    Price: itm.Price,
                    PriceStr: itm.PriceStr,
                    UrlName: itm.UrlName,
                    Total: itm.Total,
                    Removed: citm.Removed
                };
            }
        };

        return $.when(cart.loadData(forceLoad, initResFunc))
            .then(function (data) {
                opt.items = data;

                return aweUtils.gridModelBuilder(opt);
            });
    }

    function checkoutData(sgp) {
        return getCartData(sgp,
            1,
            function (res) {
                $('.checkoutTotal').html(subtotalFormat(res.Count, res.Total));
            });
    }

    function initCart() {
        $('#cartGrid').on('change', '.awe-row .awe-val', function () {
            // change cart item quantity
            var newq = $(this).val();
            var row = $(this).closest('.awe-row');
            var key = row.data('k');
            cart.update(key, newq);
        }).on('click', '.crtDel', function () {
            // remove cart item
            var row = $(this).closest('.awe-row');
            var key = row.data('k');
            cart.remove(key);
            var nameLink = row.find('.crtItmName');
            var msg = $('<p> was just removed from the shopping cart</p>').prepend(nameLink);
            row.empty().append(msg).addClass('rem');
        })
            .data('api').load();
    }

    function regResponsiveSearchBox() {
        setSrcBox();
        $(window).on('resize', setSrcBox);
        function setSrcBox() {
            var srcField = $('.searchField');
            var minipnl = $('.miniSearchPnl');
            var srcpnl = $('.searchPnl');

            if ($(window).width() < 900) {
                !srcField.closest(minipnl).length && minipnl.append(srcField);
            }
            else {
                !srcField.closest(srcpnl).length && srcpnl.append(srcField);
            }
        }
    }

    function prodUrl(item) {
        return "/" + item.UrlName + "/prod/" + item.Id;
    }

    function prodCacheSearch(o, info) {
        if (info.count >= 5) return [];

        var term = info.term;
        var c = info.cache;

        c.termsUsed = c.termsUsed || {};
        c.nrterms = c.nrterms || []; // no result terms

        if (c.termsUsed[term]) return [];
        c.termsUsed[term] = 1;

        // ignore terms that are contain nr terms
        var nores = 0;
        $.each(c.nrterms, function (i, val) {
            if (term.indexOf(val) >= 0) {
                nores = 1;
                return false;
            }
        });

        if (nores) {
            return [];
        }

        var prm = awe.params(o);
        prm.push({ name: "term", value: term });

        info.gotData = function (data) {
            cstorg.set('searchAutocompl', data);
        };

        return awe.ajx({ url: o.tag.Url, data: prm }).then(function (data) {
            if (!data || !data.length) {
                c.nrterms.push(term);
            }

            return data;
        }).fail(function () { c.termsUsed[term] = 0; });
    }

    function getWordsFilter(query) {
        var queryUp = query.toUpperCase();
        var qwords = queryUp.split(" ");
        var contains = awem.strContainsU;

        return function (content) {
            var cwords = content.split(" ");
            var matches = 0;
            awef.loop(qwords, function (qw) {
                var found = 0;
                awef.loop(cwords, function (cw) {
                    if (contains(cw, qw)) {
                        found = 1;
                    }
                });

                if (found) {
                    matches++;
                }
            });

            return matches === qwords.length;
        }
    }

    function updateCartRelatedDivs() {
        var count = cart.getCount();
        var coBox = $('.proceedChkBox, .btnPopupCheckout');
        if (count) {
            coBox.show();
        }
        else {
            coBox.hide();
        }

        $('.cartTotal').html(subtotalFormat(cart.getCount(), cart.getTotal().toFixed(2)));
    }

    function loadCaptcha(id) {
        var rcdiv = $('#' + id);
        if (!rcdiv.length || rcdiv.html()) return;

        var key = rcdiv.data('sitekey');
        grecaptcha.render(id, {
            'sitekey': key
        });

        rcdiv.closest('.awe-popup').data('o').cx.api.lay();
    }

    function disb(elm) {
        elm.attr('disabled', 'disabled');
    }

    function disbFormSubmit(sel) {
        var btn = $(this).find('[type=submit]');
        $(sel).on('awebeforepost', function () {
            disb(btn);
        }).on('aweafterpost', function () {
            btn.removeAttr('disabled');
        });
    }

    function openContactUs() {
        $(document).on('aweload', '#contactus', function () {
            site.loadCaptcha('recaptcha1');    
        });

        awe.open({
            url: '/contactus/contactuspart',
            id: 'contactus',            
            type: 'popupForm',
            title: 'Contact us',
            modal: 1,
            height: 200,
            maxw: 730,
            outClickClose: 1
        });
    }

    function scrollTo(el) {
        var offset = el.offset();    
        var nval = offset.top - 150;        
        if ($(window).scrollTop() > nval) {
            $(window).scrollTop(nval);
        }        
    }

    function prodsScrollTopOnLoad() {
        var gridProds = $('#gridProducts');
        gridProds.on('aweload', function () {
            setTimeout(function () {
                scrollTo(gridProds);
            }, 10);
        });        
    }

    function searchBoxNav() {
        var searchInput = $('#storeSearch');
        searchInput.on('aweselect', function (e, item) {
            goToProd(item);

        }).on('awecomboenterup', function (e, data) {
            var item = data.item;
            if (item) {
                goToProd(item);
                return;
            }

            searchByText(data.text);
        });
                
        $('#searchBtn').on('click', function () {
            searchByText(searchInput.val());
        });

        function searchByText(text) {
            if (!text) {
                location.href = '/';
                return;
            };

            location.href = '/s?k=' + text;
        }
         
        function goToProd(item) {
            var url = prodUrl({ UrlName: item.title, Id: item.k });
            location.href = url;
        }

        var modal = $('<div class="o-pmodal o-pu" tabindex="-1" id="storeSearch-modal" style="display: none;"></div>');

        $(document).on('focus', '#storeSearch-awed', function () {            
            openSearchModal();            
        });

        $(document).on('aweopen', '#storeSearch-dropmenu', function () {
            if (!modal.is(':visible')) {
                openSearchModal();
            }
        });

        function openSearchModal() {            
            var scont = $('.searchField');
            var sinput = $('.searchInput');

            modal.css('left', 0);
            scont.append(modal);

            modal.css('z-index', 2);
            sinput.css('z-index', 3);
            modal.show();

            $(this).one('focusout', function (e) {
                if ($(e.relatedTarget).closest('.o-menu').length) return;                
                modal.remove();
                sinput.css('z-index', '');
            })
        }
    }

    function getQuantityData() {
        var res = [];
        for (var i = 1; i <= 30; i++) {
            res.push({ k: i, c: i });
        }
        return res;
    }

    return {
        getQuantityData: getQuantityData,        
        loadCaptcha: loadCaptcha,
        disbFormSubmit: disbFormSubmit,
        getWordsFilter: getWordsFilter,
        prodCacheSearch: prodCacheSearch,
        prodUrl: prodUrl,
        addToCart, addToCart,
        updateCartCount: updateCartCount,
        getCartData: getCartData,
        checkoutData: checkoutData,
        initCart: initCart,
        init: function () {
            prodsScrollTopOnLoad();

            $('#feedback').click(openContactUs);

            initCornerCartHover();
            site.updateCartCount();
            updateCartRelatedDivs();

            cart.onCartChange = function (opt) {
                site.updateCartCount();

                updateCartRelatedDivs();

                var cartGrid = $('#cartGrid');

                if (cartGrid.length && !(opt && opt.remove)) {
                    var o = cartGrid.data('o');
                    if (o.lrs) {
                        o.api.load();
                    }
                }
            };

            // link button
            $(document).on('click', '.a-lnk-btn', function () {
                var url = $(this).data('url');
                if (!url) return;
                location.href = url;
            })

            // search box
            searchBoxNav();

            regResponsiveSearchBox();
        }
    };
}();

$(function () {
    site.init();
})