﻿using System.Web;
using System.Web.Mvc;
using System.Web.UI;
using Omu.AwemWebForms.Helpers;
using Omu.AwesomeWebForms;
using WebFormsMinSetupDemo.Utils;

namespace WebFormsMinSetupDemo.Helpers.Awesome
{
    public static class PageCrudHelpers
    {
        private static UrlHelper GetUrlHelper(Page page)
        {
            return page.Url();
        }

        public static IHtmlString InitCrudForGridNest(this Page page, string gridId, string crudController)
        {
            var url = GetUrlHelper(page);

            var result =
                page.Awe()
                    .InitPopupForm()
                    .Name("create" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Create", crudController))
                    .Mod(o => o.Inline().ShowHeader(false))
                    .Success("aweUtils.itemCreated('" + gridId + "')")
                    .ToString()
                + page.Awe()
                    .InitPopupForm()
                    .Name("edit" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Edit", crudController))
                    .Mod(o => o.Inline().ShowHeader(false))
                    .Success("aweUtils.itemEdited('" + gridId + "')")
                + page.Awe()
                    .InitPopupForm()
                    .Name("delete" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Delete", crudController))
                    .Mod(o => o.Inline().ShowHeader(false))
                    .Success("aweUtils.itemDeleted('" + gridId + "')");

            return new MvcHtmlString(result);
        }

        public static IHtmlString InitDeletePopupForGrid(
            this Page page,
            string gridId,
            string crudController = null,
            string action = "Delete")
        {
            var url = GetUrlHelper(page);

            var result =
                page.Awe()
                    .InitPopupForm()
                    .Name("delete" + gridId)
                    .Group(gridId)
                    .Url(url.Action(action, crudController))
                    .Success("aweUtils.itemDeleted('" + gridId + "')")
                    .OnLoad("aweUtils.delConfirmLoad('" + gridId + "')") // calls grid.api.select and animates the row
                    .Height(200)
                    .Modal()
                    .ToString();

            return new MvcHtmlString(result);
        }

        /*beging*/
        public static IHtmlString InitCrudPopupsForGrid(
            this Page page,
            string gridId,
            string crudController,
            int createPopupHeight = 430,
            int maxWidth = 0)
        {
            var url = GetUrlHelper(page);
            
            var result =
                page.Awe()
                    .InitPopupForm()
                    .Name("create" + gridId)
                    .Group(gridId)
                    .Height(createPopupHeight)
                    .MaxWidth(maxWidth)
                    .Url(url.Action("Create", crudController))
                    .Title("Create item")
                    .Modal()
                    .Success("aweUtils.loadOnSave('" + gridId + "')")
                    .ToString()

                + page.Awe()
                    .InitPopupForm()
                    .Name("edit" + gridId)
                    .Group(gridId)
                    .Height(createPopupHeight)
                    .MaxWidth(maxWidth)
                    .Url(url.Action("Edit", crudController))
                    .Title("Edit item")
                    .Modal()
                    .Success("aweUtils.loadOnSave('" + gridId + "')")

                + page.Awe()
                    .InitPopupForm()
                    .Name("delete" + gridId)
                    .Group(gridId)
                    .Url(url.Action("Delete", crudController))
                    .Success("aweUtils.loadOnDelete('" + gridId + "')")
                    .OnLoad("aweUtils.delConfirmLoad('" + gridId + "')") // calls grid.api.select and animates the row
                    .Height(200)
                    .Modal();

            return new MvcHtmlString(result);
        }
        /*endg*/
    }
}