﻿using Omu.AwemWebForms.Helpers;
using Omu.AwesomeWebForms;
using System.Collections.Generic;
using System.Web;
using System.Web.Script.Serialization;
using System.Web.UI;

namespace WebFormsMinSetupDemo.Utils.Awesome
{
    public static class AweWfUtils
    {
        private const string editContent = "Edit";

        private static string GetPopupName(string action, string gridId)
        {
            return action + gridId;
        }

        public static IHtmlString InlineCreateButtonForGrid(this Page page, string gridId, object parameters = null, string text = "Create")
        {
            var parms = new JavaScriptSerializer().Serialize(parameters);

            return page.Awe().Button()
                .Text(text)
                .OnClick(string.Format("$('#{0}').data('api').inlineCreate({1})", gridId, parms));
        }

        public static string InlineDeleteFormatForGrid(this Page page, string gridId, string key = "Id", bool nofocus = false, string text = "Cancel")
        {
            var popupName = GetPopupName("delete", gridId);

            return GridUtils.DeleteFormat(popupName, key, btnClass: "o-glh", nofocus: nofocus)
                   + page.Awe()
                       .Button()
                       .Text(text)
                       .CssClass("o-glcanb awe-nonselect o-gl o-glbtn");
        }

        public static Column EditColumnForGrid(this Page page, string gridId)
        {
            return new Column { ClientFormat = page.EditFormatForGrid(gridId), Width = GridUtils.EditBtnWidth }.Mod(o => o.Nohide());
        }

        public static Column DeleteColumnForGrid(this Page html, string gridId)
        {
            return new Column { ClientFormat = html.DeleteFormatForGrid(gridId), Width = GridUtils.DeleteBtnWidth }.Mod(o => o.Nohide());
        }

        public static Column InlEditColumn<T>()
        {
            return new Column { ClientFormat = GridUtils.InlineEditFormat(), Width = GridUtils.EditBtnWidth }.Mod(o => o.Nohide());
        }

        public static string DeleteFormatForGrid(this Page html, string gridId, string key = "Id", bool nofocus = false)
        {
            gridId = html.Awe().GetContextPrefix() + gridId;

            return GridUtils.DeleteFormatForGrid(gridId, key, nofocus);
        }

        private static string GetPopupName(this Page html, string action, string gridId)
        {
            return action + html.Awe().GetContextPrefix() + gridId;
        }

        public static string EditFormatForGrid(this Page html, string gridId, string key = "Id", bool setId = false, bool nofocus = false, int? height = null)
        {
            var popupName = html.GetPopupName("edit", gridId);

            var click = html.Awe().OpenPopup(popupName).Params(new { id = $".({key})" });
            if (height.HasValue)
            {
                click.Height(height.Value);
            }

            var button = html.Awe().Button()
                .CssClass("awe-nonselect editbtn")
                .HtmlAttributes(new { aria_label = "edit" })
                .Text(editContent)
                .OnClick(click);

            var attrdict = new Dictionary<string, object>();

            if (setId)
            {
                attrdict.Add("id", $"gbtn{popupName}.{key}");
            }

            if (nofocus)
            {
                attrdict.Add("tabindex", "-1");
            }

            button.HtmlAttributes(attrdict);

            return button.ToString();
        }

        public static Column InlEditColumn(this Page html)
        {
            return new Column { ClientFormat = GridUtils.InlineEditFormat(), Width = GridUtils.EditBtnWidth }.Mod(o => o.Nohide());
        }

        public static Column InlDeleteColumn(this Page html, string gridId)
        {
            return new Column { ClientFormat = html.InlineDeleteFormatForGrid(gridId), Width = GridUtils.DeleteBtnWidth }.Mod(o => o.Nohide());
        }
    }
}