import { Component, forwardRef, ViewChild, Input, AfterViewInit, OnDestroy } from '@angular/core';
import { NG_VALUE_ACCESSOR, ControlValueAccessor } from '@angular/forms'
import { awe, awem, aweui, aweUtils } from 'src/js/aweui/all.js';
import * as $ from 'jquery';

@Component({
  selector: 'awe-input',
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => AweInputComponent),
      multi: true
    }],
  templateUrl: './awe-input.component.html',
  styleUrls: ['./awe-input.component.css']
})
export class AweInputComponent implements ControlValueAccessor, AfterViewInit, OnDestroy {

  isDisabled = false;

  // used to generate unique ids
  static autoinc: number = 1;

  // component options
  // { func: funcName, opt: funcOpt, [id: 'input id'] }
  // func name  (e.g. aweui.radioList)
  // func options, setting options separately so we could set the opt.id before executing func
  // input html id
  @Input() copt;

  @ViewChild('location') location;
  id = 'aweid' + AweInputComponent.autoinc++;
  onChange;
  onTouched;

  private o;

  ngAfterViewInit(): void {    
    var compOpt = this.copt;
    if (!compOpt) {
      console.warn('[copt] component options not specified');
      return;
    }

    this.id = compOpt.id || this.id;
    var $inp = $(this.location.nativeElement);   

    $inp.attr('id', this.id);

    if (compOpt.func === aweui.datepicker || compOpt.func === aweui.autocomplete) {
      $inp.attr('type', 'text');
    }

    var self = this;
    $inp.change(() => {
      self.onChange && self.onChange($inp.val());
    });

    var onTouched = this.onTouched;

    if (compOpt.func) {
      var funcOpt = compOpt.opt || {};

      // timeout to execute after the ngModel values are set in the html, needed to get parent values (cascade)
      setTimeout(() => {
        funcOpt.id = this.id;
        funcOpt.enb = !this.isDisabled;
        compOpt.func(funcOpt);

        self.o = $inp.data('o')

        var field = $inp.closest('.awe-field');

        if (!field.length) {
          // autocomplete has no awe-field
          field = $inp;
        }

        field.on('aweblur blur focusout', e => {
          setTimeout(() => {
            if (onTouched) {
              // dropdownList, combobox menu open
              if (field.find('.awe-focus').length || field.find(':focus').length) {
                return;
              }

              // lookup
              var lpopup = $(':focus').closest('.o-pmc');
              if (lpopup.length) {
                if (lpopup.data('i') === this.id + '-awepw') {
                  lpopup.one('aweclose', onTouched);
                  return;
                }
              }

              onTouched();
            }
          });

        });
      });
    }
  }

  ngOnDestroy(): void {
    awe.destroy(this.o);    
  }

  writeValue(value): void {
    var $inp = $('#' + this.id);
    $inp.val(value);
    var api = $inp.data('api');
    if (api) {
      api.render();
    }
  }

  registerOnChange(fn: any): void {
    this.onChange = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouched = fn;
  }

  setDisabledState?(isDisabled: boolean): void {
    this.isDisabled = isDisabled;

    var $inp = $('#' + this.id);
    var api = $inp.data('api');

    if (api) {
      api.enable(!isDisabled);
    }
  }
}