import { Component, OnInit } from '@angular/core';
import { FormControl, FormBuilder, Validators } from '@angular/forms'
import { awe, awem, aweui, aweUtils } from 'src/js/aweui/all.js';
import { dataCon } from 'src/js/cons/dataCon.js';
import { site } from 'src/js/site.js';
import * as $ from 'jquery';

@Component({
  selector: 'app-form-inputs',
  templateUrl: './form-inputs.component.html',
  styleUrls: ['./form-inputs.component.css']
})
export class FormInputsComponent implements OnInit {
  constructor(private fb: FormBuilder) { }

  form1 = this.fb.group({
    meal: ['', Validators.required],
    meal2: ['', Validators.required],
    mealMulti: ['', Validators.required],
    catRadioLst:['', Validators.required],
    catChkLst: ['', Validators.required],
    date1: ['', Validators.required],
    num1: ['', Validators.required],
    autoc: ['', Validators.required],
    catBtng: ['', Validators.required],
    mealLookup: ['', Validators.required],
    mealMultiLookup: ['', Validators.required],
    chk1: ['', Validators.required],
    otoggl1: ['', Validators.required]
  });

  vm;
  
  mealOddOpt = {
    func: aweui.dropdownList,
    opt: {
      dataFunc: dataCon.getAllMeals,      
      itemFunc: site.imgItem,
      captionFunc: aweUtils.imgCaption      
    }
  };

  meal2Opt = {
    func: aweui.combobox,
    opt: {
      dataFunc: dataCon.getAllMeals,      
      itemFunc: site.imgItem      
    }
  };

  mealMultiOpt = {
    func: aweui.multiselect,
    opt: {
      dataFunc: dataCon.getAllMeals,      
      itemFunc: site.imgItem,
      captionFunc: aweUtils.imgCaption      
    }
  };

  catRadioLstOpt = {
    func: aweui.radioList,
    opt: {
      dataFunc: dataCon.getCategories,      
    }
  };

  catChkLstOpt = {
    func: aweui.checkboxList,
    opt: {
      dataFunc: dataCon.getCategories,      
    }
  };

  date1Opt = {
    func: aweui.datepicker
  };

  num1Opt = {
    func: aweui.txt,
    opt: {
      numeric: true
    }
  };

  autocOpt = {
    func: aweui.autocomplete,
    opt: {
      itemFunc: site.imgItem,
      dataFunc: dataCon.getMealsAutocomplete,
      placeholder: 'try o'
    }
  };

  catBtngOpt = {
    func: aweui.radioList,
    opt: {
      dataFunc: dataCon.getCategories,
      buttongroup: true
    }
  };

  mealLookupOpt = {
    func: aweui.lookup,
    opt: {
      dataFunc: {
        get: dataCon.getChef,
        search: dataCon.searchChef
      }
    }
  };

  mealMultiLookupOpt = {
    func: aweui.multilookup,
    opt: {
      dataFunc: {
        get: dataCon.getChefs,
        search: dataCon.searchChef,
        selected: dataCon.selectedChefs
      }
    }
  };

  // cascade demo
  categoriesOddOpt = {
    id: 'parentCategory',
    func: aweui.dropdownList,
    opt: {
      dataFunc: dataCon.getCategories,      
    }
  };

  childMealOpt = {
    func: aweui.dropdownList,
    opt: {
      dataFunc: dataCon.getMeals,
      parents: [{ name: 'categories', id: 'parentCategory' }]      
    }
  };

  chk1Opt = {
    func: aweui.chk    
  };

  otoggl1Opt = {
    func: aweui.toggle    
  };
  
  get ctrls() {
    return this.form1.controls;
  }

  onAweEditorValChange(v) {
    this.vm.meal1 = v;
  }

  ngOnInit(): void {

    let categories = dataCon.getCategories();
    let meals = dataCon.getAllMeals();
    let cat0 = categories[0].k;
    let cat1 = categories[1].k;

    this.vm = {
      cat0: cat0,
      meal1: meals[2].k,

      category1: cat1,
      childMeal: null,
    };
  }
}