import { Component, OnInit } from '@angular/core';
import { awe, awem, aweui, aweUtils } from 'src/js/aweui/all.js';
import { gridUtils } from 'src/js/infra/gridUtils.js';
import { dinnersInlCrudCon } from 'src/js/cons/dinnersInlCrudCon.js';
import { dataCon } from 'src/js/cons/dataCon.js';
import * as $ from 'jquery';

@Component({
  selector: 'app-grid-inline-edit',
  templateUrl: './grid-inline-edit.component.html',
  styleUrls: ['./grid-inline-edit.component.css']
})
export class GridInlineEditComponent implements OnInit {

  inlineEditGridOpt = {
    id: 'inlineEditGrid1',
    dataFunc: dinnersInlCrudCon.grid,
    height: 350,    
    mod: {
      inlineEdit: {
        rowClickEdit: false,
        dataFunc: {
          create: dinnersInlCrudCon.create,
          edit: dinnersInlCrudCon.edit
        },
        initRow: dinnersInlCrudCon.initRowVld
      }
    },
    columns: [
      {
        bind: 'id',
        inline: {
          func: aweui.readonly
        },
        width: 75
      },
      {
        bind: 'name',
        inline: {
          func: aweui.txt
        }
      },
      {
        bind: 'date',
        inline: {
          func: aweui.datepicker
        }
      },
      {
        bind: 'chef.firstName,chef.lastName',
        clientFormat: '.(chefName)',
        header: 'chef',
        minWidth: 170,
        inline: {
          func: aweui.lookup,
          opt: {
            dataFunc: {
              get: dataCon.getChef,
              search: dataCon.searchChef
            }
          },
          valProp: 'chefId', // model property (obj returned by map func) to get editor value
          modelProp: 'chef' // property name used to post value to dinnersInlCrudCon.edit/create
        }
      },
      {
        clientFormat: '.(meals)',
        header: 'meals',
        minWidth: 250,
        inline: {
          func: aweui.checkboxList,
          opt: { multiselect: true, dataFunc: dataCon.getAllMeals },
          valProp: 'mealsIds',
          modelProp: 'meals'
        }
      },
      {
        bind: 'bonusMeal.name',
        clientFormat: '.(bonusMeal)',
        header: 'bonus meal',
        inline: {
          func: aweui.dropdownList,
          opt: { dataFunc: dataCon.getAllMeals },
          valProp: 'bonusMealId'
        }
      },
      {
        bind: 'organic',
        width: 90,
        clientFormat: '.(dispOrganic)',
        inline: {
          func: aweui.toggle          
        }
      },
      { clientFormat: gridUtils.inlineEditFormat(), width: 100 },
      { clientFormat: gridUtils.inlineDeleteFormatForGrid('inlineEditGrid1'), width: 100 }
    ]
  };

  onGridSearch(event) {
    $('#inlineEditGrid1').data('api').load({ params: { search: event.target.value } });
  }

  ngOnInit(): void {
    gridUtils.initDeletePopupForGrid({
      gridId: 'inlineEditGrid1',
      dataFunc: dinnersInlCrudCon.confirmDelete,
      postFunc: dinnersInlCrudCon.del
    });
  }
}