import { Component, OnInit } from '@angular/core';
import { awe, awem, aweui, aweUtils } from 'src/js/aweui/all.js';
import { dataCon } from 'src/js/cons/dataCon.js';
import { site } from 'src/js/site.js';
import * as $ from 'jquery';

@Component({
  selector: 'app-main',
  templateUrl: './main.component.html',
  styleUrls: ['./main.component.css']
})
export class MainComponent implements OnInit {
  model;
  treeDd;

  // cascade demo
  categoriesOddOpt = {
    id: 'parentCategory',
    func: aweui.dropdownList,
    opt: {
      dataFunc: dataCon.getCategories      
    }
  };

  childMealOpt = {
    func: aweui.dropdownList,
    opt: {
      dataFunc: dataCon.getMeals,
      parents: [{ name: 'categories', id: 'parentCategory' }]      
    }
  };

  onGridSearch(event) {
    $('#GridFrow').data('api').load();
    // because '#searchGrid1' is a parent of the grid, on load its value will be sent to the dataFunc
    // alternatively we could remove the "parents" from the aweui.grid and use the api like this:
    // $('#grid1').data('api').load({params: { search: event.target.value }});
  }

  treeDdOpt = {
    func: aweui.dropdownList,
    opt: {
      dataFunc: dataCon.getTreeNodesLazy,      
      collapseNodes: true,
      searchFunc: function (o, info) {
        // don't call remote search when got 3 local results
        //if (info.count > 3) return [];

        return dataCon.searchTree(info.term);
        // you can also return a promise (e.g. $.ajax)
      }      
    }
  };

  filterGridOpt = {
    id: 'GridFrow',
    dataFunc: dataCon.getLunchesFilterGrid,
    height: 390,
    reorderable: true,
    columns: [
      {
        bind: 'Id', width: 75, groupable: false,
        mod: {
          filter: {
            html: function (cx) { return 'search:' }
          }
        }
      },
      {
        bind: 'Person',
        mod: {
          filter: {
            func: aweui.txt,
            opt: {
              clearBtn: true
            }
          }
        }
      },
      {
        bind: 'Food',
        clientFormatFunc: site.imgFood,
        minWidth: 200,
        mod: {
          filter: function (cx) {
            return {
              func: aweui.dropdownList,
              opt: {                
                clearBtn: true,
                itemFunc: site.imgFoodItem,
                captionFunc: site.imgFoodCaption,                
                dataFunc: dataCon.foodGetData
              }
            };
          }
        }
      },
      { bind: 'CountryName', header: 'Country' },
      {
        bind: 'Date', // sort by "Date" prop
        clientFormat: '.(DateStr)', // but display DateStr
        width: 170,
        mod: {
          nohide: true
        }
      },
      {
        clientFormat: '.(MealsStr)',
        header: 'Meals',
        grow: 2,
        mod: {
          filter: {
            type: 'multiselect',
            valProp: 'Meals'
          }
        }
      },
      { bind: "ChefName", header: "Chef" }
    ],
    parents: [
      { id: 'txtGridFrowSrc', name: 'search', load: false }
    ],
    mod: {
      pageSize: true,
      columnsSelector: true,
      pageInfo: true,
      loading: true,
      columnsAutohide: true,
      filterRow: { keyupsel: '.awe-txt' }
    }
  };


  ngOnInit(): void {
    // cascade demo
    let categories = dataCon.getCategories();
    let meals = dataCon.getAllMeals();
    let cat0 = categories[0].k;

    this.model = {
      category1: cat0,
      childMeal: meals[0].k,
    };
  }
}