import { Component, OnInit } from '@angular/core';
import { awe, awem, aweui, aweUtils } from 'src/js/aweui/all.js';
import { site } from 'src/js/site.js';
import * as $ from 'jquery';

@Component({
  selector: 'app-popup',
  templateUrl: './popup.component.html',
  styleUrls: ['./popup.component.css']
})
export class PopupComponent implements OnInit {

  onAreaClick(e) {
    var ev = e.type === 'touchstart' ? e.originalEvent.touches[0] : e;
    awe.open('popupdd1', { xy: { x: ev.clientX, y: ev.clientY } }, ev);
  }

  openSlide(){
    var p1 = awe.open('popup1');
    site.slide(p1);
  }

  ngOnInit(): void {
    aweui.initPopup({
      id: 'popup1',
      setCont: function (sp, o) {
        var msg = $('<div>hi</div>');
        o.scon.html(msg);
        awe.flash(msg);
        // we could also return a promise ( e.g. $.ajax ) that will set o.scon.html after
      },
      //url: 'someurl', // url that returns the popup content
      loadOnce: true
    });

    aweui.initPopup({
      id: 'popupTop',
      content: 'popup content',
      top: true,
      modal: true,
      width: 1000,
      title: 'top modal popup',
      outClickClose: true
    });


    aweui.initPopup({
      id: 'popupParams',
      setCont: function (sp, o) {
        var prm = aweUtils.getParams(sp);
        var msg = $('<div> parameters: ' + JSON.stringify(prm) + '</div>');
        o.scon.html(msg);
      },
      top: true,
      modal: true,
      parents: [{ name: "parent", id: "txtParam1" }],
      parameters: [
        { name: "Id", val: 123 },
        { name: "p1", val: 15 }
      ],
      parameterFunc: setParams,
      outClickClose: true
    });

    aweui.initPopup({
      id: 'popupdd1',
      width: 300,
      setCont: function (sp, o) {
        o.scon.html('dropdown popup');
      },
      loadOnce: true,
      dropdown: true
    });

    aweui.initPopup({
      id: 'popbtns',
      setCont: function (sp, o) {
        o.scon.html('popup with btns');
      },
      close: onClose,
      btns: [
        { text: "Add Hi", action: addHi },
        { text: "Close", action: closePopup }
      ]
    });

    function setParams() {
      return { a: "hi" };
    }

    function closePopup() {
      $(this).data('api').close();
    }

    function addHi() {
      $(this).prepend('<p>hi</p>');
      $(this).data('api').lay();
    }

    function onClose() {
      awe.flash($('#p3Log').html('popup was closed'));
    }

    // popup form

    aweui.initPopupForm({
      id: 'popupForm1',
      height: 200,
      setCont: function (sp, o) {
        var str = '<form>' +
          'name: <input type="text" name="name" />' +
          '</form>';
        o.scon.html(str);
      },
      postFunc: function (sp) {
        var prm = aweUtils.getParams(sp);
        return { name: prm.name };
      },
      success: function (res) {
        alert('name = ' + res.name);
      }
    });
  }
}