import { awedb } from './awedb.js';
import { awef, aweUtils, awem, aweui } from '../aweui/all.js';
import { aweq } from '../infra/aweq.js';

var dataCon = function () {
  var loop = awef.loop;
  var where = aweq.where, takePage = aweq.takePage, select = aweq.select, newKeyCon = aweq.newKeyCon, contains = aweq.contains;
  var toKeyContArr = aweq.toKeyContArr;
  var lstorg = {};

  function buildItemTrees(nodes, lvl, setLazyChild) {
    // builds oitem tree collection from treeNodes collection 
    lvl = lvl || 0;
    var list = [];

    loop(nodes, function (node) {
      var oitem = { k: node.id, c: node.name };
      list.push(oitem);

      var children = where(awedb.treeNodes,
        function (o) {
          return o.parent === node;
        });

      if (children.length) {
        // set node as lazy after lvl 0
        if (setLazyChild && lvl > 0) {
          oitem.l = true;
          oitem.cl = true;
        }
        else {
          // add children
          oitem.it = buildItemTrees(children, lvl + 1, setLazyChild);
        }
      }
    });

    return list;
  }

  function getRoot(node) {
    return awef.isNull(node.parent) ? node : getRoot(node.parent);
  }

  return {
    getThemes: function () {
      return select(
        ['wui', 'mui', 'bts', 'met', 'gui', 'gui3', 'start', 'black-cab'],
        function (o) { return newKeyCon(o, o); });
    },
    getChefs: function (sp) {
      var prm = aweUtils.getParams(sp);
      var res = aweq.getByIds(awedb.chefs, prm.v);

      return select(res, function (o) { return newKeyCon(o.id, o.firstName + ' ' + o.lastName); });
    },
    selectedChefs: function (sp) {
      var prm = aweUtils.getParams(sp);
      var ids = prm.selected || [];
      var list = where(awedb.chefs, function (o) {
        return ids.indexOf(o.id.toString()) > -1;
      });

      return aweui.listRes({
        items: select(list, function (o) { return newKeyCon(o.id, o.firstName + ' ' + o.lastName); }),
      });
    },
    getChef: function (sp) {
      var prm = aweUtils.getParams(sp);
      var res = aweq.getById(awedb.chefs, prm.v);
      return aweq.newKeyCon(res.id, res.firstName + ' ' + res.lastName);
    },
    searchChef: function (sp) {
      var prm = aweUtils.getParams(sp);
      var pageSize = 7;
      var page = prm.page;
      var search = (prm.search || '').toLowerCase().trim();

      // for multilookup
      var selected = prm.selected || [];

      var list = where(awedb.chefs, function (o) {

        return selected.indexOf(o.id.toString()) === -1
          && (o.firstName + ' ' + o.lastName).toLowerCase().indexOf(search) > -1;
      });

      var pglist = takePage(list, page, pageSize);

      return aweui.listRes({
        items: select(pglist, function (o) { return newKeyCon(o.id, o.firstName + ' ' + o.lastName); }),
        more: list.length > page * pageSize
      });
    },
    getMeal: function (sp) {
      var prm = aweUtils.getParams(sp);
      var res = aweq.getById(awedb.meals, prm.v);
      return aweq.newKeyCon(res.id, res.name);
    },
    getCategories: function () {
      return toKeyContArr(awedb.categories, 'id', 'name');
    },
    getAllCountries: function () {
      var list = [{ id: '', name: 'any country' }].concat(awedb.countries);

      return toKeyContArr(list, 'id', 'name');
    },
    getAllMeals: function () {
      // all dataFunc functions instead of returning data, can also return a promise
      // e.g. return $.ajax('myurl').done(function(result) { return toKeyContArr(result, 'id', 'name'); });
      var res = [];
      loop(awedb.meals, function (item) {
        res.push({ k: item.id, c: item.name, url: 'assets/pictures/meals/' + item.name + '.jpg' });
      });
      return res;
    },
    getMeals: function (sp) {
      var prm = aweUtils.getParams(sp);
      var result = [];

      loop(awedb.meals, function (item) {
        if (item.category.id.toString() === prm.categories.toString())
          result.push(item);
      });

      return toKeyContArr(result, 'id', 'name');
    },
    getMealsAutocomplete: function (v) {
      v = v.toLowerCase();
      var items = where(awedb.meals,
        function (o) {
          return aweq.contains(o.name, v);
        });

      return aweq.select(items, function (item) {
        return { k: item.id, c: item.name, url: 'assets/pictures/meals/' + item.name + '.jpg' };
      });
    },
    mealsGrid: function (sgp) {
      var gp = aweUtils.getGridParams(sgp);
      var query = (gp.search || '').toLowerCase();

      var meals = where(awedb.meals,
        function (m) {
          return aweq.contains(m.name, query);
        });

      return aweUtils.gridModelBuilder(
        {
          gp: gp,
          items: meals,
          key: "id"
        });
    },
    getMultiMeals: function (sp) {
      var prm = aweUtils.getParams(sp);
      var selected = prm.v || [];

      var res = aweq.getByIds(awedb.meals, selected);

      return toKeyContArr(res, 'id', 'name');
    },
    mealsGridSrl: function (sgp) {
      var gp = aweUtils.getGridParams(sgp);
      var query = (gp.search || '').toLowerCase();
      var selected = gp.selected || [];

      var meals = where(awedb.meals,
        function (m) {
          return aweq.contains(m.name, query)
            && !aweq.arrContains(selected, m.id);
        });

      return aweUtils.gridModelBuilder(
        {
          gp: gp,
          items: meals,
          key: "id"
        });
    },
    mealsGridSel: function (sgp) {
      var gp = aweUtils.getGridParams(sgp);
      var selected = gp.selected || [];
      var res = aweq.getByIds(awedb.meals, selected);

      return aweUtils.gridModelBuilder(
        {
          gp: gp,
          items: res,
          key: "id"
        });
    },
    treeGrid: function (sgp) {
      var gp = aweUtils.getGridParams(sgp);
      var rootNodes = where(awedb.treeNodes, function (o) { return awef.isNull(o.parent); });

      return aweUtils.gridModelBuilder(
        {
          gp: gp,
          items: rootNodes,
          key: 'id',
          getChildren: function (node, nodeLevel) {
            return where(awedb.treeNodes, function (o) { return o.parent === node });
          },
          map: function (o) { return { name: o.name, id: o.id }; }
        });
    },
    lunchGridPopupFilter: function (sgp) {
      var gp = aweUtils.getGridParams(sgp);
      var list = awedb.lunches;

      // eval search
      if (gp) {
        list = where(list, function (o) {
          if (contains(o.food, gp.food) &&
            contains(o.person, gp.person) &&
            contains(o.location, gp.location) &&
            aweq.equalsn(o.country.id, gp.country) &&
              contains(awem.dateToStr(o.date), gp.date) &&
            aweq.equalsn(o.chef.id, gp.chef)
          ) return true;
        });
      }

      function map(o) {
        return {
          id: o.id,
          person: o.person,
          food: o.food,
          price: o.price,
          location: o.location,
          date: awem.dateToStr( o.date),
          countryName: o.country.name,
          chefName: o.chef.firstName + ' ' + o.chef.lastName
        };
      };

      // build grid model
      var res = aweUtils.gridModelBuilder(
        {
          gp: gp,
          items: list,
          key: "id",
          map: map
        });

      return res;
    },
    foodGetData: function () {
      var tag = this.f.closest('.awe-grid').data('o').lrs.tg;
      return tag.frow['Food'];
    },
    getLunchesFilterGrid: function (sgp) {
      var lunches;
      if (lstorg.lunches) {
        lunches = lstorg.lunches;
      } else {
        lunches = select(awedb.lunches,
          function (o) {
            return {
              Id: o.id,
              Food: o.food,              
              FoodName: o.food,
              FoodPic: o.foodPic,
              Person: o.person,
              CountryName: o.country.name,
              Date: o.date,
              DateStr: awem.dateToStr(o.date),
              Year: o.date.getFullYear(),
              Meals: select(o.meals, function (m) { return { Id: m.id, Name: m.name }; }),
              MealsStr: select(o.meals, function (m) { return m.name; }).join(', '),
              ChefName: o.chef.firstName + ' ' + o.chef.lastName
            };
          });

        lstorg.lunches = lunches;
      }

      var distLstFunc = aweUtils.distLstFunc;
      var gp = aweUtils.getGridParams(sgp, ['meals']); // meals is array
      var data = lunches;

      // outside textbox search (if needed, this is a demo)
      if (gp.search) {
        var words = gp.search.toLowerCase().split(" ");

        data = where(lunches, function (o) {
          var matches = 0;
          loop(words, function (w) {
            if (contains(o.Food, w) || contains(o.Person, w) || contains(o.MealsStr, w) || contains(o.CountryName, w)
              || contains(o.DateStr, w)
              || contains(o.ChefName, w)) matches++;
          });

          return matches === words.length;
        });
      }

      // data used to populate some of the filter row dropdowns
      var frowData = {};

      // filter rules, will be applied in order ( gp.forder ),
      // because we want to get the data for the dropdowns in order of filter application
      var filterRules = {
        Person: function () {
          if (gp.person) {
            data = where(data, function (o) { return contains(o.Person, gp.person) });
          }
        },
        Food: function () {
          var items = aweUtils.distItmsFunc('Food')(data);

          frowData.Food = [{ k: '', c: 'any', url: 'pasta.png' }].concat(select(items, function (o) { return { k: o.Food, c: o.Food, url: o.FoodPic } }));

          if (gp.food) {
            data = where(data, function (o) { return o.Food === gp.food });
          }
        },
        Meals: function () {
          if (gp.meals) {
            data = where(data, function (o) {
              var hasAll = true;

              // check that each of gp.meals is present in o.Meals
              var ids = select(o.Meals, function (m) { return m.Id; });
              loop(gp.meals, function (m) {
                hasAll = hasAll && awef.vcont(m, ids);
              });

              return hasAll;
            });
          }

          // get data after querying this time, to filter the meals dropmenu as well
          var distMeals = [];
          var dict = {};
          loop(data, function (o) {
            loop(o.Meals, function (m) {
              if (!dict[m.Id]) {
                distMeals.push(m);
                dict[m.Id] = 1;
              }
            });
          });

          frowData.Meals = select(distMeals, function (m) { return { k: m.Id, c: m.Name }; });
        },
        CountryName: function () {
          frowData.CountryName = toKeyContent(distLstFunc('CountryName')(data), 'any');
          if (gp.countryName) {
            data = where(data, function (o) { return o.CountryName === gp.countryName });
          }
        },
        ChefName: function () {
          frowData.ChefName = toKeyContent(distLstFunc('ChefName')(data).sort(), 'any');
          if (gp.chefName) {
            data = where(data, function (o) { return o.ChefName === gp.chefName });
          }
        },
        Date: function () {
          frowData.Date = toKeyContent(distLstFunc('Year')(data).sort(), 'all years');
          if (gp.date) {
            data = where(data, function (o) { return o.Year.toString() === gp.date });
          }
        }
      };

      // apply rules in order
      aweUtils.applyFilters({
        rules: filterRules,
        gp: gp
      });

      function toKeyContent(arr, emptyn) {
        var res = [];

        if (emptyn) res.push({ k: '', c: emptyn });

        awef.loop(arr, function (item) {
          res.push({ k: item, c: item });
        });

        return res;
      }

      var model = aweUtils.gridModelBuilder(
        {
          key: "Id",
          gp: gp,
          items: data,

          // replace default group header value for Date column
          getHeaderVal: { Date: function (o) { return o.DateStr; } },
          tag: { frow: frowData }
        });

      return model;
    },
    lunchGrid: function (sgp) {
      var gp = aweUtils.getGridParams(sgp);
      var list = awedb.lunches;

      // eval search
      if (gp.search) {
        var words = gp.search.toLowerCase().split(" ");

        list = where(list, function (o) {
          var matches = 0;
          loop(words, function (w) {
            if (contains(o.food, w) || contains(o.person, w) || contains(o.location, w) || contains(o.country.name, w)
              || contains(awem.dateToStr(o.date), w)
              || contains(o.chef.firstName, w) || contains(o.chef.lastName, w)) matches++;
          });

          return matches === words.length;
        });
      }

      function map(o) {
        return {
          id: o.id,
          person: o.person,
          food: o.food,
          price: o.price,
          location: o.location,
          date: awem.dateToStr( o.date),
          countryName: o.country.name,
          chefName: o.chef.firstName + ' ' + o.chef.lastName,
          organic: o.organic ? 'Yes' : 'No'
        };
      };

      // build grid model
      var res = aweUtils.gridModelBuilder(
        {
          gp: gp,
          items: list,
          key: "id",
          map: map
        });

      return res;
    },
    infGrid: function (sgp) {
        var gp = aweUtils.getGridParams(sgp);
        var list = awedb.lunches;
        var nr = 0;
        function map(o) {
            return {
                nr: nextNr(),
                id: o.id,
                person: o.person,
                food: o.food,
                price: o.price,
                location: o.location,
                date: awem.dateToStr( o.date),
                countryName: o.country.name,
                chefName: o.chef.firstName + ' ' + o.chef.lastName,
                organic: o.organic ? 'Yes' : 'No'
            };
        };

        function nextNr() {
            var p = gp.page - (gp.pgsBef || 0);
            if (p < 1) p = 1;
            return ++nr + (p - 1) * gp.pageSize;
        }

        // build grid model
        var res = aweUtils.gridModelBuilder(
            {
                gp: gp,
                items: list,
                key: "id",
                map: map
            });

        return res;
    },
    getTreeNodesLazy: function (sp) {
      var prm = aweUtils.gparams(sp);
      var key = prm.key;

      var items = awef.isNotNull(key) ?
        // key has value, lazy request
        buildItemTrees(where(awedb.treeNodes, function (o) { return o.id.toString() === key.toString(); })) :
        // initial load, return roots (parent is null)
        buildItemTrees(where(awedb.treeNodes, function (o) { return awef.isNull(o.parent); }), 0, true);
      return items;
    },
    searchTree: function (term) {
      term = (term || '').toLowerCase();
      var result = where(awedb.treeNodes, function (o) { return aweq.contains(o.name, term); });
      var roots = [];
      var ids = [];
      loop(result,
        function (node) {
          var root = getRoot(node);
          if (!aweq.arrContains(ids, root.id.toString())) {
            roots.push(root);
            ids.push(root.id);
          }
        });

      var nodes = buildItemTrees(roots, 0);
      return nodes;
    }
  };
}();

export { dataCon };