import React from 'react';
import { aweui } from '../aweui/aweui.js';
import Aweinp from '../awecomp/Aweinp.js';
import { dataCon } from '../cons/dataCon.js';
import Editor from '../infra/Editor';

export default class FormExample extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            form1: {
                category1: 1014,
                category2: 1014,
                category3: 1014,
                meals1: '',
                childMeal: '',
                mealsLst1: '["1016", "1020"]'
            }
        };

        // react form state demo
        this.resetState = this.resetState.bind(this);
        this.onEditorChange = this.onEditorChange.bind(this);
        this.onCat2Change = this.onCat2Change.bind(this);
        this.handleSubmit = this.handleSubmit.bind(this);
    }

    componentDidMount() {
    }

    resetState() {
        this.setState({
            form1: {
                category1: 1014,
                category2: 1014,
                category3: 1014,
                meals1: '',
                childMeal: '',
                mealsLst1: '["1016", "1020"]'
            }
        });
    }

    handleSubmit(event) {
        event.preventDefault();
        this.setState({ msg: 'form submit: ' + JSON.stringify(this.state.form1) });
    }

    onCat2Change(val) {
        var newstate = { ...this.state };
        newstate.form1.category2 = val;
        this.setState(newstate);
    }

    onEditorChange(name) {
        var ethis = this;
        return function (val) {
            console.log('editor change', name, val);
            var newstate = { ...ethis.state };
            newstate.form1[name] = val;
            ethis.setState(newstate);
        };
    }

    render() {
        const arlOpt = {
            func: aweui.radioList,
            id: 'cat1',
            name: 'cat1',
            dataFunc: dataCon.getCategories
        };

        const oddOpt = {
            func: aweui.dropdownList,
            id: 'cat2id',
            name: 'cat2',
            dataFunc: dataCon.getCategories
        };

        const addOpt = {
            func: aweui.add,
            id: 'cat3id',
            name: 'cat3',
            dataFunc: dataCon.getCategories
        };

        const mealsOddOpt = {
            func: aweui.dropdownList,
            id: 'meals1',
            name: 'meals1',
            dataFunc: dataCon.getAllMeals
        };

        const childMealOpt = {
            func: aweui.dropdownList,
            id: 'childMeal',
            name: 'childMeal',
            dataFunc: dataCon.getMeals,
            parents: [{ name: "categories", id: "cat2id" }],
        };

        const mealsLst1Opt = {
            func: aweui.multiselect,
            id: 'mealsLst1',
            dataFunc: dataCon.getAllMeals
        };

        const form1 = this.state.form1;

        return (
            <div>
                <h2>React form with state example</h2>
                <button type="button" className="awe-btn" onClick={this.resetState}>reset</button>
                <div>
                    Category1: {form1.category1}<br />
                    Category2: {form1.category2}<br />
                    ChildMeal: {form1.childMeal}<br />
                    Category3: {form1.category3}<br />
                    Meals1: {this.state.form1.meals1}<br />
                    Meals List 1: {this.state.form1.mealsLst1}<br />
                </div>
                <form onSubmit={this.handleSubmit}>

                    <Editor label="Category1">
                        <Aweinp opt={arlOpt} value={form1.category1} onChange={this.onEditorChange('category1')} />
                    </Editor>
                    <Editor label="Category2">
                        <Aweinp opt={oddOpt} value={form1.category2} onChange={this.onEditorChange('category2')} />
                    </Editor>
                    <Editor label="Child Meal">
                        <Aweinp opt={childMealOpt} value={form1.childMeal} onChange={this.onEditorChange('childMeal')} />
                    </Editor>
                    <Editor label="Category3">
                        <Aweinp opt={addOpt} value={form1.category3} onChange={this.onEditorChange('category3')} />
                    </Editor>
                    <Editor label="Meals1">
                        <Aweinp opt={mealsOddOpt} value={form1.meals1} onChange={this.onEditorChange('meals1')} />
                    </Editor>
                    <Editor label="Meals list 1">
                        <Aweinp opt={mealsLst1Opt} value={form1.mealsLst1} onChange={this.onEditorChange('mealsLst1')} />
                    </Editor>

                    <button type="submit" className="awe-btn">Submit</button>
                </form>
                <br />
                <div>{this.state.msg}</div>
            </div>
        );
    }
}