import React from 'react';
import * as $ from 'jquery';
import { aweui } from '../aweui/all.js';
import { dataCon } from '../cons/dataCon.js';
import { gridUtils } from '../infra/gridUtils';
import { dinnersInlCrudCon } from '../cons/dinnersInlCrudCon';
import AweGrid from '../awecomp/AweGrid';

export default class GridInlineEdit extends React.Component {
    constructor(props) {
        super(props);

        gridUtils.initDeletePopupForGrid({
            gridId: 'inlineEditGrid1',
            dataFunc: dinnersInlCrudCon.confirmDelete,
            postFunc: dinnersInlCrudCon.del
        });
    }

    componentDidMount() {
    }

    inlEditSearchKeyUp() {
        $('#inlineEditGrid1').data('api').load();
    }

    dinnerInlineCreate() {
        $('#inlineEditGrid1').data('api').inlineCreate();
    }

    render() {
        const inlineEditGrid1Opt = {
            id: 'inlineEditGrid1',
            dataFunc: dinnersInlCrudCon.grid,
            height: 350,
            parents: [
                { id: 'inlEditSearch', name: 'search', load: false }
            ],
            mod: {
                inlineEdit: {
                    rowClickEdit: false,
                    dataFunc: {
                        create: dinnersInlCrudCon.create,
                        edit: dinnersInlCrudCon.edit
                    },
                    initRow: dinnersInlCrudCon.initRowVld,
                    reloadOnSave: false
                }
            },
            columns: [
                {
                    bind: 'id',
                    inline: {
                        func: aweui.readonly
                    },
                    width: 75
                },
                {
                    bind: 'name',
                    inline: {
                        func: aweui.txt
                    }
                },
                {
                    bind: 'date',
                    inline: {
                        func: aweui.datepicker
                    }
                },
                {
                    bind: 'chef.firstName,chef.lastName',
                    clientFormat: '.(chefName)',
                    header: 'chef',
                    minWidth: 170,
                    inline: {
                        func: aweui.lookup,
                        opt: {
                            dataFunc: {
                                get: dataCon.getChef,
                                search: dataCon.searchChef
                            }
                        },
                        valProp: 'chefId',
                        modelProp: 'chef'
                    }
                },
                {
                    clientFormat: '.(meals)',
                    header: 'meals',
                    minWidth: 250,
                    inline: {
                        func: aweui.checkboxList,
                        opt: { multiselect: true, dataFunc: dataCon.getAllMeals },
                        valProp: 'mealsIds',
                        modelProp: 'meals'
                    }
                },
                {
                    bind: 'bonusMeal.name',
                    clientFormat: '.(bonusMeal)',
                    header: 'bonus meal',
                    inline: {
                        func: aweui.dropdownList,
                        opt: { dataFunc: dataCon.getAllMeals },
                        valProp: 'bonusMealId'
                    }
                },
                {
                    bind: 'organic',
                    width: 90,
                    clientFormat: '.(dispOrganic)',
                    inline: {
                        func: aweui.chk,
                        opt: { otoggl: true }
                    }
                },
                { clientFormat: gridUtils.inlineEditFormat(), width: 100 },
                { clientFormat: gridUtils.inlineDeleteFormatForGrid('inlineEditGrid1'), width: 100 }
            ]
        };

        return (
            <div className="example">
                <h2>Grid inline editing</h2>

                <div className="bar">
                    <input type="text" placeholder="search..." id="inlEditSearch" onKeyUp={this.inlEditSearchKeyUp} />
                    <div className="fr">
                        <button type="button" onClick={this.dinnerInlineCreate} className="awe-btn">
                            Create
              </button>
                    </div>
                </div>
                <AweGrid opt={inlineEditGrid1Opt} syncg="dinner" />
            </div>
        );
    }
}