import {awedb} from './awedb.js';
import { awef, awem, aweUtils, aweui } from '../aweui/all.js';
import {aweq} from '../infra/aweq';
import {awevld, modelState} from '../infra/awevld';
import {dataCon} from './dataCon';
import {formBuilder} from '../infra/formBuilder';

var dinnersCrudCon = function () {
  var where = aweq.where;
  var contains = aweq.contains;
  var loop = awef.loop;

    function mapToEnt(din, p) {
        din.name = p.name;
        din.date = awem.parseToDate(awem.dateFormat, p.date);
        din.chef = aweq.getById(awedb.chefs, p.chef);
        din.meals = aweq.getByIds(awedb.meals, p.meals);
        din.bonusMeal = aweq.getById(awedb.meals, p.bonusMeal);
        din.organic = p.organic === 'true';
        return din;
    }

    function validate(sp) {
        var din = aweUtils.getParams(sp);
        var ms = modelState(din);
        var v = awevld(ms);

        v.req('name');
        v.req('date');
        v.req('chef');
        v.req('meals');
        v.req('bonusMeal', 'bonus meal');

        return ms;
    }

    function buildForm(vm, sp, o) {
        var b = formBuilder({ model: vm, sp: sp });

        b.add({ prop: 'id', hidden: true });
        b.add({ prop: 'name', func: aweui.txt });
        b.add({ prop: 'date', func: aweui.datepicker });
        b.add({
            prop: 'chef',
            func: aweui.lookup,
            opt: {
                dataFunc: {
                    get: dataCon.getChef,
                    search: dataCon.searchChef
                }
            }
        });

        b.add({
            prop: 'meals',
            func: aweui.checkboxList,
            opt: { multiselect: true, dataFunc: dataCon.getAllMeals },
            //inpcls: 'multif'
        });

        b.add({
            prop: 'organic', func: aweui.chk,
            opt: { otoggl: true }
        });

        b.add({
            prop: 'bonusMeal', func: aweui.dropdownList,
            opt: { dataFunc: dataCon.getAllMeals }
        });

        b.applyToPopup(o);

        aweUtils.initPopupInstVld(o, validate);
    }

    function map(o) {
        return {
            id: o.id,
            name: o.name,
            date: awem.dateToStr( o.date),
            chefName: o.chef.firstName + ' ' + o.chef.lastName,
            meals: o.meals ? o.meals.map(function (m) { return m.name }).join(', ') : null,
            bonusMeal: o.bonusMeal.name,
            organic: o.organic ? 'Yes' : 'No'
        };
    }

    return {
        grid: function(sgp) {
            var gp = aweUtils.getGridParams(sgp);
            var list = awedb.dinners;

            if (gp.search) {
              var words = gp.search.toLowerCase().split(" ");

              list = where(list, function (o) {
                var matches = 0;
                loop(words, function (w) {
                  if (contains(o.name, w) ||
                    contains(awem.dateToStr( o.date), w) ||
                    contains(o.chef.firstName, w) ||
                    contains(o.chef.lastName, w) ||
                    contains(o.country.name, w) ||
                    contains(o.bonusMeal.name, w)) {
                    matches++;
                  }
                });

                return matches === words.length;
              });
            }

            var res = aweUtils.gridModelBuilder(
                {
                    gp: gp,
                    items: list,
                    key: "id",
                    map: map,
                    getItem: function () {
                        return aweq.getById(awedb.dinners, gp.key);
                    }
                });

            return res;
        },
        create: function(sp, o) {
            buildForm({}, sp, o);
        },
        createPost: function(sp) {
            var vm = aweUtils.getParams(sp);
            var ms = validate(sp);
            
            if (!ms.isValid()) {
              return aweui.getErrors(ms.errors);
            }

            var din = mapToEnt({}, vm);
            awedb.add(awedb.dinners, din);

            return map(din);
        },
        edit: function(sp, o) {
            var prm = aweUtils.getParams(sp);

            var d = aweq.getById(awedb.dinners, prm.id);
            var vm = {
                id: d.id,
                name: d.name,
                date: d.date,
                chef: d.chef.id,
                meals: d.meals ? d.meals.map(function (m) { return m.id }) : null,
                bonusMeal: d.bonusMeal.id,
                organic: d.organic
            };

            buildForm(vm, sp, o);
        },
        editPost: function(sp) {
            var vm = aweUtils.getParams(sp);
            var ms = validate(sp);
            
            if (!ms.isValid()) {
                return aweui.getErrors(ms.errors);
            }

            var din = aweq.getById(awedb.dinners, vm.id);

            mapToEnt(din, vm);

            return { id: din.id };
        }
    };
}();

export {dinnersCrudCon};