import React from 'react';
import ReactDOM from "react-dom/client";
import * as serviceWorker from './serviceWorker';
import Aweinp from './awecomp/Aweinp';
import AweGrid from './awecomp/AweGrid';
import { dataCon } from './cons/dataCon.js';
import { site } from './site';
import * as $ from 'jquery';
import { awe, awem, aweUtils, aweui } from './aweui/all.js';
import Header from './components/Header';
import FormExample from './components/FormExample';
import DemoMenu from './components/DemoMenu';
import GridPopupCrud from './components/GridPopupCrud';
import GridInlineEdit from './components/GridInlineEdit';
import PopupCont from './components/PopupCont';


class Index extends React.Component {
    constructor(props) {
        super(props);

        // init aweui
        awem.isMobileOrTablet = function () { return 0; };
        awem.fdw = 0; // first day of week
        aweUtils.init('MM/dd/yyyy', false, '.');
        aweui.init();

        // popup demo
        aweui.initPopup({
            id: 'popup1',
            setCont: function (sp, o) {
                var msg = $('<div>hi</div>');
                o.scon.html(msg);
                awe.flash(msg);

                // we could also return a promise ( e.g. $.ajax ) that will set o.scon.html after
            },
            //url: 'someurl', // url that returns the popup content
            loadOnce: true
        });
    }

    componentDidMount() {
        // needed so we can call awe.open from outside react
        window.awe = awe;

        // init demo
        site.documentReady();

        // drag and drop cards and items demo
        awem.dragReor({ from: $('#board1 .card'), to: '#board1 .card', sel: '.item' });

        // don't drag board when mouse down on .item
        function cancel(cx) { return $(cx.e.target).closest('.item').length; }

        awem.dragReor({ from: $('#board1'), to: '#board1', sel: '.card', cancel: cancel });
    }

    searchGrid1KeyUp() {
        $('#GridFrow').data('api').load();
    }

    openPopup1() {
        aweui.open('popup1');
    }

    render() {
        const mealOddOpt = {
            func: aweui.dropdownList,
            id: 'mealOdd',
            dataFunc: dataCon.getAllMeals,
            autoSelectFirst: true,
            itemFunc: site.imgItem,
            captionFunc: aweUtils.imgCaption
        };

        const mealComboOpt = {
            func: aweui.combobox,
            id: 'mealCombo',
            dataFunc: dataCon.getAllMeals,            
            itemFunc: site.imgItem,
            captionFunc: aweUtils.imgCaption            
        };

        const mealOddRsOpt = {
            func: aweui.dropdownList,
            id: 'mealOddRs',
            load: false,            
            searchFunc: function (o, info) {
                return dataCon.getMealsAutocomplete(info.term);
                // you can also return a promise (e.g. $.ajax)
            },
            itemFunc: site.imgItem,
            captionFunc: aweUtils.imgCaption            
        };

        const autoc1Opt = {
            func: aweui.autocomplete,
            id: 'autoc1',
            itemFunc: site.imgItem,
            dataFunc: dataCon.getMealsAutocomplete
            //url: '...' // can be set instead of dataFunc
        };

        const dtp1Opt = {
            func: aweui.datepicker,
            id: 'dtp1'
        };

        const txt1Opt = {
            func: aweui.txt,
            id: 'txt1',
            numeric: true
        };

        const categoriesRlOpt = {
            func: aweui.radioList,
            id: 'categoriesRl',
            dataFunc: dataCon.getCategories            
        };

        const mealsChklOpt = {
            func: aweui.checkboxList,
            id: 'mealsChkl',
            dataFunc: dataCon.getMeals,            
            parents: [{ name: 'categories', id: 'categoriesRl' }]
        };

        const mealsMsOpt = {
            func: aweui.multiselect,
            id: 'mealsMs',
            dataFunc: dataCon.getAllMeals,            
            itemFunc: site.imgItem,
            captionFunc: aweUtils.imgCaption
            //noSelClose: true
            //reorder: true // enbaled by default            
        };

        const lookupMeal1Opt = {
            func: aweui.lookup,
            id: 'lookupMeal1',
            dataFunc: {
                get: dataCon.getChef,
                search: dataCon.searchChef
            }
        };

        const mealsMulti2Opt = {
            func: aweui.multilookup,
            id: 'mealsMulti2',
            dataFunc: {
                get: dataCon.getChefs,
                search: dataCon.searchChef,
                selected: dataCon.selectedChefs
            }
        };

        const chk0Opt = {
            func: aweui.chk,
            id: 'chk0'
        };        

        const toggl1Opt = {
            func: aweui.toggle,
            id: 'toggl1'
        };

        const filterGridOpt = {
            id: 'GridFrow',
            dataFunc: dataCon.getLunchesFilterGrid,
            height: 390,
            reorderable: true,
            columns: [
                {
                    bind: 'Id', width: 75, groupable: false,
                    mod: {
                        filter: {
                            html: function (cx) { return 'search:' }
                        }
                    }
                },
                {
                    bind: 'Person',
                    mod: {
                        filter: {
                            func: aweui.txt,
                            opt: {
                                clearBtn: true
                            }
                        }
                    }
                },
                {
                    bind: 'Food',
                    clientFormatFunc: site.imgFood,
                    minWidth: 200,
                    mod: {
                        filter: function (cx) {
                            return {
                                func: aweui.dropdownList,
                                opt: {                                    
                                    clearBtn: true,
                                    itemFunc: site.imgFoodItem,
                                    captionFunc: site.imgFoodCaption,                                    
                                    dataFunc: dataCon.foodGetData
                                }
                            };
                        }
                    }
                },
                { bind: 'CountryName', header: 'Country' },
                {
                    bind: 'Date', // sort by "Date" prop
                    clientFormat: '.(DateStr)', // but display DateStr
                    width: 170,
                    mod: {
                        nohide: true
                    }
                },
                {
                    clientFormat: '.(MealsStr)',
                    header: 'Meals',
                    grow: 2,
                    mod: {
                        filter: {
                            type: 'multiselect',
                            valProp: 'Meals'
                        }
                    }
                },
                { bind: "ChefName", header: "Chef" }
            ],
            parents: [
                { id: 'txtGridFrowSrc', name: 'search', load: false }
            ],
            mod: {
                pageSize: true,
                columnsSelector: true,
                pageInfo: true,
                loading: true,
                columnsAutohide: true,
                filterRow: { keyupsel: '.awe-txt' }
            }
        };

        return (
            <div className="page">
                <Header />
                <div id="main">
                    <aside>
                        <div id="menuPlaceh"></div>
                        <DemoMenu />
                    </aside>
                    <div id="demoPage">
                        <div id="maincont">
                            <h1>aweui - awesome javascript ui components, React demo</h1>
                            <div className="example cbl">
                                <h2>Odropdown</h2>
                                <Aweinp opt={mealOddOpt} />
                                <h2>Combobox</h2>
                                <Aweinp opt={mealComboOpt} />
                                <h2>Remote search</h2>
                                <Aweinp opt={mealOddRsOpt} />
                            </div>

                            <div className="example cbl">
                                <h2>Autocomplete</h2>
                                <Aweinp opt={autoc1Opt} />

                                <h2>Datepicker</h2>
                                <Aweinp opt={dtp1Opt} />

                                <h2>Textbox</h2>
                                <Aweinp opt={txt1Opt} value="123" />
                            </div>

                            <div className="example cbl">
                                <h2>Cascading RadioList and CheckboxList</h2>
                                <div className="ib arl">
                                    <Aweinp opt={categoriesRlOpt} value="1014" />
                                </div>
                                <div className="ib arl">
                                    <Aweinp opt={mealsChklOpt} />
                                </div>
                            </div>

                            <div className="example">
                                <h2>Multiselect</h2>
                                <div className="mw33">
                                    <Aweinp opt={mealsMsOpt} value="[1010,1016,1022]" />
                                </div>
                            </div>

                            <div className="example cbl">
                                <h2>Lookup</h2>
                                <Aweinp opt={lookupMeal1Opt} />
                            </div>

                            <div className="example cbl">
                                <h2>MultiLookup</h2>
                                <Aweinp opt={mealsMulti2Opt} />
                            </div>

                            <div className="example">
                                <h2>Checkbox</h2>
                                <label>Checkbox: <Aweinp opt={chk0Opt} value="true" /></label>                                
                                <label>otoggl: <Aweinp opt={toggl1Opt} value="true" /></label>
                            </div>

                            <h2>Popup</h2>
                            <div className="cbl">
                                <button type="button" className="awe-btn" onClick={this.openPopup1}>open popup</button>
                            </div>

                            <div className="example">
                                <h2>Grid with filter row</h2>
                                <div className="bar">
                                    <input type="text" placeholder="search..." id="txtGridFrowSrc" onKeyUp={this.searchGrid1KeyUp} />
                                </div>
                                <AweGrid opt={filterGridOpt}></AweGrid>
                            </div>

                            <GridInlineEdit />

                            <GridPopupCrud />

                            <div className="example">
                                <h2>Drag and Drop Cards and Items</h2>
                                <div className="board" id="board1">
                                    <div className="card" data-k="a">
                                        <div className="item">item 1</div>
                                        <div className="item">item 2</div>
                                        <div className="item">item 3</div>
                                    </div>

                                    <div className="card" data-k="b">
                                        <div className="item">item 4</div>
                                        <div className="item">item 5</div>
                                    </div>

                                    <div className="card" data-k="c">
                                        <div className="item">item 6</div>
                                        <div className="item">item 7</div>
                                    </div>
                                </div>
                            </div>
                            <br />
                            <br />
                            <FormExample />
                            <PopupCont />
                        </div>
                    </div>
                </div>
            </div >
        );
    }
}

// ========================================

const root = ReactDOM.createRoot(document.getElementById("root"));
root.render(
    <React.StrictMode>
        <Index />
    </React.StrictMode>
);

//ReactDOM.render(
//    <Index />,
//    document.getElementById('root')
//);

// If you want your app to work offline and load faster, you can change
// unregister() to register() below. Note this comes with some pitfalls.
// Learn more about service workers: https://bit.ly/CRA-PWA
serviceWorker.unregister();