import { createApp } from 'vue/dist/vue.esm-bundler';
import App from './App.vue'
import * as $ from 'jquery'
import { aweui } from './aweui/all.js'
import { awevue } from './infra/awevue.js'
import './content/themes/wui/AwesomeMvc.css'
import './content/site.css'

awevue.init();

const app = createApp(App);

app.component('Aweinp', {
    props: ['opt', 'value'],
    template: `<input :id="opt.id" :type="type" :value="value" />`,
    mounted: function () {
        var self = this;
        var el = $(this.$el);

        if (self.opt) {
            var func = self.opt.func;
            if (func) {
                func(self.opt);
            }
        }

        el.on('change', function (event, data) {
            // ignore change when we triggered it from "updated"
            if (data == null || data.ign !== true) {
                self.$emit('input', el.val());
            }
        });

        el.on('aweload', function () {
            self.$emit('aweload');
            //console.log('aweload for ', this.opt.id);      
        });
    },
    updated: function () {
        $(this.$el).data('api').render();

        // trigger change in case this component is used as a Parent
        $(this.$el).trigger('change', { ign: true });
    },
    computed: {
        // a computed getter
        type(){
            // `this` points to the vm instance
            var func = null;
            if (this.opt) {
                func = this.opt.func;
            }

            return func === aweui.datepicker || func === aweui.autocomplete ? 'text' : 'hidden';
        }
    }
});

app.component('AweGrid', {
    props: ['opt', 'syncg'],
    template: `<div :id="opt.id" :data-syncg="syncg" ></div>`,
    mounted: function () {
        var self = this;
        var el = $(this.$el);

        if (self.opt) {
            aweui.grid(self.opt);
        }

        el.on('aweload', function () {
            self.$emit('aweload');
        });
    }
});

app.component('Editor', {
    props: ['label'],
    template: '<div class="efield"><div class="elabel"><label>{{label}}</label></div><div class="einput"><slot></slot></div></div>'
});

app.mount('#app');